package cn.iocoder.yudao.module.customer.dal.mysql.customerCreditLog;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.mapper.AbstractMapper;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerCreditLog.CustomerCreditLogDO;
import cn.iocoder.yudao.module.customer.vo.customerCreditLog.CustomerCreditLogBackVO;
import cn.iocoder.yudao.module.customer.vo.customerCreditLog.CustomerCreditLogQueryVO;
import cn.iocoder.yudao.module.customer.vo.customerDetail.CustomerCreditRuleStatisticBackVo;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.ResultType;
import org.apache.ibatis.annotations.Select;

import java.math.BigDecimal;
import java.util.List;

/**
 * 客户信用日志 Mapper
 *
 * @author 系统管理员
 */
@Mapper
public interface CustomerCreditLogMapper extends AbstractMapper<CustomerCreditLogDO> {
    @Override
    default PageResult<CustomerCreditLogDO> selectPage(PageVO page, Object object) {
        if (object instanceof CustomerCreditLogQueryVO) {
            CustomerCreditLogQueryVO vo = (CustomerCreditLogQueryVO) object;
            return selectPage(page, new LambdaQuery<CustomerCreditLogDO>()
                    .betweenIfPresent(CustomerCreditLogDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                    .eqIfPresent(CustomerCreditLogDO::getCustomerId, vo.getCustomerId())
                    .eqIfPresent(CustomerCreditLogDO::getRuleId, vo.getRuleId())
                    .eqIfPresent(CustomerCreditLogDO::getRemark, vo.getRemark())
                    .orderByDesc(CustomerCreditLogDO::getId));
        }

        return null;
    }

    @Override
    default List<CustomerCreditLogDO> selectList(Object object) {
        if (object instanceof CustomerCreditLogQueryVO) {
            CustomerCreditLogQueryVO vo = (CustomerCreditLogQueryVO) object;
            return selectList(new LambdaQuery<CustomerCreditLogDO>()
                    .betweenIfPresent(CustomerCreditLogDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                    .eqIfPresent(CustomerCreditLogDO::getCustomerId, vo.getCustomerId())
                    .eqIfPresent(CustomerCreditLogDO::getRuleId, vo.getRuleId())
                    .eqIfPresent(CustomerCreditLogDO::getRemark, vo.getRemark())
                    .orderByDesc(CustomerCreditLogDO::getId));
        }

        return null;
    }


    @ResultType(CustomerCreditLogBackVO.class)
    @Select({
            "<script>",
            "select log.*, ",
            "rule.name, rule.name_en,",
            "rule.type, log.rule_score as score from ecw_customer_credit_log log ",
            "inner join ecw_customer_credit_rule rule on rule.id = log.rule_id ",
            "where log.deleted = 0 and rule.deleted = 0 ",
            "<when test = 'query.customerId != null'>",
            "AND log.`customer_id` = #{query.customerId}",
            "</when>",
            "order by id desc",
            "</script>"
    })
    IPage<CustomerCreditLogBackVO> selectPage(@Param("mPage") IPage<CustomerCreditLogBackVO> mpPage, @Param("query") CustomerCreditLogQueryVO query);


    @ResultType(CustomerCreditRuleStatisticBackVo.class)
    @Select({
            "<script>",
            "select  rule.`type`  as type, sum(log.rule_score) as score from ecw_customer_credit_log log ",
            "inner join ecw_customer_credit_rule rule on rule.id = log.rule_id ",
            "where log.deleted = 0 and rule.deleted = 0 ",
            "and log.`customer_id` = #{customerId} group by rule.type ",
            "</script>"
    })
    List<CustomerCreditRuleStatisticBackVo> getCreditRuleStatistic(@Param("customerId") Long customerId);


    @Select({
            "<script>",
            "select   sum(log.rule_score) as score from ecw_customer_credit_log log ",
            "inner join ecw_customer_credit_rule rule on rule.id = log.rule_id ",
            "where log.deleted = 0 and rule.deleted = 0 ",
            "and log.`customer_id` = #{customerId}  ",
            "</script>"
    })
    BigDecimal getCreditRuleStatisticAll(@Param("customerId") Long customerId);

    @ResultType(CustomerCreditLogBackVO.class)
    @Select({
            "<script>",
            "select log.*, ",
                    "rule.name, rule.name_en,",
                    "rule.type, log.rule_score as score, ",
                    "(select nickname from `system_user` where id=log.creator) as create_name ",
            "from ecw_customer_credit_log log ",
            "inner join ecw_customer_credit_rule rule on rule.id = log.rule_id ",
            "where log.deleted = 0 and rule.deleted = 0 ",
            "AND log.`customer_id` = #{customerId}",
            "order by id desc",
            "</script>"
    })
    IPage<CustomerCreditLogBackVO> selectPageByCustomerId(@Param("mPage") IPage<CustomerCreditLogBackVO> mpPage,
                                                          @Param("customerId") Long customerId);

}
