package cn.iocoder.yudao.module.customer.dto;

import cn.iocoder.yudao.framework.common.pojo.PageParam;
import com.fasterxml.jackson.annotation.JsonIgnore;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.ToString;
import org.springframework.format.annotation.DateTimeFormat;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND;

@ApiModel("管理后台 - 客户分页 Request DTO")
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
public class CustomerPageReqDTO extends PageParam {

    @ApiModelProperty(value = "客户编号")
    private String number;

    @ApiModelProperty(value = "客户名称")
    private String name;

    @ApiModelProperty(value = "客户等级")
    private List<Integer> level;

    @ApiModelProperty(value = "客户来源")
    private List<Integer> source;

    @ApiModelProperty(value = "客户类别")
    private List<String> type;

    @ApiModelProperty(value = "出货渠道", notes = "参见customer_transport_type数据字典")
    private List<String> transportType;

    @ApiModelProperty(value = "客户资源类型", notes = "参见customer_resource_type数据字典")
    private List<Integer> resourceType;

    @ApiModelProperty(value = "市场获客", notes = "参见customer_market_type数据字典")
    private List<Integer> marketType;


    @ApiModelProperty(value = "跟进客服")
    private List<Long> customerService;

    @ApiModelProperty(value = "客户状态")
    private List<Integer> status;

    // @ApiModelProperty(value = "部门")
    // private String department;

    @ApiModelProperty(value = "部门")
    private List<Long> deptIds;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "开始创建时间")
    private Date beginCreateTime;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "结束创建时间")
    private Date endCreateTime;



    @ApiModelProperty(value = "主联系人")
    private String defaultContactName;

    @ApiModelProperty(value = "主联系电话")
    private String defaultContactPhone;



    @ApiModelProperty(value = "信用等级")
    private List<Integer> creditLevel;

    @ApiModelProperty(value = "国家")
    private List<Integer> country;

    //api不显示此3个字段
    //是否接收，后台使用
    @JsonIgnore
    private Boolean isCustomerServiceConfirmed;

    //因接口公用方法新增：24小时内数据查询条件标识
    private String flag4QueryCondition;

    @ApiModelProperty(value = "是否在公海")
    private Boolean isInOpenSea;

    @ApiModelProperty(value = "是否潜在客户")
    private Boolean isPotential;

    @ApiModelProperty(value = "会员编号")
    private String memberCode;
    @ApiModelProperty(value = "会员名称")
    private String memberName;
    @ApiModelProperty(value = "会员手机")
    private String memberMobile;
    @ApiModelProperty(value = "会员区号")
    private String memberAreaCode;




    // 20240904 add start
    @ApiModelProperty(value = "客户角色")
    private List<String> role;


    /**
     * 业务国家 逗号分隔
     */
    @ApiModelProperty(value = "业务国家")
    private List<Long> busiCountryIds;


    @ApiModelProperty(value = "常用提货网点")
    private List<Long> pickupPoints;


    @ApiModelProperty(value = "创建人")
    private Long founder;


    // 开始入公海时间和结束入公海时间
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "开始入公海时间")
    private Date beginEnterOpenSeaTime;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "结束入公海时间")
    private Date endEnterOpenSeaTime;

    @ApiModelProperty(value = "业绩类型（老客户：0 新客户：1）")
    private Boolean isNew;


    // 大于等于年度发货量
    @ApiModelProperty(value = "大于等于年度发货量")
    private BigDecimal geWeightYearly;
    // 小于等于年度发货量
    @ApiModelProperty(value = "小于等于年度发货量")
    private BigDecimal leWeightYearly;
    // 等于年度发货量
    @ApiModelProperty(value = "等于年度发货量")
    private BigDecimal eqWeightYearly;


    // 大于等于年度发货次数
    @ApiModelProperty(value = "大于等于年度发货次数")
    private Integer geNumYearly;
    // 小于等于年度发货次数
    @ApiModelProperty(value = "小于等于年度发货次数")
    private Integer leNumYearly;
    // 等于年度发货次数
    @ApiModelProperty(value = "等于年度发货次数")
    private Integer eqNumYearly;


    @ApiModelProperty(value = "主营商品类型")
    private List<Long> productTypes;
    @ApiModelProperty(value = "主营商品名称")
    private List<Long> productIds;
    @ApiModelProperty(value = "主要竞争对手")
    private List<Long> competitorIds;


    @ApiModelProperty(value = "推介人")
    private Long promoter;

    @ApiModelProperty(value = "到仓确认")
    private Integer arrivalConfirm;

    @ApiModelProperty(value = "控货无收货人,1-是，0-否")
    private Boolean noConsignee;
    @ApiModelProperty(value = "默认付款")
    private Boolean defaultPay;
    @ApiModelProperty(value = "是否显示提单价格")
    private Boolean isShowTidanPrice;

    //归属时间
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "开始归属时间")
    private Date beginCustomerServiceConfirmedTime;
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "结束归属时间")
    private Date endCustomerServiceConfirmedTime;

    // 首次成交时间
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "开始首次成交时间")
    private Date beginFirstDealTime;
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "结束首次成交时间")
    private Date endFirstDealTime;


    @ApiModelProperty(value = "获取方式")
    private List<String> getMethods;
    //创建入口
    @ApiModelProperty(value = "创建入口")
    private List<String> createFroms;
    // 结算方式
    @ApiModelProperty(value = "结算方式")
    private List<String> balances;

    // 20240904 add end

}
