package cn.iocoder.yudao.module.customer.service;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.CustomerDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerContacts.CustomerContactsDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.syncAirCustomer.SyncAirCustomerDO;
import cn.iocoder.yudao.module.customer.dal.mysql.syncAirCustomer.SyncAirCustomerMapper;
import cn.iocoder.yudao.module.customer.service.customer.CustomerService;
import cn.iocoder.yudao.module.customer.service.customerContacts.CustomerContactsService;
import cn.iocoder.yudao.module.customer.vo.customer.vo.UserVo;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.*;

@Service
public class SyncAirCustomerJob {

    @Resource
    CustomerService customerService;

    @Resource
    CustomerContactsService customerContactsService;

    @Resource
    SyncAirCustomerMapper syncAirCustomerMapper;

    /**
     * 同步全量空运客户
     */
    public void syncAllAirCustomer() {
        String resultStr = HttpUtil.get("http://air.groupage.cn/api/getUsers.php?act=all");
        if (StrUtil.isBlank(resultStr)) {
            return;
        }
        JSONObject resultObj = JSONUtil.parseObj(resultStr);
        if (resultObj.getInt("code") != 200) {
            return;
        }
        JSONArray jsonArray = resultObj.getJSONArray("list");
        if (jsonArray.isEmpty()) {
            return;
        }
        for (int i = 0; i < jsonArray.size(); i++) {
            JSONObject result = jsonArray.getJSONObject(i);


            SyncAirCustomerDO newDo = new SyncAirCustomerDO();
            newDo.setName(result.getStr("name"));
            newDo.setPhone(result.getStr("Phone"));
            newDo.setCustomerName(result.getStr("customerName"));
            syncAirCustomerMapper.insert(newDo);

        }

    }

    /**
     * 同步今天的空运客户
     */
    public void syncTodayAirCustomer() {
        String resultStr = HttpUtil.get("http://air.groupage.cn/api/getUsers.php?act=today");
        if (StrUtil.isBlank(resultStr)) {
            return;
        }
        JSONObject resultObj = JSONUtil.parseObj(resultStr);
        if (resultObj.getInt("code") != 200) {
            return;
        }
        JSONArray jsonArray = resultObj.getJSONArray("list");
        if (jsonArray.isEmpty()) {
            return;
        }
        for (int i = 0; i < jsonArray.size(); i++) {
            JSONObject result = jsonArray.getJSONObject(i);

            SyncAirCustomerDO newDo = new SyncAirCustomerDO();
            newDo.setName(result.getStr("name"));
            newDo.setPhone(result.getStr("phone"));
            newDo.setCustomerName(result.getStr("customerName"));

            syncAirCustomerMapper.insert(newDo);
        }
    }

    /**
     * 处理今天同步过来的空运客户
     */
    public void handlerTodayAirCustomer() {
        List<UserVo> userMapList = syncAirCustomerMapper.getUserMapList();
        Map<String, Long> userMap = build(userMapList);
        final String airTransportType = "3";
        LambdaQueryWrapper<SyncAirCustomerDO> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.apply("date_format(create_time,'%Y-%m-%d')={0}", DateUtil.format(LocalDateTime.now(), DatePattern.NORM_DATE_PATTERN));
        List<SyncAirCustomerDO> list = syncAirCustomerMapper.selectList(queryWrapper);
        for (SyncAirCustomerDO syncAirCustomerDO : list) {
            if (StrUtil.isNotBlank(syncAirCustomerDO.getPhone()) && StrUtil.isNotBlank(syncAirCustomerDO.getCustomerName())) {
                String newPhone = syncAirCustomerDO.getPhone().replaceAll("/+", "");
                //根据电话查询当前用户是否在系统中存在
                List<CustomerContactsDO> customerConstantList = syncAirCustomerMapper.getAllCustomerContactByPhone(newPhone);
                if (null == customerConstantList || customerConstantList.size() == 0) {
                    //------------------------------------------代表用户不存在-----------------------------------------
                    CustomerDO newCustomerDo = new CustomerDO();
                    newCustomerDo.setCustomerService(getServiceIdByCustomerName(userMap, syncAirCustomerDO.getCustomerName()));

                    if (null != newCustomerDo.getCustomerService()) {
                        // 分配时间
                        newCustomerDo.setCustomerServiceAssignedTime(new Date());
                        newCustomerDo.setIsCustomerServiceConfirmed(false);
                        newCustomerDo.setIsInOpenSea(false);
                    }
                    newCustomerDo.setName(syncAirCustomerDO.getName());
                    newCustomerDo.setTransportType(airTransportType);
                    newCustomerDo.setStatus(3);
                    newCustomerDo.setIsAuto(0);
                    //newCustomerDo.setType(airTransportType);
                    //newCustomerDo.setResourceType(1);
                    customerService.save(newCustomerDo);

                    CustomerContactsDO newCustomerContactsDO = new CustomerContactsDO();
                    newCustomerContactsDO.setCustomerId(newCustomerDo.getId());
                    newCustomerContactsDO.setPhoneNew(StrUtil.subPre(newPhone, 2));
                    newCustomerContactsDO.setPhoneNew(StrUtil.subSuf(newPhone, 2));
                    newCustomerContactsDO.setUsername(syncAirCustomerDO.getName());
                    newCustomerContactsDO.setIsDefault(1);
                    customerContactsService.save(newCustomerContactsDO);

                } else {
                    //------------------------------------------代表用户已存在-----------------------------------------
                    CustomerContactsDO customerContactsDO = customerConstantList.get(0);
                    if (null != customerContactsDO.getCustomerId()) {
                        //得到当前用户
                        CustomerDO customerDO = customerService.getById(customerContactsDO.getCustomerId());
                        if (null != customerDO) {
                            //给当前用户追加空运
                            List<String> transportTypeList = new ArrayList<>();
                            if (StrUtil.isNotBlank(customerDO.getTransportType())) {
                                String[] arr = customerDO.getTransportType().split(",");
                                transportTypeList.addAll(Arrays.asList(arr));
                                if (!transportTypeList.contains(airTransportType)) {
                                    transportTypeList.add(airTransportType);
                                }
                            } else {
                                transportTypeList.add(airTransportType);
                            }
                            CustomerDO update = new CustomerDO();
                            update.setId(customerDO.getId());
                            update.setCustomerService(getServiceIdByCustomerName(userMap, syncAirCustomerDO.getCustomerName()));
                            update.setTransportType(StringUtils.join(transportTypeList, ","));
                            customerService.updateById(update);
                        }
                    }
                }
            }
        }
    }

    private Map<String, Long> build(List list) {
        Map<String, Long> returnMap = new HashMap<>();
        for (int i = 0; i < list.size(); i++) {
            UserVo map = (UserVo) list.get(i);
            returnMap.put(map.getUsername(), map.getId());
        }
        return returnMap;
    }

    private Long getServiceIdByCustomerName(Map<String, Long> map, String customerName) {
        if (customerName.indexOf("001") != -1) {
            return map.get("001");
        } else if (customerName.indexOf("002") != -1) {
            return map.get("002");
        } else if (customerName.indexOf("003") != -1) {
            return map.get("003");
        } else if (customerName.indexOf("004") != -1) {
            return map.get("004");
        } else if (customerName.indexOf("005") != -1) {
            return map.get("005");
        } else if (customerName.indexOf("006") != -1) {
            return map.get("006");
        } else if (customerName.indexOf("007") != -1) {
            return map.get("007");
        } else if (customerName.indexOf("008") != -1) {
            return map.get("008");
        } else if (customerName.indexOf("009") != -1) {
            return map.get("009");
        } else {
            return null;
        }
    }

    public static void main(String[] args) {
        String phone = "8618681548901";
        System.out.println(StrUtil.subPre(phone, 2));
        System.out.println(StrUtil.subSuf(phone, 2));
    }
}
