package cn.iocoder.yudao.module.customer.service.customer.commission;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.commission.CustomerCommissionInfoDO;
import cn.iocoder.yudao.module.customer.dto.CustomerCommissionDto;
import cn.iocoder.yudao.module.customer.dto.CustomerCommissionQueryDto;
import cn.iocoder.yudao.module.customer.vo.customer.commission.CustomerCommissionInfoCreateReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.commission.CustomerCommissionInfoExportReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.commission.CustomerCommissionInfoPageReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.commission.CustomerCommissionInfoUpdateReqVO;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 客户佣金详情 Service 接口
 *
 * @author 系统管理员
 */
public interface CustomerCommissionInfoService extends IService<CustomerCommissionInfoDO> {

    /**
     * 创建客户佣金详情
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createCustomerCommissionInfo(@Valid CustomerCommissionInfoCreateReqVO createReqVO);

    /**
     * 更新客户佣金详情
     *
     * @param updateReqVO 更新信息
     */
    void updateCustomerCommissionInfo(@Valid CustomerCommissionInfoUpdateReqVO updateReqVO);

    /**
     * 删除客户佣金详情
     *
     * @param id 编号
     */
    void deleteCustomerCommissionInfo(Long id);

    /**
     * 获得客户佣金详情
     *
     * @param id 编号
     * @return 客户佣金详情
     */
    CustomerCommissionInfoDO getCustomerCommissionInfo(Long id);

    /**
     * 获得客户佣金详情
     *
     * @param commissionId commission编号
     * @return 客户佣金详情
     */
    CustomerCommissionInfoDO getInfoByCommissionId(Long commissionId);


    /**
     * 获取根据条件获取用户当前订单可用佣金信息
     *
     * @param customerCommissionQueryDto     参数
     * @return 客户佣金详情
     */
    CustomerCommissionDto customerCommission(CustomerCommissionQueryDto customerCommissionQueryDto);

    /**
     * 获得客户佣金详情列表
     *
     * @param ids 编号
     * @return 客户佣金详情列表
     */
    List<CustomerCommissionInfoDO> getCustomerCommissionInfoList(Collection<Long> ids);

    /**
     * 获得客户佣金详情分页
     *
     * @param pageReqVO 分页查询
     * @return 客户佣金详情分页
     */
    PageResult<CustomerCommissionInfoDO> getCustomerCommissionInfoPage(CustomerCommissionInfoPageReqVO pageReqVO);

    /**
     * 获得客户佣金详情列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 客户佣金详情列表
     */
    List<CustomerCommissionInfoDO> getCustomerCommissionInfoList(CustomerCommissionInfoExportReqVO exportReqVO);

}
