package cn.iocoder.yudao.module.customer.service.customer.commission;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.commission.CustomerCommissionDO;
import cn.iocoder.yudao.module.customer.vo.customer.commission.*;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.web.bind.annotation.RequestBody;

import javax.validation.Valid;
import java.util.List;

/**
 * 客户佣金类型 Service 接口
 *
 * @author 系统管理员
 */
public interface CustomerCommissionService extends IService<CustomerCommissionDO> {

    /**
     * 创建客户佣金类型
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createCustomerCommission(@Valid CustomerCommissionCreateReqVO createReqVO);

    /**
     * 更新客户佣金类型
     *
     * @param updateReqVO 更新信息
     */
    void updateCustomerCommission(@Valid CustomerCommissionUpdateReqVO updateReqVO);

    /**
     * 删除客户佣金类型
     *
     * @param id 编号
     */
    void deleteCustomerCommission(Long id);

    /**
     * 获得客户佣金类型
     *
     * @param id 编号
     * @return 客户佣金类型
     */
    CustomerCommissionDO getCustomerCommissionById(Long id);

    CustomerCommissionDO getCustomerCommissionByCustomerId(Long customerId);


    ///**
    // * 获得客户佣金类型列表
    // *
    // * @param ids 编号
    // * @return 客户佣金类型列表
    // */
    //List<CustomerCommissionDO> getCustomerCommissionList(Collection<Long> ids);

    /**
     * 获得客户佣金类型分页
     *
     * @param pageReqVO 分页查询
     * @return 客户佣金类型分页
     */
    PageResult<CustomerCommissionDO> getCustomerCommissionPage(CustomerCommissionPageReqVO pageReqVO);



    /**
     * 获得客户佣金类型列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 客户佣金类型列表
     */
    List<CustomerCommissionDO> getCustomerCommissionList(CustomerCommissionExportReqVO exportReqVO);

    /**
     * 根据具体条件查询暗佣
     *
     * @param reqVO
     * @return
     */
    CustomerDarkReturnCommissionQueryRespVO getDarkReturnCommission(@Valid @RequestBody CustomerDarkReturnCommissionQueryReqVO reqVO);

    CustomerCommissionDO selectOne(LambdaQueryWrapper<CustomerCommissionDO> wrapper);
}
