package cn.iocoder.yudao.module.customer.service.customer.commission;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.query.QueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.module.customer.convert.customer.commission.CustomerCommissionConvert;
import cn.iocoder.yudao.module.customer.convert.customer.commission.CustomerCommissionInfoConvert;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.commission.CustomerCommissionDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.commission.CustomerCommissionInfoDO;
import cn.iocoder.yudao.module.customer.dal.mysql.customer.commission.CustomerCommissionInfoMapper;
import cn.iocoder.yudao.module.customer.dal.mysql.customer.commission.CustomerCommissionMapper;
import cn.iocoder.yudao.module.customer.vo.customer.commission.*;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.CUSTOMER_COMMISSION_NOT_EXISTS;

/**
 * 客户佣金类型 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class CustomerCommissionServiceImpl extends AbstractService<CustomerCommissionMapper, CustomerCommissionDO> implements CustomerCommissionService {

    @Resource
    private CustomerCommissionMapper customerCommissionMapper;

    @Resource
    private CustomerCommissionInfoMapper customerCommissionInfoMapper;



    @Override
    public Long createCustomerCommission(CustomerCommissionCreateReqVO createReqVO) {
        // 插入
        CustomerCommissionDO customerCommission = CustomerCommissionConvert.INSTANCE.convert(createReqVO);
        customerCommissionMapper.insert(customerCommission);
        // 返回
        Long commissionId = customerCommission.getId();
        
        //插入详情
        List<CustomerCommissionInfoBaseVO> customerCommissionInfoBaseVOList = createReqVO.getCustomerCommissionInfoBaseVOList();
        if (!CollectionUtils.isEmpty(customerCommissionInfoBaseVOList)) {

            List<CustomerCommissionInfoDO> customerCommissionInfoDOList = customerCommissionInfoBaseVOList.stream().map(customerCommissionInfoBaseVO -> {
                CustomerCommissionInfoDO customerCommissionInfoDO = CustomerCommissionInfoConvert.INSTANCE.convert(customerCommissionInfoBaseVO);
                customerCommissionInfoDO.setCommissionId(commissionId);
                return customerCommissionInfoDO;
            }).collect(Collectors.toList());
            customerCommissionInfoMapper.insertBatch(customerCommissionInfoDOList);
        }

        return commissionId;
    }

    @Override
    public void updateCustomerCommission(CustomerCommissionUpdateReqVO updateReqVO) {
        // 校验存在
        Long commissionId = updateReqVO.getId();
        this.validateCustomerCommissionExists(commissionId);

        //
        CustomerCommissionDO updateObj = CustomerCommissionConvert.INSTANCE.convert(updateReqVO);
        customerCommissionMapper.updateById(updateObj);

        //update info
        List<CustomerCommissionInfoUpdateReqVO> customerCommissionInfoBaseVOList = updateReqVO.getCustomerCommissionInfoBaseVOList();
        if (!CollectionUtils.isEmpty(customerCommissionInfoBaseVOList)) {
            List<CustomerCommissionInfoDO> updateList = customerCommissionInfoBaseVOList.stream().filter(t -> t.getId() != null)
                    .map(customerCommissionInfoUpdateReqVO -> CustomerCommissionInfoConvert.INSTANCE.convert(customerCommissionInfoUpdateReqVO))
                    .collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(updateList)) {
                customerCommissionInfoMapper.updateBatch(updateList);
            }
            List<CustomerCommissionInfoDO> addList = customerCommissionInfoBaseVOList.stream().filter(t -> t.getId() == null)
                    .map(customerCommissionInfoUpdateReqVO -> CustomerCommissionInfoConvert.INSTANCE.convert(customerCommissionInfoUpdateReqVO))
                    .collect(Collectors.toList());
            if (!CollectionUtils.isEmpty(addList)) {
                customerCommissionInfoMapper.insertBatch(addList);
            }
        }
    }

    @Override
    public void deleteCustomerCommission(Long id) {
        // 校验存在
        this.validateCustomerCommissionExists(id);
        // 删除
        customerCommissionMapper.deleteById(id);

        //删除详情
        customerCommissionInfoMapper.deleteByCommissionId(id);
    }

    private void validateCustomerCommissionExists(Long id) {
        if (customerCommissionMapper.selectById(id) == null) {
            throw exception(CUSTOMER_COMMISSION_NOT_EXISTS);
        }
    }

    @Override
    public CustomerCommissionDO getCustomerCommissionById(Long id) {
        return customerCommissionMapper.selectById(id);
    }

    @Override
    public CustomerCommissionDO getCustomerCommissionByCustomerId(Long customerId) {
        CustomerCommissionExportReqVO reqVO = new CustomerCommissionExportReqVO();
        reqVO.setCustomerId(customerId);

        List<CustomerCommissionDO> customerCommissionList = getCustomerCommissionList(reqVO);
        if (!CollectionUtils.isEmpty(customerCommissionList)) {
            return customerCommissionList.get(0);
        }
        return null;
    }

    //@Override
    //public List<CustomerCommissionDO> getCustomerCommissionList(Collection<Long> ids) {
    //    return customerCommissionMapper.selectBatchIds(ids);
    //}

    @Override
    public PageResult<CustomerCommissionDO> getCustomerCommissionPage(CustomerCommissionPageReqVO reqVO) {
        IPage<CustomerCommissionDO> mpPage = MyBatisUtils.buildPage(reqVO);

        LambdaQueryWrapper<CustomerCommissionDO> customerCommissionDOLambdaQueryWrapperX = new LambdaQueryWrapperX<CustomerCommissionDO>()
                .eqIfPresent(CustomerCommissionDO::getCustomerId, reqVO.getCustomerId())
                .eqIfPresent(CustomerCommissionDO::getType, reqVO.getType())
                .eqIfPresent(CustomerCommissionDO::getTransportId, reqVO.getTransportId())
                .eqIfPresent(CustomerCommissionDO::getDepartureId, reqVO.getDepartureId())
                .eqIfPresent(CustomerCommissionDO::getObjectiveId, reqVO.getObjectiveId())
                .eqIfPresent(CustomerCommissionDO::getCustomsType, reqVO.getCustomsType())
                .eqIfPresent(CustomerCommissionDO::getApproval, reqVO.getApproval())
                .eqIfPresent(CustomerCommissionDO::getDarkReturnType, reqVO.getDarkReturnType())
                //主联系电话
                .apply(StringUtils.isNotEmpty(reqVO.getDefaultContactPhone()), "default_contact_phone like {0}", String.format("%%%s%%", reqVO.getDefaultContactPhone()))

                .eq(BaseDO::getDeleted, false)
                .orderByDesc(CustomerCommissionDO::getId);

        customerCommissionMapper.getPage(mpPage, customerCommissionDOLambdaQueryWrapperX);

        return new PageResult<>(mpPage.getRecords(), mpPage.getTotal());
    }

    @Override
    public List<CustomerCommissionDO> getCustomerCommissionList(CustomerCommissionExportReqVO reqVO) {

        LambdaQueryWrapper<CustomerCommissionDO> customerCommissionDOLambdaQueryWrapperX = new LambdaQueryWrapperX<CustomerCommissionDO>()
                .eqIfPresent(CustomerCommissionDO::getCustomerId, reqVO.getCustomerId())
                .eqIfPresent(CustomerCommissionDO::getType, reqVO.getType())
                .eqIfPresent(CustomerCommissionDO::getTransportId, reqVO.getTransportId())
                .eqIfPresent(CustomerCommissionDO::getDepartureId, reqVO.getDepartureId())
                .eqIfPresent(CustomerCommissionDO::getObjectiveId, reqVO.getObjectiveId())
                .eqIfPresent(CustomerCommissionDO::getCustomsType, reqVO.getCustomsType())
                .eqIfPresent(CustomerCommissionDO::getApproval, reqVO.getApproval())
                .eqIfPresent(CustomerCommissionDO::getDarkReturnType, reqVO.getDarkReturnType())
                //主联系电话
                .likeIfPresent(CustomerCommissionDO::getDefaultContactPhone, reqVO.getDefaultContactPhone())
                .eq(BaseDO::getDeleted, false)
                .orderByDesc(CustomerCommissionDO::getId);

        return customerCommissionMapper.getList(customerCommissionDOLambdaQueryWrapperX);
    }


    @Override
    public CustomerDarkReturnCommissionQueryRespVO getDarkReturnCommission(CustomerDarkReturnCommissionQueryReqVO reqVO) {

        QueryWrapperX queryWrapper = new QueryWrapperX();
        queryWrapper.eq("info.deleted", 0);
        queryWrapper.eq("com.deleted", 0);
        queryWrapper.eq("com.customer_id", reqVO.getCustomerId());
        queryWrapper.eqIfPresent("com.type", reqVO.getType());

        apply(queryWrapper, "com.transport_id", reqVO.getTransportId());

        apply(queryWrapper, "com.departure_id", reqVO.getDepartureId());
        apply(queryWrapper, "com.objective_id", reqVO.getObjectiveId());
        apply(queryWrapper, "com.customs_type", reqVO.getCustomsType());
        apply(queryWrapper, "com.approval", reqVO.getApproval());
        apply(queryWrapper, "com.dark_return_type", reqVO.getDarkReturnType());
        apply(queryWrapper, "com.objective_id", reqVO.getObjectiveId());
        apply(queryWrapper, "info.product_type", reqVO.getProductType());

        String productAttr = reqVO.getProductAttr().stream().map(String::valueOf).collect(Collectors.joining(","));
        queryWrapper.apply(CollectionUtil.isNotEmpty(reqVO.getProductAttr()), "(FIND_IN_SET (info.product_attr,  '"+productAttr+"') or info.product_attr = -1)");

        apply(queryWrapper, "info.dark_currency", reqVO.getDarkCurrency());
        apply(queryWrapper, "info.dark_unit", reqVO.getDarkUnit());

        List<CustomerDarkReturnCommissionQueryRespVO> darkReturnCommissionList = customerCommissionMapper.getDarkReturnCommission(queryWrapper);
        if (CollectionUtil.isNotEmpty(darkReturnCommissionList)) {
            CustomerDarkReturnCommissionQueryRespVO result = darkReturnCommissionList.stream().sorted((o1, o2) -> o2.getDarkCommission().compareTo(o1.getDarkCommission())).findFirst().get();
            return result;
        }
        return null;
    }

    private void apply(QueryWrapperX queryWrapper, String column, Object value) {
        queryWrapper.apply(value != null, "(" + column + " = " + value + " or " + column + " = -1)");
    }

    @Override
    public CustomerCommissionDO selectOne(LambdaQueryWrapper<CustomerCommissionDO> wrapper) {
        return customerCommissionMapper.selectOne(wrapper);
    }
}
