package cn.iocoder.yudao.module.customer.service.customer.complaint;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.complaint.CustomerComplaintDO;
import cn.iocoder.yudao.module.customer.vo.customer.complaint.*;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 客户投诉 Service 接口
 *
 * @author yanghao
 */
public interface CustomerComplaintService extends IService<CustomerComplaintDO> {

    /**
     * 创建客户投诉
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createCustomerComplaint(@Valid CustomerComplaintCreateReqVO createReqVO);

    /**
     * 更新客户投诉
     *
     * @param updateReqVO 更新信息
     */
    void updateCustomerComplaint(@Valid CustomerComplaintUpdateReqVO updateReqVO);

    /**
     * 删除客户投诉
     *
     * @param id 编号
     */
    void deleteCustomerComplaint(Long id);

    /**
     * 获得客户投诉
     *
     * @param id 编号
     * @return 客户投诉
     */
    CustomerComplaintDO getCustomerComplaint(Long id);

    /**
     * 获得客户投诉列表
     *
     * @param ids 编号
     * @return 客户投诉列表
     */
    List<CustomerComplaintDO> getCustomerComplaintList(Collection<Long> ids);

    /**
     * 获得客户投诉分页
     *
     * @param pageReqVO 分页查询
     * @return 客户投诉分页
     */
    PageResult<CustomerComplaintRespVO> getCustomerComplaintPage(
            CustomerComplaintPageReqVO pageReqVO);

    /**
     * 获得客户投诉列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 客户投诉列表
     */
    List<CustomerComplaintExcelVO> getCustomerComplaintList(
            CustomerComplaintExportReqVO exportReqVO);

}
