package cn.iocoder.yudao.module.customer.service.customer.complaint;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.customer.convert.customer.complaint.CustomerComplaintConvert;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.complaint.CustomerComplaintDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerContacts.CustomerContactsDO;
import cn.iocoder.yudao.module.customer.dal.mysql.customer.CustomerComplaintMapper;
import cn.iocoder.yudao.module.customer.service.customerContacts.CustomerContactsService;
import cn.iocoder.yudao.module.customer.vo.customer.complaint.*;
import cn.iocoder.yudao.module.ecw.api.internalMessage.ClientInternalMessageApi;
import cn.iocoder.yudao.module.ecw.api.internalMessage.dto.InternalMessageCreateDto;
import cn.iocoder.yudao.module.member.api.user.MemberUserApi;
import cn.iocoder.yudao.module.member.api.user.dto.UserRespDTO;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.CUSTOMER_COMPLAINT_NOT_EXISTS;

/**
 * 客户投诉 Service 实现类
 *
 * @author yanghao
 */
@Service
@Validated
public class CustomerComplaintServiceImpl extends AbstractService<CustomerComplaintMapper, CustomerComplaintDO> implements CustomerComplaintService {

    @Resource
    private CustomerComplaintMapper customerComplaintMapper;

    @Resource
    private MemberUserApi memberUserApi;

    @Resource
    private CustomerContactsService customerContactsService;

    @Resource
    private ClientInternalMessageApi clientInternalMessageApi;

    @Override
    public Long createCustomerComplaint(CustomerComplaintCreateReqVO createReqVO) {

        CustomerComplaintDO customerComplaint = CustomerComplaintConvert.INSTANCE.convert(createReqVO);

        Long memberUserId = createReqVO.getMemberUserId();
        if (memberUserId != null && createReqVO.getCustomerId() == null) {
            UserRespDTO user = memberUserApi.getUser(memberUserId);

            if (user != null) {
                CustomerContactsDO customerContactsDO = customerContactsService.selectOne(
                        CustomerContactsDO::getAreaCode, user.getAreaCode(), CustomerContactsDO::getPhoneNew, user.getMobile());
                if (customerContactsDO != null) {
                    customerComplaint.setCustomerId(customerContactsDO.getCustomerId());
                }
            }
        } else {
            //设置登记客户
            customerComplaint.setAdminId(SecurityFrameworkUtils.getLoginUserId());
        }
        // 插入
        customerComplaintMapper.insert(customerComplaint);

        //更新编号
        customerComplaint.setCode("TS" + customerComplaint.getId());
        customerComplaintMapper.updateById(customerComplaint);

        // 返回
        return customerComplaint.getId();
    }

    @Override
    public void updateCustomerComplaint(CustomerComplaintUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateCustomerComplaintExists(updateReqVO.getId());
        // 更新
        CustomerComplaintDO updateObj = CustomerComplaintConvert.INSTANCE.convert(updateReqVO);
        customerComplaintMapper.updateById(updateObj);

        //处理投诉
        InternalMessageCreateDto dto = InternalMessageCreateDto.builder().toIdList(Arrays.asList(updateReqVO.getCustomerId()))
                .fromId("")
                .sendTime(new Date())
                .title(I18nMessage.getMessage("app.message.customer.complain.title"))
                .content(I18nMessage.getMessage("app.message.customer.complain.content"))
                .type(3).build();
        clientInternalMessageApi.createInternalMessage(dto);
    }

    @Override
    public void deleteCustomerComplaint(Long id) {
        // 校验存在
        this.validateCustomerComplaintExists(id);
        // 删除
        customerComplaintMapper.deleteById(id);
    }

    private void validateCustomerComplaintExists(Long id) {
        if (customerComplaintMapper.selectById(id) == null) {
            throw exception(CUSTOMER_COMPLAINT_NOT_EXISTS);
        }
    }

    @Override
    public CustomerComplaintDO getCustomerComplaint(Long id) {
        return customerComplaintMapper.selectById(id);
    }

    @Override
    public List<CustomerComplaintDO> getCustomerComplaintList(Collection<Long> ids) {
        return customerComplaintMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<CustomerComplaintRespVO> getCustomerComplaintPage(CustomerComplaintPageReqVO pageReqVO) {
        IPage<CustomerComplaintRespVO> mPage = MyBatisUtils.buildPage(pageReqVO);
        customerComplaintMapper.selectPage(mPage, pageReqVO);
        return PageResult.of(mPage);
    }

    @Override
    public List<CustomerComplaintExcelVO> getCustomerComplaintList(CustomerComplaintExportReqVO exportReqVO) {
        return customerComplaintMapper.selectList(exportReqVO);
    }

}
