package cn.iocoder.yudao.module.customer.service.customer.complaint.follow;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.follow.CustomerFollowDO;
import cn.iocoder.yudao.module.customer.vo.customer.follow.CustomerFollowCreateReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.follow.CustomerFollowExportReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.follow.CustomerFollowPageReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.follow.CustomerFollowUpdateReqVO;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 客户投诉跟进 Service 接口
 *
 * @author yanghao
 */
public interface CustomerFollowService extends IService<CustomerFollowDO> {

    /**
     * 创建客户投诉跟进
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createCustomerFollow(@Valid CustomerFollowCreateReqVO createReqVO);

    /**
     * 更新客户投诉跟进
     *
     * @param updateReqVO 更新信息
     */
    void updateCustomerFollow(@Valid CustomerFollowUpdateReqVO updateReqVO);

    /**
     * 删除客户投诉跟进
     *
     * @param id 编号
     */
    void deleteCustomerFollow(Long id);

    /**
     * 获得客户投诉跟进
     *
     * @param id 编号
     * @return 客户投诉跟进
     */
    CustomerFollowDO getCustomerFollow(Long id);

    /**
     * 获得客户投诉跟进列表
     *
     * @param ids 编号
     * @return 客户投诉跟进列表
     */
    List<CustomerFollowDO> getCustomerFollowList(Collection<Long> ids);

    /**
     * 获得客户投诉跟进分页
     *
     * @param pageReqVO 分页查询
     * @return 客户投诉跟进分页
     */
    PageResult<CustomerFollowDO> getCustomerFollowPage(CustomerFollowPageReqVO pageReqVO);
    PageResult<CustomerFollowDO> customerFollowPage(CustomerFollowPageReqVO pageReqVO);

    /**
     * 获得客户投诉跟进列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 客户投诉跟进列表
     */
    List<CustomerFollowDO> getCustomerFollowList(CustomerFollowExportReqVO exportReqVO);

    List<CustomerFollowDO> offerFollowList(Long offerId);
}
