package cn.iocoder.yudao.module.customer.service.customer.complaint.follow;

import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.customer.convert.customer.complaint.follow.CustomerFollowConvert;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.follow.CustomerFollowDO;
import cn.iocoder.yudao.module.customer.dal.mysql.customer.CustomerFollowMapper;
import cn.iocoder.yudao.module.customer.vo.customer.follow.CustomerFollowCreateReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.follow.CustomerFollowExportReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.follow.CustomerFollowPageReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.follow.CustomerFollowUpdateReqVO;
import com.baomidou.mybatisplus.core.metadata.IPage;
import jodd.util.StringUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.CUSTOMER_FOLLOW_NOT_EXISTS;

/**
 * 客户投诉跟进 Service 实现类
 *
 * @author yanghao
 */
@Service
@Validated
public class CustomerFollowServiceImpl extends AbstractService<CustomerFollowMapper, CustomerFollowDO> implements CustomerFollowService {

    @Resource
    private CustomerFollowMapper customerFollowMapper;

    @Override
    public Long createCustomerFollow(CustomerFollowCreateReqVO createReqVO) {
        // 插入
        CustomerFollowDO customerFollowDO = CustomerFollowConvert.INSTANCE.convert(createReqVO);
        customerFollowMapper.insert(customerFollowDO);
        // 返回
        return customerFollowDO.getId();
    }

    @Override
    public void updateCustomerFollow(CustomerFollowUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateCustomerFollowExists(updateReqVO.getId());
        // 更新
        CustomerFollowDO updateObj = CustomerFollowConvert.INSTANCE.convert(updateReqVO);
        customerFollowMapper.updateById(updateObj);
    }

    @Override
    public void deleteCustomerFollow(Long id) {
        // 校验存在
        this.validateCustomerFollowExists(id);
        // 删除
        customerFollowMapper.deleteById(id);
    }

    private void validateCustomerFollowExists(Long id) {
        if (customerFollowMapper.selectById(id) == null) {
            throw exception(CUSTOMER_FOLLOW_NOT_EXISTS);
        }
    }

    @Override
    public CustomerFollowDO getCustomerFollow(Long id) {
        return customerFollowMapper.selectById(id);
    }

    @Override
    public List<CustomerFollowDO> getCustomerFollowList(Collection<Long> ids) {
        return customerFollowMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<CustomerFollowDO> getCustomerFollowPage(CustomerFollowPageReqVO pageReqVO) {
        return customerFollowMapper.selectPage(pageReqVO);
    }

    @Override
    public PageResult<CustomerFollowDO> customerFollowPage(CustomerFollowPageReqVO pageReqVO) {
        PageVO page = new PageVO();
        page.setPage(pageReqVO.getPageNo());
        page.setRows(pageReqVO.getPageSize());
        page.setField("f.id");
        IPage<CustomerFollowDO> mpPage = MyBatisUtils.buildPage(page);
        customerFollowMapper.customerFollowPage(mpPage, pageReqVO, StringUtils.isNotBlank(pageReqVO.getFollowType()) ? Arrays.asList(pageReqVO.getFollowType().split(StrUtil.COMMA)) : null);
        return PageResult.of(mpPage);
    }

    @Override
    public List<CustomerFollowDO> getCustomerFollowList(CustomerFollowExportReqVO exportReqVO) {
        return customerFollowMapper.selectList(exportReqVO);
    }

    @Override
    public List<CustomerFollowDO> offerFollowList(Long offerId) {
        return customerFollowMapper.offerFollowList(offerId);
    }
}
