package cn.iocoder.yudao.module.customer.service.customer.line;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.line.CustomerLineDO;
import cn.iocoder.yudao.module.customer.vo.customer.line.CustomerLineCreateReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.line.CustomerLineExportReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.line.CustomerLinePageReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.line.CustomerLineUpdateReqVO;
import cn.iocoder.yudao.module.customer.vo.zhongPao.CustomerZhongPaoQueryVO;

import javax.validation.Valid;
import java.util.List;

/**
 * 客户线路 Service 接口
 *
 * @author yanghao
 */
public interface CustomerLineService extends IService<CustomerLineDO> {

    /**
     * 创建客户线路
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createCustomerLine(@Valid CustomerLineCreateReqVO createReqVO);

    /**
     * 更新客户线路
     *
     * @param updateReqVO 更新信息
     */
    void updateCustomerLine(@Valid CustomerLineUpdateReqVO updateReqVO);

    /**
     * 删除客户线路
     *
     * @param id 编号
     */
    void deleteCustomerLine(Long id);

    /**
     * 获得客户线路
     *
     * @param id 编号
     * @return 客户线路
     */
    CustomerLineDO getCustomerLine(Long id);

    /**
     * 获得客户线路列表
     *
     *
     * @param customerZhongPaoQueryVO@return 客户线路列表
     */
    List<CustomerLineDO> getCustomerLineList(CustomerZhongPaoQueryVO customerZhongPaoQueryVO);

    /**
     * 获得客户线路分页
     *
     * @param pageReqVO 分页查询
     * @return 客户线路分页
     */
    PageResult<CustomerLineDO> getCustomerLinePage(CustomerLinePageReqVO pageReqVO);

    /**
     * 获得客户线路列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 客户线路列表
     */
    List<CustomerLineDO> getCustomerLineList(CustomerLineExportReqVO exportReqVO);


}
