package cn.iocoder.yudao.module.customer.service.customer.line;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.customer.convert.customer.line.CustomerLineConvert;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.line.CustomerLineDO;
import cn.iocoder.yudao.module.customer.dal.mysql.customer.CustomerLineMapper;
import cn.iocoder.yudao.module.customer.vo.customer.line.CustomerLineCreateReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.line.CustomerLineExportReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.line.CustomerLinePageReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.line.CustomerLineUpdateReqVO;
import cn.iocoder.yudao.module.customer.vo.zhongPao.CustomerZhongPaoQueryVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.CUSTOMER_LINE_NOT_EXISTS;

/**
 * 客户线路 Service 实现类
 *
 * @author yanghao
 */
@Service
@Validated
public class CustomerLineServiceImpl extends AbstractService<CustomerLineMapper, CustomerLineDO> implements CustomerLineService {

    @Resource
    private CustomerLineMapper customerLineMapper;

    @Override
    public Long createCustomerLine(CustomerLineCreateReqVO createReqVO) {
        // 插入
        CustomerLineDO customerLine = CustomerLineConvert.INSTANCE.convert(createReqVO);
        customerLineMapper.insert(customerLine);
        // 返回
        return customerLine.getId();
    }

    @Override
    public void updateCustomerLine(CustomerLineUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateCustomerLineExists(updateReqVO.getId());
        // 更新
        CustomerLineDO updateObj = CustomerLineConvert.INSTANCE.convert(updateReqVO);
        customerLineMapper.updateById(updateObj);
    }

    @Override
    public void deleteCustomerLine(Long id) {
        // 校验存在
        this.validateCustomerLineExists(id);
        // 删除
        customerLineMapper.deleteById(id);
    }

    private void validateCustomerLineExists(Long id) {
        if (customerLineMapper.selectById(id) == null) {
            throw exception(CUSTOMER_LINE_NOT_EXISTS);
        }
    }

    @Override
    public CustomerLineDO getCustomerLine(Long id) {
        return customerLineMapper.selectById(id);
    }

    @Override
    public List<CustomerLineDO> getCustomerLineList(CustomerZhongPaoQueryVO customerZhongPaoQueryVO) {

        LambdaQueryWrapper<CustomerLineDO> customerLineDOLambdaQueryWrapper = new LambdaQueryWrapperX<CustomerLineDO>()
                .eq(BaseDO::getDeleted, false)
                .eqIfPresent(CustomerLineDO::getCustomerId, customerZhongPaoQueryVO.getCustomerId())
                .eqIfPresent(CustomerLineDO::getDepartureId, customerZhongPaoQueryVO.getDepartureId())
                .apply(customerZhongPaoQueryVO.getObjectiveId() != null, "FIND_IN_SET ('" + customerZhongPaoQueryVO.getObjectiveId() + "',  objective_ids)");

        return customerLineMapper.selectList(customerLineDOLambdaQueryWrapper);
    }

    @Override
    public PageResult<CustomerLineDO> getCustomerLinePage(CustomerLinePageReqVO pageReqVO) {
        return customerLineMapper.selectPage(pageReqVO);
    }

    @Override
    public List<CustomerLineDO> getCustomerLineList(CustomerLineExportReqVO exportReqVO) {
        return customerLineMapper.selectList(exportReqVO);
    }

}
