package cn.iocoder.yudao.module.customer.service.customerAirLog;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.customer.vo.customerAirLog.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerAirLog.CustomerAirLogDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 客户空运日志 Service 接口
 *
 * @author 江楠
 */
public interface CustomerAirLogService extends IService<CustomerAirLogDO> {

    /**
     * 创建客户空运日志
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createAirLog(@Valid CustomerAirLogCreateReqVO createReqVO);

    /**
     * 更新客户空运日志
     * @param updateReqVO 更新信息
     */
    void updateAirLog(@Valid CustomerAirLogUpdateReqVO updateReqVO);

    /**
     * 删除客户空运日志
     * @param id 编号
     */
    void deleteAirLog(Long id);

    /**
     * 获得客户空运日志
     * @param id 编号
     * @return 客户空运日志
     */
    CustomerAirLogDO getAirLog(Long id);

    /**
     * 获得客户空运日志列表
     * @param ids 编号
     * @return 客户空运日志列表
     */
    List<CustomerAirLogDO> getAirLogList(Collection<Long> ids);

    /**
     * 获得客户空运日志分页
     * @param page 分页查询
     * @param query 查询
     * @return 客户空运日志分页
     */
    PageResult<CustomerAirLogDO> getAirLogPage(CustomerAirLogQueryVO query, PageVO page);

    /**
     * 获得客户空运日志列表, 用于 Excel 导出
     * @param query 查询
     * @return 客户空运日志列表
     */
    List<CustomerAirLogDO> getAirLogList(CustomerAirLogQueryVO query);
}
