package cn.iocoder.yudao.module.customer.service.customerAirLog;

import java.util.*;
import javax.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.customer.vo.customerAirLog.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerAirLog.CustomerAirLogDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.customer.convert.customerAirLog.CustomerAirLogConvert;
import cn.iocoder.yudao.module.customer.dal.mysql.customerAirLog.CustomerAirLogMapper;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.customer.enums.ErrorCodeConstants.*;

/**
 * 客户空运日志 Service 实现类
 *
 * @author 江楠
 */
@Service
@Validated
public class CustomerAirLogServiceImpl extends AbstractService<CustomerAirLogMapper, CustomerAirLogDO> implements CustomerAirLogService {

    @Resource
    private CustomerAirLogMapper airLogMapper;

    @Override
    public Long createAirLog(CustomerAirLogCreateReqVO createReqVO) {
        // 插入
        CustomerAirLogDO airLog = CustomerAirLogConvert.INSTANCE.convert(createReqVO);
        airLogMapper.insert(airLog);
        // 返回
        return airLog.getId();
    }

    @Override
    public void updateAirLog(CustomerAirLogUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateAirLogExists(updateReqVO.getId());
        // 更新
        CustomerAirLogDO updateObj = CustomerAirLogConvert.INSTANCE.convert(updateReqVO);
        airLogMapper.updateById(updateObj);
    }

    @Override
    public void deleteAirLog(Long id) {
        // 校验存在
        this.validateAirLogExists(id);
        // 删除
        airLogMapper.deleteById(id);
    }

    private void validateAirLogExists(Long id) {
        // if (airLogMapper.selectById(id) == null) {
        //     throw exception(AIR_LOG_NOT_EXISTS);
        // }
    }

    @Override
    public CustomerAirLogDO getAirLog(Long id) {
        return airLogMapper.selectById(id);
    }

    @Override
    public List<CustomerAirLogDO> getAirLogList(Collection<Long> ids) {
        return airLogMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<CustomerAirLogDO> getAirLogPage(CustomerAirLogQueryVO query, PageVO page) {
        return airLogMapper.selectPage(page, query);
    }

    @Override
    public List<CustomerAirLogDO> getAirLogList(CustomerAirLogQueryVO query) {
        return airLogMapper.selectList(query);
    }
}
