package cn.iocoder.yudao.module.customer.service.customerBank;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.customer.vo.customerBank.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerBank.CustomerBankDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 客户银行账户 Service 接口
 *
 * @author yanghao
 */
public interface CustomerBankService extends IService<CustomerBankDO> {

    /**
     * 创建客户银行账户
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createBank(@Valid CustomerBankCreateReqVO createReqVO);

    /**
     * 更新客户银行账户
     * @param updateReqVO 更新信息
     */
    void updateBank(@Valid CustomerBankUpdateReqVO updateReqVO);

    /**
     * 删除客户银行账户
     * @param id 编号
     */
    void deleteBank(Long id);

    /**
     * 获得客户银行账户
     * @param id 编号
     * @return 客户银行账户
     */
    CustomerBankDO getBank(Long id);

    /**
     * 获得客户银行账户列表
     * @param ids 编号
     * @return 客户银行账户列表
     */
    List<CustomerBankDO> getBankList(Collection<Long> ids);

    /**
     * 获得客户银行账户分页
     * @param page 分页查询
     * @param query 查询
     * @return 客户银行账户分页
     */
    PageResult<CustomerBankDO> getBankPage(CustomerBankQueryVO query, PageVO page);

    /**
     * 获得客户银行账户列表, 用于 Excel 导出
     * @param query 查询
     * @return 客户银行账户列表
     */
    List<CustomerBankDO> getBankList(CustomerBankQueryVO query);
}
