package cn.iocoder.yudao.module.customer.service.customerBank;

import java.util.*;
import javax.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.customer.vo.customerBank.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerBank.CustomerBankDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.customer.convert.customerBank.CustomerBankConvert;
import cn.iocoder.yudao.module.customer.dal.mysql.customerBank.CustomerBankMapper;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.customer.enums.ErrorCodeConstants.*;

/**
 * 客户银行账户 Service 实现类
 *
 * @author yanghao
 */
@Service
@Validated
public class CustomerBankServiceImpl extends AbstractService<CustomerBankMapper, CustomerBankDO> implements CustomerBankService {

    @Resource
    private CustomerBankMapper customerBankMapper;

    @Override
    public Long createBank(CustomerBankCreateReqVO createReqVO) {
        // 插入
        CustomerBankDO bank = CustomerBankConvert.INSTANCE.convert(createReqVO);
        customerBankMapper.insert(bank);
        // 返回
        return bank.getId();
    }

    @Override
    public void updateBank(CustomerBankUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateBankExists(updateReqVO.getId());
        // 更新
        CustomerBankDO updateObj = CustomerBankConvert.INSTANCE.convert(updateReqVO);
        customerBankMapper.updateById(updateObj);
    }

    @Override
    public void deleteBank(Long id) {
        // 校验存在
        this.validateBankExists(id);
        // 删除
        customerBankMapper.deleteById(id);
    }

    private void validateBankExists(Long id) {
        if (customerBankMapper.selectById(id) == null) {
            throw exception(BANK_NOT_EXISTS);
        }
    }

    @Override
    public CustomerBankDO getBank(Long id) {
        return customerBankMapper.selectById(id);
    }

    @Override
    public List<CustomerBankDO> getBankList(Collection<Long> ids) {
        return customerBankMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<CustomerBankDO> getBankPage(CustomerBankQueryVO query, PageVO page) {
        return customerBankMapper.selectPage(page, query);
    }

    @Override
    public List<CustomerBankDO> getBankList(CustomerBankQueryVO query) {
        return customerBankMapper.selectList(query);
    }
}
