package cn.iocoder.yudao.module.customer.service.customerContacts;

import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerContacts.CustomerContactsDO;
import cn.iocoder.yudao.module.customer.dto.CustomerContactsDto;
import cn.iocoder.yudao.module.customer.vo.customer.customerContacts.CustomerContactsCreateReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.customerContacts.CustomerContactsExportReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.customerContacts.CustomerContactsPageReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.customerContacts.CustomerContactsUpdateReqVO;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 客户联系人 Service 接口
 *
 * @author 系统管理员
 */
public interface CustomerContactsService extends IService<CustomerContactsDO> {

    /**
     * 创建客户联系人
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createCustomerContacts(@Valid CustomerContactsCreateReqVO createReqVO);

    /**
     * 更新客户联系人
     *
     * @param updateReqVO 更新信息
     */
    void updateCustomerContacts(@Valid CustomerContactsUpdateReqVO updateReqVO);

    /**
     * 删除客户联系人
     *
     * @param id 编号
     */
    void deleteCustomerContacts(Long id);

    /**
     * 获得客户联系人
     *
     * @param id 编号
     * @return 客户联系人
     */
    CustomerContactsDO getCustomerContacts(Long id);

    /**
     * 获得客户联系人列表
     *
     * @param ids 编号
     * @return 客户联系人列表
     */
    List<CustomerContactsDO> getCustomerContactsListByCustomerId(Collection<Long> ids);

    /**
     * 获得客户联系人列表
     *
     * @param areaCode 区号
     * @param phone 电话
     * @return 客户联系人列表
     */
    List<CustomerContactsDO> getByPhone(String areaCode, String phone);


    List<CustomerContactsDO> getByMemberUserId(Long memberUserId);


    /**
     * 获得客户联系人列表
     *
     * @param customerId 编号
     * @return 客户联系人列表
     */
    List<CustomerContactsDO> getCustomerContactsListByCustomerId(Long customerId);

    /**
     * 获得客户联系人分页
     *
     * @param pageReqVO 分页查询
     * @return 客户联系人分页
     */
    PageResult<CustomerContactsDO> getCustomerContactsPage(CustomerContactsPageReqVO pageReqVO);

    /**
     * 获得客户联系人列表, 用于 Excel 导出
     *
     * @param exportReqVO 查询条件
     * @return 客户联系人列表
     */
    List<CustomerContactsDO> getCustomerContactsListByCustomerId(CustomerContactsExportReqVO exportReqVO);

    CustomerContactsDO getDefaultContacts(Long customerId);

    /**
     * 模糊搜索指定数量的联系数据
     *
     * @param searchKey  关键字
     * @param type
     * @param ids        联系人ID
     * @param customerId 客户ID
     * @return 联系人列表
     */
    PageResult<CustomerContactsDto> selectBySearchKey(String searchKey, String type, Collection<Long> ids, Long customerId, PageParam page);

    /*
      添加的客户选择函数

     */
    PageResult<CustomerContactsDto> selectBySearchKey2(
            String searchKey, String type, Collection<Long> ids,
            Long customerId, PageParam page, Long UserId);



    /**
     * 根据客户ID更新EMAIL和公司名称
     * @return
     */
    void asyncUpdateEmail(Long customerContactsId, String email);





    /**
     * @param areaCode          区号  eg: 86
     * @param phone             电话  eg: 13233334444
     * @param excludeCustomerId 需要排除的客户id。目的是判断此客户外还有没有其他人拥有此联系人。更新时就需要使用这个字段
     * @return
     */
    CustomerContactsDO getUniqueOneAndValidate(String areaCode, String phone, Long excludeCustomerId);

}
