package cn.iocoder.yudao.module.customer.service.customerCredit;

import java.util.*;
import javax.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.customer.vo.customerCredit.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerCredit.CustomerCreditDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.customer.convert.customerCredit.CustomerCreditConvert;
import cn.iocoder.yudao.module.customer.dal.mysql.customerCredit.CustomerCreditMapper;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.customer.enums.ErrorCodeConstants.*;

/**
 * 信用等级 Service 实现类
 *
 * @author yanghao
 */
@Service
@Validated
public class CustomerCreditServiceImpl extends AbstractService<CustomerCreditMapper, CustomerCreditDO> implements CustomerCreditService {

    @Resource
    private CustomerCreditMapper creditMapper;

    @Override
    public Long createCredit(CustomerCreditCreateReqVO createReqVO) {
        // 插入
        CustomerCreditDO credit = CustomerCreditConvert.INSTANCE.convert(createReqVO);
        creditMapper.insert(credit);
        // 返回
        return credit.getId();
    }

    @Override
    public void updateCredit(CustomerCreditUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateCreditExists(updateReqVO.getId());
        // 更新
        CustomerCreditDO updateObj = CustomerCreditConvert.INSTANCE.convert(updateReqVO);
        creditMapper.updateById(updateObj);
    }

    @Override
    public void deleteCredit(Long id) {
        // 校验存在
        this.validateCreditExists(id);
        // 删除
        creditMapper.deleteById(id);
    }

    private void validateCreditExists(Long id) {
        if (creditMapper.selectById(id) == null) {
            throw exception(CREDIT_NOT_EXISTS);
        }
    }

    @Override
    public CustomerCreditDO getCredit(Long id) {
        return creditMapper.selectById(id);
    }

    @Override
    public List<CustomerCreditDO> getCreditList(Collection<Long> ids) {
        return creditMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<CustomerCreditDO> getCreditPage(CustomerCreditQueryVO query, PageVO page) {
        return creditMapper.selectPage(page, query);
    }

    @Override
    public List<CustomerCreditDO> getCreditList(CustomerCreditQueryVO query) {
        return creditMapper.selectList(query);
    }
}
