package cn.iocoder.yudao.module.customer.service.customerCreditLog;

import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerCreditLog.CustomerCreditLogDO;
import cn.iocoder.yudao.module.customer.vo.customerCreditLog.CustomerCreditLogBackVO;
import cn.iocoder.yudao.module.customer.vo.customerCreditLog.CustomerCreditLogCreateReqVO;
import cn.iocoder.yudao.module.customer.vo.customerCreditLog.CustomerCreditLogQueryVO;
import cn.iocoder.yudao.module.customer.vo.customerCreditLog.CustomerCreditLogUpdateReqVO;
import cn.iocoder.yudao.module.customer.vo.customerDetail.CustomerCreditRuleStatisticBackVo;

import javax.validation.Valid;
import java.math.BigDecimal;
import java.util.Collection;
import java.util.Date;
import java.util.List;

/**
 * 客户信用日志 Service 接口
 *
 * @author 系统管理员
 */
public interface CustomerCreditLogService extends IService<CustomerCreditLogDO> {

    /**
     * 计算提货信用日志
     *
     * @param customerId 订单收货人customerId
     * @param unloadTime 卸柜时间
     * @param pickTime   提货时间
     * @param orderId
     * @param orderNo
     */
    void pickupCredit(Long customerId, Date unloadTime, Date pickTime, Long orderId, String orderNo);

    void sendPackageCreditLog(Long customerContactId, Long orderId, String orderNo);

    /**
     * 创建客户信用日志
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createCreditLog(@Valid CustomerCreditLogCreateReqVO createReqVO);

    /**
     * 删除插入日志并更新客户
     *
     * @param
     * @param orderId
     * @return
     */
    void deleteCreditLogAndUpdateCustomer(Long customerId, Long orderId, String orderNo);

    /**
     * 更新客户信用日志
     *
     * @param updateReqVO 更新信息
     */
    void updateCreditLog(@Valid CustomerCreditLogUpdateReqVO updateReqVO);

    /**
     * 删除客户信用日志
     *
     * @param id 编号
     */
    void deleteCreditLog(Long id);

    /**
     * 获得客户信用日志
     *
     * @param id 编号
     * @return 客户信用日志
     */
    CustomerCreditLogDO getCreditLog(Long id);

    /**
     * 获得客户信用日志列表
     *
     * @param ids 编号
     * @return 客户信用日志列表
     */
    List<CustomerCreditLogDO> getCreditLogList(Collection<Long> ids);

    /**
     * 获得客户信用日志分页
     *
     * @param page  分页查询
     * @param query 查询
     * @return 客户信用日志分页
     */
    PageResult<CustomerCreditLogBackVO> getCreditLogPage(CustomerCreditLogQueryVO query, PageVO page);


    /**
     * 获得客户信用日志分页
     *
     * @param param  分页查询
     * @param customerId 查询
     * @return 客户信用日志分页
     */
    PageResult<CustomerCreditLogBackVO> getCreditLogPage(Long customerId, PageParam param);


    /**
     * 客户信用统计
     * @param customerId
     * @return
     */
    List<CustomerCreditRuleStatisticBackVo> getCreditRuleStatistic(Long customerId);

    /**
     * 客户信用统计
     * @param customerId
     * @return
     */
    BigDecimal getCreditRuleStatisticAll(Long customerId);

    /**
     * 获得客户信用日志列表, 用于 Excel 导出
     *
     * @param query 查询
     * @return 客户信用日志列表
     */
    List<CustomerCreditLogDO> getCreditLogList(CustomerCreditLogQueryVO query);
}
