package cn.iocoder.yudao.module.customer.service.customerCreditLog;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.customer.convert.customerCreditLog.CustomerCreditLogConvert;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerCredit.CustomerCreditDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerCreditLog.CustomerCreditLogDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerCreditRule.CustomerCreditRuleDO;
import cn.iocoder.yudao.module.customer.dal.mysql.customerCreditLog.CustomerCreditLogMapper;
import cn.iocoder.yudao.module.customer.service.customerCredit.CustomerCreditService;
import cn.iocoder.yudao.module.customer.service.customerCreditRule.CustomerCreditRuleService;
import cn.iocoder.yudao.module.customer.vo.customerCredit.CustomerCreditQueryVO;
import cn.iocoder.yudao.module.customer.vo.customerCreditLog.CustomerCreditLogBackVO;
import cn.iocoder.yudao.module.customer.vo.customerCreditLog.CustomerCreditLogCreateReqVO;
import cn.iocoder.yudao.module.customer.vo.customerCreditLog.CustomerCreditLogQueryVO;
import cn.iocoder.yudao.module.customer.vo.customerCreditLog.CustomerCreditLogUpdateReqVO;
import cn.iocoder.yudao.module.customer.vo.customerDetail.CustomerCreditRuleStatisticBackVo;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.CustomerDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerContacts.CustomerContactsDO;
import cn.iocoder.yudao.module.customer.service.customer.CustomerService;
import cn.iocoder.yudao.module.customer.service.customerContacts.CustomerContactsService;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.customer.enums.ErrorCodeConstants.CREDIT_LOG_NOT_EXISTS;

/**
 * 客户信用日志 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class CustomerCreditLogServiceImpl extends AbstractService<CustomerCreditLogMapper, CustomerCreditLogDO> implements CustomerCreditLogService {

    @Resource
    private CustomerCreditLogMapper creditLogMapper;

    @Resource
    private CustomerService customerService;

    @Resource
    private CustomerContactsService customerContactsService;

    @Resource
    private CustomerCreditRuleService customerCreditRuleService;

    @Resource
    private CustomerCreditService customerCreditService;


    @Override
    public void pickupCredit(Long customerId, Date unloadTime, Date pickTime, Long orderId, String orderNo) {

        if (unloadTime != null && pickTime != null) {

            CustomerCreditLogCreateReqVO reqVO = new CustomerCreditLogCreateReqVO();

            //收货人
            reqVO.setCustomerId(customerId);
            reqVO.setOrderId(orderId);
            reqVO.setOrderNo(orderNo);

            long day = DateUtil.between(pickTime, unloadTime, DateUnit.DAY);
            //2周内提货
            if (day <= 14) {
                reqVO.setRuleId(6L);
            }
            //2-3周内提货
            else if (day > 14 && day <= 21) {
                reqVO.setRuleId(7L);
            }
            //3-4周内提货
            else if (day > 21 && day <= 28) {
                reqVO.setRuleId(8L);
            }
            //1月后提货
            else if (day > 28 && day <= 150) {
                reqVO.setRuleId(9L);
            }
            //5个月后提货
            else if (day > 150) {
                reqVO.setRuleId(10L);
            }

            //插入日志
            this.createCreditLog(reqVO);
        }
    }

    @Override
    public void sendPackageCreditLog(Long customerContactId, Long orderId, String orderNo) {
        //发货人
        if (customerContactId != null) {
            CustomerContactsDO customerContacts = customerContactsService.getCustomerContacts(customerContactId);
            if (customerContacts != null) {
                //发货人对应的客户id
                Long customerId = customerContacts.getCustomerId();
                if (customerId != null) {
                    CustomerCreditLogCreateReqVO createReqVO = new CustomerCreditLogCreateReqVO();
                    createReqVO.setCustomerId(customerId);
                    createReqVO.setOrderId(orderId);
                    createReqVO.setOrderNo(orderNo);
                    //发货频次
                    createReqVO.setRuleId(11L);

                    this.createCreditLog(createReqVO);
                }
            }
        }

    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createCreditLog(CustomerCreditLogCreateReqVO createReqVO) {
        // 插入
        CustomerCreditLogDO creditLog = CustomerCreditLogConvert.INSTANCE.convert(createReqVO);

        //更新客户信用等级
        CustomerDO customer = customerService.getCustomer(createReqVO.getCustomerId());
        if (customer != null) {
            //查询当前规则分数
            CustomerCreditRuleDO creditRule = customerCreditRuleService.getCreditRule(createReqVO.getRuleId());
            List<CustomerCreditDO> creditList = customerCreditService.getCreditList(new CustomerCreditQueryVO());

            if (creditRule != null) {
                Integer score = creditRule.getScore();
                creditLog.setRuleScore(score);
                creditLog.setRemark(creditRule.getName());
                long scoreSum = (null == customer.getCreditLevelScore() ? 0 : customer.getCreditLevelScore()) + (null == score ? 0 : score);
                //vip等级分数
                customer.setCreditLevelScore(scoreSum);
                //设置等级
                creditList.stream().filter(t -> scoreSum >= t.getScoreMin() && scoreSum <= t.getScoreMax()).findFirst().ifPresent(customerCreditDO -> {
                    customer.setCreditLevel(customerCreditDO.getId().intValue());
                });

                customerService.updateById(customer);
            }
        }
        creditLogMapper.insert(creditLog);

        // 返回
        return creditLog.getId();
    }

    @Override
    public void deleteCreditLogAndUpdateCustomer(Long customerId, Long orderId, String orderNo) {
        //删除此客户此订单的等级提升数据
        List<CustomerCreditLogDO> customerCreditLogDOS = creditLogMapper.selectList(new LambdaQueryWrapperX<CustomerCreditLogDO>()
                .eq(CustomerCreditLogDO::getCustomerId, customerId).eq(CustomerCreditLogDO::getOrderId, orderId).eq(CustomerCreditLogDO::getOrderNo, orderNo)
                //发货量规则id
                .in(CustomerCreditLogDO::getRuleId, Arrays.asList(6,7,8,9,10)));
        if (CollectionUtil.isNotEmpty(customerCreditLogDOS)) {
            CustomerDO customer = customerService.getById(customerId);
            if (customer != null) {
                int oldAddScore = customerCreditLogDOS.stream().mapToInt(CustomerCreditLogDO::getRuleScore).sum();

                //减去分数
                long scoreSum = customer.getVipLevelScore() - oldAddScore;
                customer.setCreditLevelScore(scoreSum);

                //设置等级
                List<CustomerCreditDO> levelList = customerCreditService.getCreditList(new CustomerCreditQueryVO());
                if (CollectionUtil.isNotEmpty(levelList)) {
                    levelList.stream().filter(t -> scoreSum >= t.getScoreMin() && scoreSum <= t.getScoreMax()).findFirst().ifPresent(customerLevelDO -> {
                        customer.setCreditLevel(customerLevelDO.getId().intValue());
                    });
                    customerService.updateById(customer);
                }
            }
            creditLogMapper.deleteBatchIds(customerCreditLogDOS.stream().map(CustomerCreditLogDO::getId).collect(Collectors.toList()));
        }
    }

    @Override
    public void updateCreditLog(CustomerCreditLogUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateCreditLogExists(updateReqVO.getId());
        // 更新
        CustomerCreditLogDO updateObj = CustomerCreditLogConvert.INSTANCE.convert(updateReqVO);
        creditLogMapper.updateById(updateObj);
    }

    @Override
    public void deleteCreditLog(Long id) {
        // 校验存在
        this.validateCreditLogExists(id);
        // 删除
        creditLogMapper.deleteById(id);
    }

    private void validateCreditLogExists(Long id) {
        if (creditLogMapper.selectById(id) == null) {
            throw exception(CREDIT_LOG_NOT_EXISTS);
        }
    }

    @Override
    public CustomerCreditLogDO getCreditLog(Long id) {
        return creditLogMapper.selectById(id);
    }

    @Override
    public List<CustomerCreditLogDO> getCreditLogList(Collection<Long> ids) {
        return creditLogMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<CustomerCreditLogBackVO> getCreditLogPage(CustomerCreditLogQueryVO query, PageVO page) {
        IPage<CustomerCreditLogBackVO> mpPage = MyBatisUtils.buildPage(page);
        creditLogMapper.selectPage(mpPage, query);
        return PageResult.of(mpPage);
    }

    @Override
    public PageResult<CustomerCreditLogBackVO> getCreditLogPage(Long customerId, PageParam param) {
        IPage<CustomerCreditLogBackVO> mpPage = MyBatisUtils.buildPage(param);
        creditLogMapper.selectPageByCustomerId(mpPage, customerId);
        return PageResult.of(mpPage);
    }

    @Override
    public List<CustomerCreditRuleStatisticBackVo> getCreditRuleStatistic(Long customerId) {
        return creditLogMapper.getCreditRuleStatistic(customerId);
    }

    @Override
    public BigDecimal getCreditRuleStatisticAll(Long customerId) {
        return creditLogMapper.getCreditRuleStatisticAll(customerId);
    }

    @Override
    public List<CustomerCreditLogDO> getCreditLogList(CustomerCreditLogQueryVO query) {
        return creditLogMapper.selectList(query);
    }
}
