package cn.iocoder.yudao.module.customer.service.customerCreditRule;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.customer.vo.customerCreditRule.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerCreditRule.CustomerCreditRuleDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 信用规则 Service 接口
 *
 * @author yanghao
 */
public interface CustomerCreditRuleService extends IService<CustomerCreditRuleDO> {

    /**
     * 创建信用规则
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createCreditRule(@Valid CustomerCreditRuleCreateReqVO createReqVO);

    /**
     * 更新信用规则
     * @param updateReqVO 更新信息
     */
    void updateCreditRule(@Valid CustomerCreditRuleUpdateReqVO updateReqVO);

    /**
     * 删除信用规则
     * @param id 编号
     */
    void deleteCreditRule(Long id);

    /**
     * 获得信用规则
     * @param id 编号
     * @return 信用规则
     */
    CustomerCreditRuleDO getCreditRule(Long id);

    /**
     * 获得信用规则列表
     * @param ids 编号
     * @return 信用规则列表
     */
    List<CustomerCreditRuleDO> getCreditRuleList(Collection<Long> ids);

    /**
     * 获得信用规则分页
     * @param page 分页查询
     * @param query 查询
     * @return 信用规则分页
     */
    PageResult<CustomerCreditRuleDO> getCreditRulePage(CustomerCreditRuleQueryVO query, PageVO page);

    /**
     * 获得信用规则列表, 用于 Excel 导出
     * @param query 查询
     * @return 信用规则列表
     */
    List<CustomerCreditRuleDO> getCreditRuleList(CustomerCreditRuleQueryVO query);
}
