package cn.iocoder.yudao.module.customer.service.customerCreditRule;

import java.util.*;
import javax.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.customer.vo.customerCreditRule.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerCreditRule.CustomerCreditRuleDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.customer.convert.customerCreditRule.CustomerCreditRuleConvert;
import cn.iocoder.yudao.module.customer.dal.mysql.customerCreditRule.CustomerCreditRuleMapper;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.customer.enums.ErrorCodeConstants.*;

/**
 * 信用规则 Service 实现类
 *
 * @author yanghao
 */
@Service
@Validated
public class CustomerCreditRuleServiceImpl extends AbstractService<CustomerCreditRuleMapper, CustomerCreditRuleDO> implements CustomerCreditRuleService {

    @Resource
    private CustomerCreditRuleMapper creditRuleMapper;

    @Override
    public Long createCreditRule(CustomerCreditRuleCreateReqVO createReqVO) {
        // 插入
        CustomerCreditRuleDO creditRule = CustomerCreditRuleConvert.INSTANCE.convert(createReqVO);
        creditRuleMapper.insert(creditRule);
        // 返回
        return creditRule.getId();
    }

    @Override
    public void updateCreditRule(CustomerCreditRuleUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateCreditRuleExists(updateReqVO.getId());
        // 更新
        CustomerCreditRuleDO updateObj = CustomerCreditRuleConvert.INSTANCE.convert(updateReqVO);
        creditRuleMapper.updateById(updateObj);
    }

    @Override
    public void deleteCreditRule(Long id) {
        // 校验存在
        this.validateCreditRuleExists(id);
        // 删除
        creditRuleMapper.deleteById(id);
    }

    private void validateCreditRuleExists(Long id) {
        if (creditRuleMapper.selectById(id) == null) {
            throw exception(CREDIT_RULE_NOT_EXISTS);
        }
    }

    @Override
    public CustomerCreditRuleDO getCreditRule(Long id) {
        return creditRuleMapper.selectById(id);
    }

    @Override
    public List<CustomerCreditRuleDO> getCreditRuleList(Collection<Long> ids) {
        return creditRuleMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<CustomerCreditRuleDO> getCreditRulePage(CustomerCreditRuleQueryVO query, PageVO page) {
        return creditRuleMapper.selectPage(page, query);
    }

    @Override
    public List<CustomerCreditRuleDO> getCreditRuleList(CustomerCreditRuleQueryVO query) {
        return creditRuleMapper.selectList(query);
    }
}
