package cn.iocoder.yudao.module.customer.service.customerDetail;

import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.QueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.module.customer.dal.mysql.customerDetail.CustomerDetailMapper;
import cn.iocoder.yudao.module.customer.vo.customerDetail.*;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

/**
 * 客户详细信息 Service 实现类
 *
 * @author yanghao
 */
@Service
public class CustomeDetailServiceImpl implements CustomerDetailService {

    @Resource
    CustomerDetailMapper customerDetailMapper;

    @Override
    public PageResult<CustomerOrderBackVo> getCustomerOrderPage(CustomerOrderQueryVO customerOrderQueryVO) {
        IPage<CustomerOrderBackVo> mpPage = MyBatisUtils.buildPage(customerOrderQueryVO);
        QueryWrapperX queryWrapper = new QueryWrapperX();
        queryWrapper.eq("deleted", 0);
        queryWrapper.eq("customer_id", customerOrderQueryVO.getCustomerId());
        queryWrapper.eqIfPresent("customs_type", customerOrderQueryVO.getCustomerType());
        queryWrapper.eqIfPresent("transport_id", customerOrderQueryVO.getTransportId());
        queryWrapper.eqIfPresent("is_cargo_control", customerOrderQueryVO.getIsCargoControl());
        queryWrapper.betweenIfPresent("rucang_time", customerOrderQueryVO.getHouseStartDate(), customerOrderQueryVO.getHouseEndDate());
        customerDetailMapper.selectCustomerOrderPage(mpPage, queryWrapper);
        return PageResult.of(mpPage);
    }

    @Override
    public PageResult<CustomerOfferBackVo> getCustomerOfferPage(CustomerOfferQueryVo offerQueryVo, PageParam pageParam) {
        IPage<CustomerOfferBackVo> mpPage = MyBatisUtils.buildPage(pageParam);
        QueryWrapperX queryWrapper = new QueryWrapperX();
        queryWrapper.eq("t.deleted", 0);
        queryWrapper.eq("t.relation_id", offerQueryVo.getCustomerId());
        queryWrapper.inIfPresent("dst.guojia", offerQueryVo.getDestCountryId());
        queryWrapper.inIfPresent("dst.shi", offerQueryVo.getObjectiveId());
        queryWrapper.inIfPresent("dst.id", offerQueryVo.getDestWarehouseId());
        customerDetailMapper.selectCustomerOfferPage(mpPage, queryWrapper);
        return PageResult.of(mpPage);
    }

    @Override
    public Long getOrderCountByTransportId(Long customerId, Long transportId) {
        return customerDetailMapper.getOrderCountByTransportId(customerId, transportId);
    }

    @Override
    public Long getOrderCountByControl(Long customerId, Boolean isControl) {
        return customerDetailMapper.getOrderCountByControl(customerId, isControl);
    }

    @Override
    public Long getBaojiaCount(Long customerId) {
        return customerDetailMapper.getBaojiaCount(customerId);
    }

    @Override
    public Long getRucangCount(Long customerId) {
        return customerDetailMapper.getRucangCount(customerId);
    }

    @Override
    public Long getXiadanCount(Long customerId) {
        return customerDetailMapper.getXiadanCount(customerId);
    }

    @Override
    public Long getChuhuoCount(Long customerId) {
        return customerDetailMapper.getChuhuoCount(customerId);
    }

    @Override
    public Long getStartWarehouseCount(Long customerId) {
        return customerDetailMapper.getStartWareHouseCount(customerId);
    }

    @Override
    public Long getEndWarehouseCount(Long customerId) {
        return customerDetailMapper.getEndWareHouseCount(customerId);
    }

    @Override
    public OrderDto getLastOrder(Long customer) {
        return customerDetailMapper.getLastOrderByCustomerId(customer);
    }

    @Override
    public PageResult<CustomerReceivableBackVO> getReceiptPage(Long customerId, PageParam pageParam) {
        IPage<CustomerReceivableBackVO> mpPage = MyBatisUtils.buildPage(pageParam);
        QueryWrapperX queryWrapper = new QueryWrapperX();
        queryWrapper.eqIfPresent("receipt.customer_id", customerId);
        customerDetailMapper.selectCustomerReceiptPage(mpPage, queryWrapper);
        return PageResult.of(mpPage);
    }

    @Override
    public PageResult<CustomerFollowBackVo> getFollowPage(Long customerId, PageParam pageParam) {
        IPage<CustomerFollowBackVo> mpPage = MyBatisUtils.buildPage(pageParam);
        QueryWrapperX queryWrapper = new QueryWrapperX();
        queryWrapper.eqIfPresent("f.customer_id", customerId);
        customerDetailMapper.selectCustomerFollowPage(mpPage, queryWrapper);
        return PageResult.of(mpPage);
    }

    @Override
    public PageResult<CustomerComplaintBackVO> getComplaintPage(Long customerId, PageParam pageParam) {
        IPage<CustomerComplaintBackVO> mpPage = MyBatisUtils.buildPage(pageParam);
        QueryWrapperX queryWrapper = new QueryWrapperX();
        queryWrapper.eqIfPresent("f.customer_id", customerId);
        customerDetailMapper.selectCustomerComplaintPage(mpPage, queryWrapper);
        return PageResult.of(mpPage);
    }
}
