package cn.iocoder.yudao.module.customer.service.customerDetail;

import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.customer.vo.customerDetail.*;

/**
 * 客户详细信息Service 接口
 *
 * @author yanghao
 */
public interface CustomerDetailService {

    /**
     * 获得客户订单分页
     *
     * @param customerOrderQueryVO 查询参数
     * @return 客户级别分页
     */
    PageResult<CustomerOrderBackVo> getCustomerOrderPage(CustomerOrderQueryVO customerOrderQueryVO);

    /**
     * 获得客户报价分页
     *
     * @param offerQueryVo 客户ID
     * @param pageParam  分页参数
     * @return 客户级别分页
     */
    PageResult<CustomerOfferBackVo> getCustomerOfferPage(CustomerOfferQueryVo offerQueryVo, PageParam pageParam);

    /**
     * 根据运输方式得到订单总数
     *
     * @return
     */
    Long getOrderCountByTransportId(Long customerId, Long transportId);

    /**
     * 得到所有订单总数
     *
     * @return
     */
    Long getOrderCountByControl(Long customerId, Boolean isControl);

    /**
     * 得到报价单数
     *
     * @param customerId
     * @return
     */
    Long getBaojiaCount(Long customerId);

    /**
     * 得到入仓的总数
     *
     * @param customerId
     * @return
     */
    Long getRucangCount(Long customerId);


    /**
     * 得到下单数
     *
     * @param customerId
     * @return
     */
    Long getXiadanCount(Long customerId);

    /**
     * 得到出货订单数
     *
     * @param customerId
     * @return
     */
    Long getChuhuoCount(Long customerId);

    /**
     * 得到始发仓库库存订单
     *
     * @param customerId
     * @return
     */
    Long getStartWarehouseCount(Long customerId);

    /**
     * 得到始发仓库库存订单
     *
     * @param customerId
     * @return
     */
    Long getEndWarehouseCount(Long customerId);



    /**
     * 得到最后的订单数据
     *
     * @param customer
     * @return
     */
    OrderDto getLastOrder(Long customer);

    /**
     * 获得账单分页
     *
     * @param customerId 客户ID
     * @param pageParam  分页参数
     * @return 客户级别分页
     */
    PageResult<CustomerReceivableBackVO> getReceiptPage(Long customerId, PageParam pageParam);

    /**
     * 获得跟进分页
     *
     * @param customerId 客户ID
     * @param pageParam  分页参数
     * @return 客户级别分页
     */
    PageResult<CustomerFollowBackVo> getFollowPage(Long customerId, PageParam pageParam);

    /**
     * 获得客诉分页
     *
     * @param customerId 客户ID
     * @param pageParam  分页参数
     * @return 客户级别分页
     */
    PageResult<CustomerComplaintBackVO> getComplaintPage(Long customerId, PageParam pageParam);


}
