package cn.iocoder.yudao.module.customer.service.customerLevel;

import java.util.*;
import javax.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.customer.vo.customerLevel.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerLevel.CustomerLevelDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.customer.convert.customerLevel.CustomerLevelConvert;
import cn.iocoder.yudao.module.customer.dal.mysql.customerLevel.CustomerLevelMapper;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.customer.enums.ErrorCodeConstants.*;

/**
 * 客户级别 Service 实现类
 *
 * @author yanghao
 */
@Service
@Validated
public class CustomerLevelServiceImpl extends AbstractService<CustomerLevelMapper, CustomerLevelDO> implements CustomerLevelService {

    @Resource
    private CustomerLevelMapper levelMapper;

    @Override
    public Long createLevel(CustomerLevelCreateReqVO createReqVO) {
        // 插入
        CustomerLevelDO level = CustomerLevelConvert.INSTANCE.convert(createReqVO);
        levelMapper.insert(level);
        // 返回
        return level.getId();
    }

    @Override
    public void updateLevel(CustomerLevelUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateLevelExists(updateReqVO.getId());
        // 更新
        CustomerLevelDO updateObj = CustomerLevelConvert.INSTANCE.convert(updateReqVO);
        levelMapper.updateById(updateObj);
    }

    @Override
    public void deleteLevel(Long id) {
        // 校验存在
        this.validateLevelExists(id);
        // 删除
        levelMapper.deleteById(id);
    }

    private void validateLevelExists(Long id) {
        if (levelMapper.selectById(id) == null) {
            throw exception(LEVEL_NOT_EXISTS);
        }
    }

    @Override
    public CustomerLevelDO getLevel(Long id) {
        return levelMapper.selectById(id);
    }

    @Override
    public List<CustomerLevelDO> getLevelList(Collection<Long> ids) {
        return levelMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<CustomerLevelDO> getLevelPage(CustomerLevelQueryVO query, PageVO page) {
        return levelMapper.selectPage(page, query);
    }

    @Override
    public List<CustomerLevelDO> getLevelList(CustomerLevelQueryVO query) {
        return levelMapper.selectList(query);
    }
}
