package cn.iocoder.yudao.module.customer.service.customerLevelLog;

import java.math.BigDecimal;
import java.util.*;
import javax.validation.*;

import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.customer.vo.customerLevelLog.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerLevelLog.CustomerLevelLogDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.ecw.dal.dataobject.orderCode.OrderCodeDO;

/**
 * 客户等级日志 Service 接口
 *
 * @author 系统管理员
 */
public interface CustomerLevelLogService extends IService<CustomerLevelLogDO> {


    /**
     * 删除插入日志并更新客户
     *
     * @param
     * @param orderId
     * @return
     */
    void deleteLevelLogAndUpdateCustomer(Long customerId, Long orderId, String orderNo);


    /**
     * 插入日志并更新客户
     *
     * @param
     * @param sumVolume
     * @param orderId
     * @param orderNo
     * @return
     */
    Long createLevelLogAndUpdateCustomer(Long customerId, BigDecimal sumVolume, Long orderId, String orderNo,Long inRuleId);

    /**
     * 创建客户等级日志
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createLevelLog(@Valid CustomerLevelLogCreateReqVO createReqVO);

    /**
     * 更新客户等级日志
     * @param updateReqVO 更新信息
     */
    void updateLevelLog(@Valid CustomerLevelLogUpdateReqVO updateReqVO);

    /**
     * 删除客户等级日志
     * @param id 编号
     */
    void deleteLevelLog(Long id);

    /**
     * 获得客户等级日志
     * @param id 编号
     * @return 客户等级日志
     */
    CustomerLevelLogDO getLevelLog(Long id);

    /**
     * 获得客户等级日志列表
     * @param ids 编号
     * @return 客户等级日志列表
     */
    List<CustomerLevelLogDO> getLevelLogList(Collection<Long> ids);

    /**
     * 获得客户等级日志分页
     * @param page 分页查询
     * @param query 查询
     * @return 客户等级日志分页
     */
    PageResult<CustomerLevelLogBackVO> getLevelLogPage(CustomerLevelLogQueryVO query, PageVO page);

    /**
     * 获得客户等级日志分页
     * @param page 分页查询
     * @param customerId 查询
     * @return 客户等级日志分页
     */
    PageResult<CustomerLevelLogBackVO> getLevelLogPage(Long customerId, PageParam page);

    /**
     * 获得客户等级日志列表, 用于 Excel 导出
     * @param query 查询
     * @return 客户等级日志列表
     */
    List<CustomerLevelLogDO> getLevelLogList(CustomerLevelLogQueryVO query);
}
