package cn.iocoder.yudao.module.customer.service.customerLevelLog;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.customer.convert.customerLevelLog.CustomerLevelLogConvert;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerLevel.CustomerLevelDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerLevelLog.CustomerLevelLogDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerLevelRule.CustomerLevelRuleDO;
import cn.iocoder.yudao.module.customer.dal.mysql.customerLevelLog.CustomerLevelLogMapper;
import cn.iocoder.yudao.module.customer.service.customerLevel.CustomerLevelService;
import cn.iocoder.yudao.module.customer.service.customerLevelRule.CustomerLevelRuleService;
import cn.iocoder.yudao.module.customer.vo.customerLevelLog.CustomerLevelLogBackVO;
import cn.iocoder.yudao.module.customer.vo.customerLevelLog.CustomerLevelLogCreateReqVO;
import cn.iocoder.yudao.module.customer.vo.customerLevelLog.CustomerLevelLogQueryVO;
import cn.iocoder.yudao.module.customer.vo.customerLevelLog.CustomerLevelLogUpdateReqVO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.CustomerDO;
import cn.iocoder.yudao.module.customer.service.customer.CustomerService;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.customer.enums.ErrorCodeConstants.LEVEL_LOG_NOT_EXISTS;

/**
 * 客户等级日志 Service 实现类
 *
 * @author 系统管理员
 */
@Slf4j
@Service
@Validated
public class CustomerLevelLogServiceImpl extends
        AbstractService<CustomerLevelLogMapper, CustomerLevelLogDO>
        implements CustomerLevelLogService {

    @Resource
    private CustomerLevelLogMapper levelLogMapper;


    @Resource
    private CustomerService customerService;


    @Resource
    private CustomerLevelService customerLevelService;

    @Resource
    private CustomerLevelRuleService customerLevelRuleService;


    @Override
    public Long createLevelLogAndUpdateCustomer(Long customerId, BigDecimal sumVolume, Long orderId, String orderNo,Long inRuleId) {

        //更新客户vip等级
        CustomerDO customer = customerService.getCustomer(customerId);
        CustomerLevelLogDO levelLogDO = new CustomerLevelLogDO();
        if (customer != null) {
            levelLogDO.setCustomerId(customerId);

            levelLogDO.setOrderId(orderId);
            levelLogDO.setOrderNo(orderNo);

            Long ruleId = null;
            if (inRuleId != null && inRuleId > 0L) {
                ruleId = inRuleId;
            } else {
                //规则id
                if (sumVolume.compareTo(BigDecimal.TEN) == 1) {
                    ruleId = 3L;
                } else if (sumVolume.compareTo(BigDecimal.valueOf(5)) == 1) {
                    ruleId = 2L;
                } else if (sumVolume.compareTo(BigDecimal.valueOf(1)) == 1) {
                    ruleId = 1L;
                }
            }
            levelLogDO.setRuleId(ruleId);

            //查询当前规则分数
            CustomerLevelRuleDO levelRule = customerLevelRuleService.getLevelRule(ruleId);
            if (levelRule != null) {
                Integer ruleScore = levelRule.getScore();
                //规则分
                levelLogDO.setRuleScore(ruleScore);
                levelLogDO.setRemark(levelRule.getName());
                long oldScore = null == customer.getVipLevelScore() ? 0 : customer.getVipLevelScore();
                long scoreSum = oldScore + (null == ruleScore ? 0 : ruleScore);

                //vip等级分数
                customer.setVipLevelScore(scoreSum);

                //设置等级
                List<CustomerLevelDO> levelList = customerLevelService.selectList();
                if (CollectionUtil.isNotEmpty(levelList)) {
                    levelList.stream().filter(t -> scoreSum >= t.getScoreMin() && scoreSum <= t.getScoreMax()).findFirst().ifPresent(customerLevelDO -> {
                        customer.setLevel(customerLevelDO.getId().intValue());
                    });
                }
                customerService.updateById(customer);

                log.info("更新客户{}的vip积分，货量{}：等级积分加{}", customer.getName(), sumVolume, ruleScore);
            }

            // 插入
            levelLogMapper.insert(levelLogDO);
        }

        return levelLogDO.getId();
    }

    @Override
    public void deleteLevelLogAndUpdateCustomer(Long customerId, Long orderId, String orderNo) {

        //删除此客户此订单的等级提升数据
        List<CustomerLevelLogDO> customerLevelLogDOList = levelLogMapper.selectList(new LambdaQueryWrapperX<CustomerLevelLogDO>()
                .eq(CustomerLevelLogDO::getCustomerId, customerId).eq(CustomerLevelLogDO::getOrderId, orderId).eq(CustomerLevelLogDO::getOrderNo, orderNo)
                //发货量规则id
                .in(CustomerLevelLogDO::getRuleId, Arrays.asList(1,2,3,4,5)));
        if (CollectionUtil.isNotEmpty(customerLevelLogDOList)) {
            CustomerDO customer = customerService.getById(customerId);
            if (customer != null) {
                int oldAddScore = customerLevelLogDOList.stream().mapToInt(CustomerLevelLogDO::getRuleScore).sum();

                //减去分数
                long scoreSum = customer.getVipLevelScore() - oldAddScore;
                customer.setVipLevelScore(scoreSum);

                //设置等级
                List<CustomerLevelDO> levelList = customerLevelService.selectList();
                if (CollectionUtil.isNotEmpty(levelList)) {
                    levelList.stream().filter(t -> scoreSum >= t.getScoreMin() && scoreSum <= t.getScoreMax()).findFirst().ifPresent(customerLevelDO -> {
                        customer.setLevel(customerLevelDO.getId().intValue());
                    });
                    customerService.updateById(customer);
                    log.info("降低客户分灵敏，更新客户{}的vip积分，订单号：{}：等级分扣减{}", customer.getName(), orderNo, scoreSum);
                }
            }
            levelLogMapper.deleteBatchIds(customerLevelLogDOList.stream().map(CustomerLevelLogDO::getId).collect(Collectors.toList()));
        }

    }

    @Override
    public Long createLevelLog(CustomerLevelLogCreateReqVO createReqVO) {
        // 插入
        CustomerLevelLogDO levelLog = CustomerLevelLogConvert.INSTANCE.convert(createReqVO);
        levelLogMapper.insert(levelLog);

        // 返回
        return levelLog.getId();
    }

    @Override
    public void updateLevelLog(CustomerLevelLogUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateLevelLogExists(updateReqVO.getId());
        // 更新
        CustomerLevelLogDO updateObj = CustomerLevelLogConvert.INSTANCE.convert(updateReqVO);
        levelLogMapper.updateById(updateObj);
    }

    @Override
    public void deleteLevelLog(Long id) {
        // 校验存在
        this.validateLevelLogExists(id);
        // 删除
        levelLogMapper.deleteById(id);
    }

    private void validateLevelLogExists(Long id) {
        if (levelLogMapper.selectById(id) == null) {
            throw exception(LEVEL_LOG_NOT_EXISTS);
        }
    }

    @Override
    public CustomerLevelLogDO getLevelLog(Long id) {
        return levelLogMapper.selectById(id);
    }

    @Override
    public List<CustomerLevelLogDO> getLevelLogList(Collection<Long> ids) {
        return levelLogMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<CustomerLevelLogBackVO> getLevelLogPage(CustomerLevelLogQueryVO query, PageVO page) {
        IPage<CustomerLevelLogBackVO> mpPage = MyBatisUtils.buildPage(page);
        levelLogMapper.selectPage(mpPage, query);
        return PageResult.of(mpPage);
    }

    @Override
    public PageResult<CustomerLevelLogBackVO> getLevelLogPage(Long customerId, PageParam page) {
        IPage<CustomerLevelLogBackVO> mpPage = MyBatisUtils.buildPage(page);
        CustomerLevelLogQueryVO query = new CustomerLevelLogQueryVO();
        query.setCustomerId(customerId);
        levelLogMapper.selectPage(mpPage, query);
        return PageResult.of(mpPage);
    }

    @Override
    public List<CustomerLevelLogDO> getLevelLogList(CustomerLevelLogQueryVO query) {
        return levelLogMapper.selectList(query);
    }
}
