package cn.iocoder.yudao.module.customer.service.customerLevelRule;

import java.util.*;
import javax.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.customer.vo.customerLevelRule.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerLevelRule.CustomerLevelRuleDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.customer.convert.customerLevelRule.CustomerLevelRuleConvert;
import cn.iocoder.yudao.module.customer.dal.mysql.customerLevelRule.CustomerLevelRuleMapper;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.customer.enums.ErrorCodeConstants.*;

/**
 * 客户级别规则 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class CustomerLevelRuleServiceImpl extends AbstractService<CustomerLevelRuleMapper, CustomerLevelRuleDO> implements CustomerLevelRuleService {

    @Resource
    private CustomerLevelRuleMapper levelRuleMapper;

    @Override
    public Long createLevelRule(CustomerLevelRuleCreateReqVO createReqVO) {
        // 插入
        CustomerLevelRuleDO levelRule = CustomerLevelRuleConvert.INSTANCE.convert(createReqVO);
        levelRuleMapper.insert(levelRule);
        // 返回
        return levelRule.getId();
    }

    @Override
    public void updateLevelRule(CustomerLevelRuleUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateLevelRuleExists(updateReqVO.getId());
        // 更新
        CustomerLevelRuleDO updateObj = CustomerLevelRuleConvert.INSTANCE.convert(updateReqVO);
        levelRuleMapper.updateById(updateObj);
    }

    @Override
    public void deleteLevelRule(Long id) {
        // 校验存在
        this.validateLevelRuleExists(id);
        // 删除
        levelRuleMapper.deleteById(id);
    }

    private void validateLevelRuleExists(Long id) {
        if (levelRuleMapper.selectById(id) == null) {
            throw exception(LEVEL_RULE_NOT_EXISTS);
        }
    }

    @Override
    public CustomerLevelRuleDO getLevelRule(Long id) {
        return levelRuleMapper.selectById(id);
    }

    @Override
    public List<CustomerLevelRuleDO> getLevelRuleList(Collection<Long> ids) {
        return levelRuleMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<CustomerLevelRuleDO> getLevelRulePage(CustomerLevelRuleQueryVO query, PageVO page) {
        return levelRuleMapper.selectPage(page, query);
    }

    @Override
    public List<CustomerLevelRuleDO> getLevelRuleList(CustomerLevelRuleQueryVO query) {
        return levelRuleMapper.selectList(query);
    }
}
