package cn.iocoder.yudao.module.customer.service.customerOperateLog;

import java.util.*;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.customer.vo.customerOperateLog.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerOperateLog.CustomerOperateLogDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.customer.convert.customerOperateLog.CustomerOperateLogConvert;
import cn.iocoder.yudao.module.customer.dal.mysql.customerOperateLog.CustomerOperateLogMapper;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.customer.enums.ErrorCodeConstants.*;

/**
 * 客户操作日志 Service 实现类
 *
 * @author yanghao
 */
@Service
@Validated
public class CustomerOperateLogServiceImpl extends AbstractService<CustomerOperateLogMapper, CustomerOperateLogDO> implements CustomerOperateLogService {

    @Resource
    private CustomerOperateLogMapper customerOperateLogMapper;

    @Override
    public Long createOperateLog(CustomerOperateLogCreateReqVO createReqVO) {
        try {
            // 插入
            CustomerOperateLogDO operateLog = CustomerOperateLogConvert.INSTANCE.convert(createReqVO);
            customerOperateLogMapper.insert(operateLog);
            // 返回
            return operateLog.getId();
        } catch (Exception e) {
            log.error("插入客户操作日志出现异常", e);
            return 0L;
        }
    }

    @Override
    public void batchCreateOperateLog(List<CustomerOperateLogCreateReqVO> createReqVOList) {
        List<CustomerOperateLogDO> customerOperateLogDOList = createReqVOList.stream().map(CustomerOperateLogConvert.INSTANCE::convert).collect(Collectors.toList());
        customerOperateLogMapper.insertBatch(customerOperateLogDOList);
    }

    @Override
    public void updateOperateLog(CustomerOperateLogUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateOperateLogExists(updateReqVO.getId());
        // 更新
        CustomerOperateLogDO updateObj = CustomerOperateLogConvert.INSTANCE.convert(updateReqVO);
        customerOperateLogMapper.updateById(updateObj);
    }

    @Override
    public void deleteOperateLog(Long id) {
        // 校验存在
        this.validateOperateLogExists(id);
        // 删除
        customerOperateLogMapper.deleteById(id);
    }

    private void validateOperateLogExists(Long id) {
        if (customerOperateLogMapper.selectById(id) == null) {
            throw exception(OPERATE_LOG_NOT_EXISTS);
        }
    }

    @Override
    public CustomerOperateLogDO getOperateLog(Long id) {
        return customerOperateLogMapper.selectById(id);
    }

    @Override
    public List<CustomerOperateLogDO> getOperateLogList(Collection<Long> ids) {
        return customerOperateLogMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<CustomerOperateLogDO> getOperateLogPage(
            CustomerOperateLogQueryVO query, PageVO page) {
        return customerOperateLogMapper.selectPage(page, query);
    }

    @Override
    public List<CustomerOperateLogDO> getOperateLogList(CustomerOperateLogQueryVO query) {
        return customerOperateLogMapper.selectList(query);
    }
}
