package cn.iocoder.yudao.module.customer.service.customerPublicCatchLog;

import java.util.*;
import javax.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.customer.vo.customerPublicCatchLog.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerPublicCatchLog.CustomerPublicCatchLogDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.customer.convert.customerPublicCatchLog.CustomerPublicCatchLogConvert;
import cn.iocoder.yudao.module.customer.dal.mysql.customerPublicCatchLog.CustomerPublicCatchLogMapper;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.customer.enums.ErrorCodeConstants.*;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.PUBLIC_CATCH_LOG_NOT_EXISTS;

/**
 * 公海客户捞取日志 Service 实现类
 *
 * @author yanghao
 */
@Service
@Validated
public class CustomerPublicCatchLogServiceImpl extends AbstractService<CustomerPublicCatchLogMapper, CustomerPublicCatchLogDO> implements CustomerPublicCatchLogService {

    @Resource
    private CustomerPublicCatchLogMapper publicCatchLogMapper;

    @Override
    public Long createPublicCatchLog(CustomerPublicCatchLogCreateReqVO createReqVO) {
        // 插入
        CustomerPublicCatchLogDO publicCatchLog = CustomerPublicCatchLogConvert.INSTANCE.convert(createReqVO);
        publicCatchLogMapper.insert(publicCatchLog);
        // 返回
        return publicCatchLog.getId();
    }

    @Override
    public void updatePublicCatchLog(CustomerPublicCatchLogUpdateReqVO updateReqVO) {
        // 校验存在
        this.validatePublicCatchLogExists(updateReqVO.getId());
        // 更新
        CustomerPublicCatchLogDO updateObj = CustomerPublicCatchLogConvert.INSTANCE.convert(updateReqVO);
        publicCatchLogMapper.updateById(updateObj);
    }

    @Override
    public void deletePublicCatchLog(Long id) {
        // 校验存在
        this.validatePublicCatchLogExists(id);
        // 删除
        publicCatchLogMapper.deleteById(id);
    }

    private void validatePublicCatchLogExists(Long id) {
        if (publicCatchLogMapper.selectById(id) == null) {
            throw exception(PUBLIC_CATCH_LOG_NOT_EXISTS);
        }
    }

    @Override
    public CustomerPublicCatchLogDO getPublicCatchLog(Long id) {
        return publicCatchLogMapper.selectById(id);
    }

    @Override
    public List<CustomerPublicCatchLogDO> getPublicCatchLogList(Collection<Long> ids) {
        return publicCatchLogMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<CustomerPublicCatchLogDO> getPublicCatchLogPage(CustomerPublicCatchLogQueryVO query, PageVO page) {
        return publicCatchLogMapper.selectPage(page, query);
    }

    @Override
    public List<CustomerPublicCatchLogDO> getPublicCatchLogList(CustomerPublicCatchLogQueryVO query) {
        return publicCatchLogMapper.selectList(query);
    }
}
