package cn.iocoder.yudao.module.customer.service.indirectCustomer;

import java.util.*;
import javax.validation.*;

import cn.iocoder.yudao.framework.apollo.core.event.AdminClientCreateEvent;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.customer.dto.indirectCustomer.IndirectCustomerExportReqDTO;
import cn.iocoder.yudao.module.customer.dto.indirectCustomer.IndirectCustomerPageReqDTO;
import cn.iocoder.yudao.module.customer.vo.customer.vo.CustomerUpdateReqVO;
import cn.iocoder.yudao.module.customer.vo.indirectCustomer.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.indirectCustomer.IndirectCustomerDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 间接客户 Service 接口
 *
 * @author yanghao
 */
public interface IndirectCustomerService extends IService<IndirectCustomerDO> {

    /**
     * 创建间接客户
     *
     * @param adminClientCreateEvent
     * @return 编号
     */
    void createIndirectCustomer(AdminClientCreateEvent adminClientCreateEvent);

    /**
     * 创建间接客户
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createIndirectCustomer(@Valid IndirectCustomerCreateReqVO createReqVO);

    /**
     * 更新客户
     *
     * @param updateReqVO 更新信息
     */
    void updateCustomer(@Valid IndirectCustomerUpdateReqVO updateReqVO);

    /**
     * 删除间接客户
     * @param id 编号
     */
    void deleteIndirectCustomer(Long id);

    /**
     * 获得间接客户
     * @param id 编号
     * @return 间接客户
     */
    IndirectCustomerDO getIndirectCustomer(Long id);

    /**
     * 获得间接客户列表
     * @param ids 编号
     * @return 间接客户列表
     */
    List<IndirectCustomerDO> getIndirectCustomerList(Collection<Long> ids);

    /**
     * 获得间接客户分页
     * @param pageReqVO
     * @return 间接客户分页
     */
    PageResult<IndirectCustomerDO> getIndirectCustomerPage(IndirectCustomerPageReqVO pageReqVO);

    PageResult<IndirectCustomerDO> getIndirectCustomerPageReq(IndirectCustomerPageReqDTO pageReqVO, PageVO page);

    List<IndirectCustomerDO> getIndirectCustomerListReq(IndirectCustomerPageReqDTO reqVO);

    /**
     * 获得间接客户列表, 用于 Excel 导出
     * @param reqVO 查询
     * @return 间接客户列表
     */
    List<IndirectCustomerDO> getIndirectCustomerList(IndirectCustomerExportReqVO reqVO);

}
