package cn.iocoder.yudao.module.customer.service.indirectCustomer;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.RandomUtil;
import cn.iocoder.yudao.framework.apollo.core.event.AdminClientCreateEvent;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.web.core.util.WebFrameworkUtils;
import cn.iocoder.yudao.module.customer.convert.indirectCustomer.IndirectCustomerConvert;
import cn.iocoder.yudao.module.customer.convert.indirectCustomerContacts.IndirectCustomerContactsConvert;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.CustomerDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerContacts.CustomerContactsDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.indirectCustomer.IndirectCustomerDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.indirectCustomerContacts.IndirectCustomerContactsDO;
import cn.iocoder.yudao.module.customer.dal.mysql.indirectCustomer.IndirectCustomerMapper;
import cn.iocoder.yudao.module.customer.dal.mysql.indirectCustomerContacts.IndirectCustomerContactsMapper;
import cn.iocoder.yudao.module.customer.dto.indirectCustomer.IndirectCustomerPageReqDTO;
import cn.iocoder.yudao.module.customer.service.customer.CustomerService;
import cn.iocoder.yudao.module.customer.service.customerContacts.CustomerContactsService;
import cn.iocoder.yudao.module.customer.vo.indirectCustomer.IndirectCustomerCreateReqVO;
import cn.iocoder.yudao.module.customer.vo.indirectCustomer.IndirectCustomerExportReqVO;
import cn.iocoder.yudao.module.customer.vo.indirectCustomer.IndirectCustomerPageReqVO;
import cn.iocoder.yudao.module.customer.vo.indirectCustomer.IndirectCustomerUpdateReqVO;
import cn.iocoder.yudao.module.customer.vo.indirectCustomerContacts.IndirectCustomerContactsCreateReqVO;
import cn.iocoder.yudao.module.customer.vo.indirectCustomerContacts.IndirectCustomerContactsUpdateReqVO;
import cn.iocoder.yudao.module.ecw.dal.dataobject.country.CountryDO;
import cn.iocoder.yudao.module.ecw.enums.CustomerStatusEnum;
import cn.iocoder.yudao.module.ecw.service.country.CountryService;
import cn.iocoder.yudao.module.ecw.vo.country.CountryQueryVO;
import cn.iocoder.yudao.module.member.enums.ErrorCodeConstants;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.customer.enums.ErrorCodeConstants.INDIRECT_CUSTOMER_NOT_EXISTS;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.CUSTOMER_NOT_EXISTS;

/**
 * 间接客户 Service 实现类
 *
 * @author yanghao
 */
@Service
@Validated
public class IndirectCustomerServiceImpl extends AbstractService<IndirectCustomerMapper, IndirectCustomerDO> implements IndirectCustomerService {

    @Resource
    private IndirectCustomerMapper indirectCustomerMapper;

    @Resource
    private IndirectCustomerContactsMapper indirectCustomerContactsMapper;

    @Resource
    private CustomerService customerService;

    @Resource
    private CountryService countryService;

    @Resource
    private CustomerContactsService customerContactsService;


    public String genCustomerNum() {
        String numbers = RandomUtil.randomNumbers(10);
        long count = this.count(new LambdaQueryWrapperX<IndirectCustomerDO>().eq(IndirectCustomerDO::getNumber, numbers));
        if (count > 0) {
            return genCustomerNum();
        }
        return numbers;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void createIndirectCustomer(AdminClientCreateEvent event) {

        log.trace("开始创建间接客户：" + event.toString());

        if (event == null || event.getUid() == null) {
            throw exception(ErrorCodeConstants.USER_NOT_EXISTS);
        }
        Long userId = event.getUid();
        if (userId == 0) {
            log.error("用户id为0，直接返回");
            return;
        }
        Integer dialCode = event.getDialCode();
        String phone = event.getPhone();
        if (dialCode == null || StringUtils.isEmpty(phone)) {
            log.error("电话信息不完整");
        }

        // 获取当前会员关联的customerId
        List<CustomerContactsDO> customerContactsDOS =
                customerContactsService.getByMemberUserId(userId);
        if (CollectionUtil.isEmpty(customerContactsDOS)) {
            log.error("此会员不存在关联联系人");
            return;
        }
        if (CollectionUtil.isNotEmpty(customerContactsDOS) &&
                customerContactsDOS.size() > 1) {
            log.error(String.format("此用户的关联客户不止一人，请处理, userid=%d", userId));
            return;
        }

        CustomerContactsDO customerContactsDO = customerContactsDOS.get(0);
        Long customerId = customerContactsDO.getCustomerId();

        // 会员关联的客户
        CustomerDO agentCustomer = customerService.getCustomer(customerId);
        if (agentCustomer == null) {
            log.error(String.format("用户不存在, id=%d", customerId));
            return;
        }

        String type = agentCustomer.getType();
        // 同行或代理
        if (StringUtils.isEmpty(type) || (!type.contains("2") && !type.contains("3"))) {
            log.trace("非同行或代理，不需要建立间接客户");
            return;
        }

        // 先通过手机号查询此客户是否存在
        CustomerContactsDO customerContacts = customerContactsService.getUniqueOneAndValidate(dialCode.toString(), phone, null);
        if (customerContacts != null) {
            log.error(String.format("+%d%s在系统中已有档案， 不需要建立间接客户", dialCode, phone));
            return;
        }

        //看间接客户是否已存在
        IndirectCustomerExportReqVO query1 = new IndirectCustomerExportReqVO();
        query1.setName(event.getName());
        query1.setDefaultContactPhone(event.getPhone());
        List<IndirectCustomerDO> indirectCustomerList = this.getIndirectCustomerList(query1);
        if (CollectionUtil.isNotEmpty(indirectCustomerList)) {
            log.warn("间接联系人已存在");
            return;
        }


        IndirectCustomerDO entity = new IndirectCustomerDO();
        entity.setNumber(this.genCustomerNum());

        //间接客户的代理人，就是此会员对应的customer
        entity.setAgentCustomerId(agentCustomer.getId());

        entity.setName(event.getName());

        entity.setAddress(event.getAddress());
        if (dialCode != null) {
            CountryQueryVO query = new CountryQueryVO();
            query.setTel(dialCode.toString());
            List<CountryDO> countryList = countryService.getCountryList(query);
            if (CollectionUtil.isNotEmpty(countryList)) {
                entity.setCountry(countryList.get(0).getId());
            }
        }
        // 普通
        entity.setLevel(1);
        // 收货人
        entity.setType("1");

        // 来源： 发货人推荐
        entity.setSource(12);

        // 客户经理： 间接客户的客户经理为发货人（下单会员关联的同行或代理客户）的客户经理
        entity.setCustomerService(agentCustomer.getCustomerService());

        // 线索
        entity.setStatus(CustomerStatusEnum.CLUE.getValue());

        indirectCustomerMapper.insert(entity);


        // 设置间接联系人

        Long indirectCustomerId = entity.getId();
        IndirectCustomerContactsDO contacts = new IndirectCustomerContactsDO();
        contacts.setCustomerId(indirectCustomerId);
        contacts.setIsDefault(1);
        contacts.setName(event.getName());
        if (dialCode != null) {
            contacts.setAreaCode(dialCode.toString());
        }
        contacts.setIsAuto(2);
        contacts.setEmail(event.getEmail());
        contacts.setPhoneNew(event.getPhone());
        indirectCustomerContactsMapper.insert(contacts);

        log.trace("间接客户创建完成 间接客户id=" + indirectCustomerId);
    }

    @Override
    public Long createIndirectCustomer(IndirectCustomerCreateReqVO createReqVO) {
        // 插入
        IndirectCustomerDO indirectCustomer = IndirectCustomerConvert.INSTANCE.convert(createReqVO);
        indirectCustomerMapper.insert(indirectCustomer);
        // 返回
        return indirectCustomer.getId();
    }


    private void validateCustomerExists(Long id) {
        if (indirectCustomerMapper.selectById(id) == null) {
            throw exception(CUSTOMER_NOT_EXISTS);
        }
    }

    @Override
    public void updateCustomer(IndirectCustomerUpdateReqVO updateReqVO) {
        // 校验存在
        Long customerId = updateReqVO.getId();

        this.validateCustomerExists(customerId);

        IndirectCustomerDO customerOld = indirectCustomerMapper.selectById(customerId);

        // 更新
        IndirectCustomerDO customerNew = IndirectCustomerConvert.INSTANCE.convert(updateReqVO);

        //完善信息后，就代表被接收，就退出公海池
        if (!Boolean.TRUE.equals(customerOld.getIsCustomerServiceConfirmed())
                && Boolean.TRUE.equals(customerNew.getIsCustomerServiceConfirmed())) {
            Date now = new Date();
            //退出公海
            customerNew.setIsInOpenSea(false);

            //更新客户经理
            customerNew.setCustomerService(WebFrameworkUtils.getLoginUserId());
            //设置接收
            customerNew.setIsCustomerServiceConfirmed(true);
            //老的无分配时间时，需要设置分配时间
            if (customerOld.getCustomerServiceAssignedTime() == null) {
                customerNew.setCustomerServiceAssignedTime(now);
            }

            //接收后一个月，无任何报价、下单则掉入公海
            //Date date = DateUtil.offsetMonth(now, 1).toJdkDate();
            Date date = DateUtil.offsetMonth(now, 3).toJdkDate();//改为3个月
            if (customerNew.getEstimateEnterOpenSeaTime() == null || customerNew.getEstimateEnterOpenSeaTime().compareTo(date) == -1) {
                customerNew.setEstimateEnterOpenSeaTime(date);
            }
        }

        indirectCustomerMapper.updateById(customerNew);

        //更新联系人
        List<IndirectCustomerContactsUpdateReqVO> customerContacts = updateReqVO.getCustomerContacts();
        if (!CollectionUtils.isEmpty(customerContacts)) {

            List<IndirectCustomerContactsDO> customerContactsOld = indirectCustomerContactsMapper.selectList(IndirectCustomerContactsDO::getCustomerId, customerId);
            if (!CollectionUtils.isEmpty(customerContactsOld)) {

                //old contact id
                List<Long> idsOld = customerContactsOld.stream().map(IndirectCustomerContactsDO::getId).collect(Collectors.toList());

                //update contact
                List<IndirectCustomerContactsDO> updateList = customerContacts.stream().filter(t -> idsOld.contains(t.getId())).map(customerContactsUpdateReqVO ->
                {
                    IndirectCustomerContactsDO convert1 = IndirectCustomerContactsConvert.INSTANCE.convert(customerContactsUpdateReqVO);
                    convert1.setCustomerId(customerId);
                    return convert1;
                }).collect(Collectors.toList());
                if (!CollectionUtils.isEmpty(updateList)) {
                    indirectCustomerContactsMapper.updateBatch(updateList);
                }

                //delete contact
                List<Long> deleteIds = idsOld.stream().filter(old -> customerContacts.stream().noneMatch(u -> old.equals(u.getId()))).collect(Collectors.toList());
                if (!CollectionUtils.isEmpty(deleteIds)) {
                    indirectCustomerContactsMapper.deleteBatchIds(deleteIds);
                }
            }

            List<IndirectCustomerContactsDO> customerContactsOldTmp = CollectionUtils.isEmpty(customerContactsOld) ? new ArrayList<>() : customerContactsOld;
            //add contact
            customerContacts.stream()
                    .filter(t -> customerContactsOldTmp.stream().noneMatch(old -> old.getId().equals(t.getId())))
                    .forEach(customerContactsUpdateReqVO -> {

                        IndirectCustomerContactsCreateReqVO customerContactsCreateReqVO =
                                BeanUtil.copyProperties(customerContactsUpdateReqVO, IndirectCustomerContactsCreateReqVO.class);
                        customerContactsCreateReqVO.setCustomerId(customerId);
                        associateContact(customerId, customerContactsCreateReqVO);
                    });
        }


    }


    private void associateContact(Long customerId, IndirectCustomerContactsCreateReqVO customerContactReq) {
        IndirectCustomerContactsDO customerContactsDO = indirectCustomerContactsMapper.selectOne(new LambdaQueryWrapperX<IndirectCustomerContactsDO>()
                .eq(IndirectCustomerContactsDO::getAreaCode, customerContactReq.getAreaCode())
                .eq(IndirectCustomerContactsDO::getPhoneNew, customerContactReq.getPhoneNew())
                .orderByDesc(IndirectCustomerContactsDO::getId).last("limit 1"));
        if (customerContactsDO != null) {
            customerContactsDO.setCustomerId(customerId);
            //设置是否默认
            customerContactsDO.setIsDefault(customerContactReq.getIsDefault());
            //当请求联系人设置有关系帐号时，则将其设置进去
            if (customerContactReq.getUserid() != null && customerContactReq.getUserid() == null) {
                customerContactsDO.setUserid(customerContactReq.getUserid());
            }
            indirectCustomerContactsMapper.updateById(customerContactsDO);
        } else {
            customerContactsDO = IndirectCustomerContactsConvert.INSTANCE.convert(customerContactReq);
            customerContactsDO.setCustomerId(customerId);
            indirectCustomerContactsMapper.insert(customerContactsDO);
        }
    }


    @Override
    public void deleteIndirectCustomer(Long id) {
        // 校验存在
        this.validateIndirectCustomerExists(id);
        // 删除
        indirectCustomerMapper.deleteById(id);
    }

    private void validateIndirectCustomerExists(Long id) {
        if (indirectCustomerMapper.selectById(id) == null) {
            throw exception(INDIRECT_CUSTOMER_NOT_EXISTS);
        }
    }

    @Override
    public IndirectCustomerDO getIndirectCustomer(Long id) {
        return indirectCustomerMapper.selectById(id);
    }

    @Override
    public List<IndirectCustomerDO> getIndirectCustomerList(Collection<Long> ids) {
        return indirectCustomerMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<IndirectCustomerDO> getIndirectCustomerPage(IndirectCustomerPageReqVO pageReqVO) {
        IPage<IndirectCustomerDO> mpPage = MyBatisUtils.buildPage(pageReqVO);

        LambdaQueryWrapper<IndirectCustomerDO> customerDOLambdaQueryWrapperX = new LambdaQueryWrapperX<IndirectCustomerDO>()
                .likeIfPresent(IndirectCustomerDO::getNumber, pageReqVO.getNumber())
                .likeIfPresent(IndirectCustomerDO::getName, pageReqVO.getName())
                .eqIfPresent(IndirectCustomerDO::getCustomerService, pageReqVO.getCustomerService())
                .eqIfPresent(IndirectCustomerDO::getSource, pageReqVO.getSource())
                .betweenIfPresent(IndirectCustomerDO::getCreateTime, pageReqVO.getBeginCreateTime(), pageReqVO.getEndCreateTime())
                .eq(BaseDO::getDeleted, false)

                //主联系人
                .apply(StringUtils.isNotEmpty(pageReqVO.getDefaultContactName()), "default_contact_name like {0}", String.format("%%%s%%", pageReqVO.getDefaultContactName()))
                //主联系电话
                .apply(StringUtils.isNotEmpty(pageReqVO.getDefaultContactPhone()), "default_contact_phone like {0}", String.format("%%%s%%", pageReqVO.getDefaultContactPhone()))


                .orderByDesc(IndirectCustomerDO::getId);

        //添加获客方式
        if (null != pageReqVO.getMarketType()) {
            if (pageReqVO.getMarketType() == 1) {
                customerDOLambdaQueryWrapperX.apply(" customer_service is not null ");
            }
            if (pageReqVO.getMarketType() == 2) {
                customerDOLambdaQueryWrapperX.apply(" customer_service is null ");
            }

        }

        indirectCustomerMapper.getPage(mpPage, customerDOLambdaQueryWrapperX);

        return new PageResult<>(mpPage.getRecords(), mpPage.getTotal());
    }

    @Override
    public PageResult<IndirectCustomerDO> getIndirectCustomerPageReq(
            IndirectCustomerPageReqDTO pageReqVO, PageVO page){
        IPage<IndirectCustomerDO> mpPage = MyBatisUtils.buildPage(page);
        log.warn(I18nMessage.getLang().toString());


        int start = (page.getPage() - 1) * page.getRows();
        int size = page.getRows();
        List<IndirectCustomerDO> list =
                indirectCustomerMapper.getIndirectCustomerPageReq(start,size,pageReqVO);
        long total = indirectCustomerMapper.getOpenSeaCustomerPageReqCount(pageReqVO) ;
        return new PageResult<>(list, total, mpPage.getSize(),
                page.getPage(), (total + mpPage.getSize() - 1) / mpPage.getSize());
    }

    @Override
    public List<IndirectCustomerDO> getIndirectCustomerListReq(IndirectCustomerPageReqDTO reqVO) {
        return indirectCustomerMapper.getIndirectCustomerListReq(reqVO) ;
    }

    @Override
    public List<IndirectCustomerDO> getIndirectCustomerList(IndirectCustomerExportReqVO reqVO) {
        LambdaQueryWrapper<IndirectCustomerDO> customerDOLambdaQueryWrapperX = new LambdaQueryWrapperX<IndirectCustomerDO>()
                .likeIfPresent(IndirectCustomerDO::getNumber, reqVO.getNumber())
                .likeIfPresent(IndirectCustomerDO::getName, reqVO.getName())
                .eqIfPresent(IndirectCustomerDO::getCustomerService, reqVO.getCustomerService())
                .betweenIfPresent(IndirectCustomerDO::getCreateTime, reqVO.getBeginCreateTime(), reqVO.getEndCreateTime())
                .eq(BaseDO::getDeleted, false)

                //主联系人
                .apply(StringUtils.isNotEmpty(reqVO.getDefaultContactName()), "default_contact_name like {0}", String.format("%%%s%%", reqVO.getDefaultContactName()))
                //主联系电话
                .apply(StringUtils.isNotEmpty(reqVO.getDefaultContactPhone()), "default_contact_phone like {0}", String.format("%%%s%%", reqVO.getDefaultContactPhone()))

                .orderByDesc(IndirectCustomerDO::getId);

        return indirectCustomerMapper.getList(customerDOLambdaQueryWrapperX);
    }
}
