package cn.iocoder.yudao.module.customer.service.indirectCustomerContacts;

import java.util.*;
import javax.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.customer.vo.indirectCustomerContacts.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.indirectCustomerContacts.IndirectCustomerContactsDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.customer.convert.indirectCustomerContacts.IndirectCustomerContactsConvert;
import cn.iocoder.yudao.module.customer.dal.mysql.indirectCustomerContacts.IndirectCustomerContactsMapper;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.customer.enums.ErrorCodeConstants.*;

/**
 * 间接客户联系人 Service 实现类
 *
 * @author yanghao
 */
@Service
@Validated
public class IndirectCustomerContactsServiceImpl extends AbstractService<IndirectCustomerContactsMapper, IndirectCustomerContactsDO> implements IndirectCustomerContactsService {

    @Resource
    private IndirectCustomerContactsMapper indirectCustomerContactsMapper;

    @Override
    public Long createIndirectCustomerContacts(IndirectCustomerContactsCreateReqVO createReqVO) {
        // 插入
        IndirectCustomerContactsDO indirectCustomerContacts = IndirectCustomerContactsConvert.INSTANCE.convert(createReqVO);
        indirectCustomerContactsMapper.insert(indirectCustomerContacts);
        // 返回
        return indirectCustomerContacts.getId();
    }

    @Override
    public void updateIndirectCustomerContacts(IndirectCustomerContactsUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateIndirectCustomerContactsExists(updateReqVO.getId());
        // 更新
        IndirectCustomerContactsDO updateObj = IndirectCustomerContactsConvert.INSTANCE.convert(updateReqVO);
        indirectCustomerContactsMapper.updateById(updateObj);
    }

    @Override
    public void deleteIndirectCustomerContacts(Long id) {
        // 校验存在
        this.validateIndirectCustomerContactsExists(id);
        // 删除
        indirectCustomerContactsMapper.deleteById(id);
    }

    private void validateIndirectCustomerContactsExists(Long id) {
        if (indirectCustomerContactsMapper.selectById(id) == null) {
            throw exception(INDIRECT_CUSTOMER_CONTACTS_NOT_EXISTS);
        }
    }

    @Override
    public IndirectCustomerContactsDO getIndirectCustomerContacts(Long id) {
        return indirectCustomerContactsMapper.selectById(id);
    }

    @Override
    public List<IndirectCustomerContactsDO> getIndirectCustomerContactsList(Collection<Long> ids) {
        return indirectCustomerContactsMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<IndirectCustomerContactsDO> getIndirectCustomerContactsPage(IndirectCustomerContactsQueryVO query, PageVO page) {
        return indirectCustomerContactsMapper.selectPage(page, query);
    }

    @Override
    public List<IndirectCustomerContactsDO> getIndirectCustomerContactsList(IndirectCustomerContactsQueryVO query) {
        return indirectCustomerContactsMapper.selectList(query);
    }
}
