package cn.iocoder.yudao.module.customer.service.zhongPao;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.customer.dal.dataobject.zhongPao.ZhongPaoDO;
import cn.iocoder.yudao.module.customer.vo.zhongPao.*;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 重泡货配置 Service 接口
 *
 * @author yanghao
 */
public interface ZhongPaoService extends IService<ZhongPaoDO> {

    /**
     * 创建重泡货配置
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createZhongPao(@Valid ZhongPaoCreateReqVO createReqVO);

    /**
     * 更新重泡货配置
     * @param updateReqVO 更新信息
     */
    void updateZhongPao(@Valid ZhongPaoUpdateReqVO updateReqVO);

    /**
     * 删除重泡货配置
     * @param id 编号
     */
    void deleteZhongPao(Long id);

    /**
     * 获得重泡货配置
     * @param id 编号
     * @return 重泡货配置
     */
    ZhongPaoDO getZhongPao(Long id);

    /**
     * 获得重泡货配置列表
     * @param ids 编号
     * @return 重泡货配置列表
     */
    List<ZhongPaoDO> getZhongPaoList(Collection<Long> ids);

    /**
     * 获得重泡货配置列表
     * @param customerZhongPaoQueryVO 编号
     * @return 重泡货配置列表
     */
    List<ZhongPaoDO> getZhongPaoList(CustomerZhongPaoQueryVO customerZhongPaoQueryVO);

    /**
     * 获得重泡货配置 起始地（全部）-目的地（全部）的那条数据配置
     *
     * @return 重泡货配置列表
     */
    ZhongPaoBestVO getZhongPaoFullConfig();

    /**
     * 获得重泡货配置分页
     * @param page 分页查询
     * @param query 查询
     * @return 重泡货配置分页
     */
    PageResult<ZhongPaoDO> getZhongPaoPage(ZhongPaoQueryVO query, PageVO page);

    /**
     * 获得重泡货配置列表, 用于 Excel 导出
     * @param query 查询
     * @return 重泡货配置列表
     */
    List<ZhongPaoDO> getZhongPaoList(ZhongPaoQueryVO query);


    /**
     * 获取用户最优惠的重泡货配置
     *
     * @param customerZhongPaoQueryVO 请求
     * @return
     */
    ZhongPaoBestVO getCustomerBestZhongPao(CustomerZhongPaoQueryVO customerZhongPaoQueryVO);

    void matchUser(List<ZhongPaoBackVO> list);
}
