package cn.iocoder.yudao.module.customer.service.zhongPao;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.constant.Constant;
import cn.iocoder.yudao.framework.common.util.spring.enums.CommonStatusEnum;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.customer.convert.zhongPao.ZhongPaoConvert;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.CustomerDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.line.CustomerLineDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.zhongPao.ZhongPaoDO;
import cn.iocoder.yudao.module.customer.dal.mysql.zhongPao.ZhongPaoMapper;
import cn.iocoder.yudao.module.customer.service.customer.CustomerService;
import cn.iocoder.yudao.module.customer.service.customer.line.CustomerLineService;
import cn.iocoder.yudao.module.customer.vo.zhongPao.*;
import cn.iocoder.yudao.module.ecw.enums.ZhongPaoTypeEnum;
import cn.iocoder.yudao.module.system.api.user.AdminUserApi;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.ZHONG_PAO_NOT_EXISTS;

/**
 * 重泡货配置 Service 实现类
 *
 * @author yanghao
 */
@Service
@Validated
public class ZhongPaoServiceImpl extends AbstractService<ZhongPaoMapper, ZhongPaoDO> implements ZhongPaoService {

    @Resource
    private ZhongPaoMapper zhongPaoMapper;

    @Resource
    private CustomerService customerService;

    @Resource
    private CustomerLineService customerLineService;

    @Resource
    AdminUserApi adminUserApi;

    @Override
    public Long createZhongPao(ZhongPaoCreateReqVO createReqVO) {
        // 插入
        ZhongPaoDO zhongPao = ZhongPaoConvert.INSTANCE.convert(createReqVO);
        zhongPaoMapper.insert(zhongPao);
        // 返回
        return zhongPao.getId();
    }

    @Override
    public void updateZhongPao(ZhongPaoUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateZhongPaoExists(updateReqVO.getId());
        // 更新
        ZhongPaoDO updateObj = ZhongPaoConvert.INSTANCE.convert(updateReqVO);
        zhongPaoMapper.updateById(updateObj);
    }

    @Override
    public void deleteZhongPao(Long id) {
        // 校验存在
        this.validateZhongPaoExists(id);
        // 删除
        zhongPaoMapper.deleteById(id);
    }

    private void validateZhongPaoExists(Long id) {
        if (zhongPaoMapper.selectById(id) == null) {
            throw exception(ZHONG_PAO_NOT_EXISTS);
        }
    }

    @Override
    public ZhongPaoDO getZhongPao(Long id) {
        return zhongPaoMapper.selectById(id);
    }

    @Override
    public List<ZhongPaoDO> getZhongPaoList(Collection<Long> ids) {
        return zhongPaoMapper.selectBatchIds(ids);
    }

    @Override
    public List<ZhongPaoDO> getZhongPaoList(CustomerZhongPaoQueryVO customerZhongPaoQueryVO) {
        LambdaQueryWrapper<ZhongPaoDO> wrapper = new LambdaQueryWrapperX<ZhongPaoDO>()
                .eq(true, ZhongPaoDO::getStatus, CommonStatusEnum.ENABLE.getStatus())
                .and(i -> i.eq(customerZhongPaoQueryVO.getDepartureId() != null, ZhongPaoDO::getDepartureId, customerZhongPaoQueryVO.getDepartureId())
                        .or()
                        .eq(ZhongPaoDO::getDepartureId, Constant.DEPARTURE_FULL))
                .and(i -> i.apply(customerZhongPaoQueryVO.getObjectiveId() != null, "FIND_IN_SET ('" + customerZhongPaoQueryVO.getObjectiveId() + "',  objective_ids)")
                        .or()
                        .eq(ZhongPaoDO::getObjectiveIds, Constant.DEPARTURE_FULL));
        return zhongPaoMapper.selectList(wrapper);
    }

    @Override
    public ZhongPaoBestVO getZhongPaoFullConfig() {
        LambdaQueryWrapper<ZhongPaoDO> wrapper = new LambdaQueryWrapperX<ZhongPaoDO>()
                .eq(true, ZhongPaoDO::getStatus, CommonStatusEnum.ENABLE.getStatus())
                .eq(ZhongPaoDO::getDepartureId, Constant.DEPARTURE_FULL)
                .eq(ZhongPaoDO::getObjectiveIds, Constant.OBJECTIVE_FULL);

        List<ZhongPaoDO> zhongPaoDOS = zhongPaoMapper.selectList(wrapper);

        if (!CollectionUtils.isEmpty(zhongPaoDOS)) {

            ZhongPaoBestVO zhongPaoBestVO = new ZhongPaoBestVO();
            zhongPaoDOS.stream().filter(t -> ZhongPaoTypeEnum.ZHONG.getValue().equals(t.getType())).findFirst().ifPresent(zhongPaoDO -> {
                zhongPaoBestVO.setZhongEdge(zhongPaoDO.getEdge());
                zhongPaoBestVO.setZhongFormulas(zhongPaoDO.getFormulas());
            });
            zhongPaoDOS.stream().filter(t -> ZhongPaoTypeEnum.PAO.getValue().equals(t.getType())).findFirst().ifPresent(zhongPaoDO -> {
                zhongPaoBestVO.setPaoEdge(zhongPaoDO.getEdge());
                zhongPaoBestVO.setPaoFormulas(zhongPaoDO.getFormulas());
            });

            return zhongPaoBestVO;
        }

        return null;
    }

    @Override
    public PageResult<ZhongPaoDO> getZhongPaoPage(ZhongPaoQueryVO query, PageVO page) {
        return zhongPaoMapper.selectPage(page, query);
    }

    @Override
    public List<ZhongPaoDO> getZhongPaoList(ZhongPaoQueryVO query) {
        return zhongPaoMapper.selectList(query);
    }

    @Override
    public ZhongPaoBestVO getCustomerBestZhongPao(CustomerZhongPaoQueryVO customerZhongPaoQueryVO) {

        //重货取最大，泡货取最小
        List<ZhongPaoBestVO> zhongPaoBestVOList = new ArrayList<>();

        Long startWarehouseId = customerZhongPaoQueryVO.getStartWarehouseId();
        Long destWarehouseId = customerZhongPaoQueryVO.getDestWarehouseId();

        if (Objects.nonNull(customerZhongPaoQueryVO.getCustomerId())) {
            // 客户有没有配置指定路线
            findCustomerZhongPao(customerZhongPaoQueryVO.getCustomerId(), startWarehouseId, destWarehouseId, zhongPaoBestVOList);
        }
        if (Objects.nonNull(customerZhongPaoQueryVO.getConsignorId())) {
            // 发货客户有没有配置指定路线
            findCustomerZhongPao(customerZhongPaoQueryVO.getConsignorId(), startWarehouseId, destWarehouseId, zhongPaoBestVOList);
        }
        if (Objects.nonNull(customerZhongPaoQueryVO.getConsigneeId())) {
            // 收货客户有没有配置指定路线
            findCustomerZhongPao(customerZhongPaoQueryVO.getConsigneeId(), startWarehouseId, destWarehouseId, zhongPaoBestVOList);
        }
        //2、 全局针对此条件下的配置，包含全局全部配置 （全部起始和全部目的的）
        List<ZhongPaoDO> zhongPaoLineList = this.getZhongPaoList(customerZhongPaoQueryVO);
        if (CollectionUtil.isNotEmpty(zhongPaoLineList)) {

            zhongPaoLineList.stream().filter(t -> ZhongPaoTypeEnum.ZHONG.getValue().equals(t.getType())).max(Comparator.comparing(ZhongPaoDO::getEdge)).ifPresent(zhongPaoDO -> {
                ZhongPaoBestVO lineZhongPaoBestVO = new ZhongPaoBestVO();
                lineZhongPaoBestVO.setZhongEdge(zhongPaoDO.getEdge());
                lineZhongPaoBestVO.setZhongFormulas(zhongPaoDO.getFormulas());
                lineZhongPaoBestVO.setPaoEdge(null);
                lineZhongPaoBestVO.setPaoFormulas(null);
                zhongPaoBestVOList.add(lineZhongPaoBestVO);
            });
            zhongPaoLineList.stream().filter(t -> ZhongPaoTypeEnum.PAO.getValue().equals(t.getType())).min(Comparator.comparing(ZhongPaoDO::getEdge)).ifPresent(zhongPaoDO -> {
                ZhongPaoBestVO lineZhongPaoBestVO = new ZhongPaoBestVO();
                lineZhongPaoBestVO.setZhongEdge(null);
                lineZhongPaoBestVO.setZhongFormulas(null);
                lineZhongPaoBestVO.setPaoEdge(zhongPaoDO.getEdge());
                lineZhongPaoBestVO.setPaoFormulas(zhongPaoDO.getFormulas());
                zhongPaoBestVOList.add(lineZhongPaoBestVO);
            });
        }

        // 找到最优的配置返回
        ZhongPaoBestVO result = new ZhongPaoBestVO();
        if (CollectionUtil.isNotEmpty(zhongPaoBestVOList)) {

            ZhongPaoBestVO zhongBestVO = zhongPaoBestVOList.stream().filter(t -> t.getZhongEdge() != null).max(Comparator.comparing(ZhongPaoBestVO::getZhongEdge)).orElse(null);
            if (zhongBestVO != null) {
                result.setZhongEdge(zhongBestVO.getZhongEdge());
                result.setZhongFormulas(zhongBestVO.getZhongFormulas());
            }
            ZhongPaoBestVO paoBestVO = zhongPaoBestVOList.stream().filter(t -> t.getPaoEdge() != null).min(Comparator.comparing(ZhongPaoBestVO::getPaoEdge)).orElse(null);
            if (paoBestVO != null) {
                result.setPaoEdge(paoBestVO.getPaoEdge());
                result.setPaoFormulas(paoBestVO.getPaoFormulas());
            }
        }

        return result;
    }

    private void findCustomerZhongPao(Long customerId, Long startWarehouseId, Long destWarehouseId, List<ZhongPaoBestVO> zhongPaoList) {
        // 客户有没有配置指定路线
        CustomerZhongPaoQueryVO vo = new CustomerZhongPaoQueryVO();
        vo.setCustomerId(customerId);
        List<CustomerLineDO> customerLineList = customerLineService.getCustomerLineList(vo);

        boolean zhongMatch = isCustomerLineZhongPaoMatch(customerLineList, ZhongPaoTypeEnum.ZHONG, startWarehouseId, destWarehouseId);
        boolean paoMatch = isCustomerLineZhongPaoMatch(customerLineList, ZhongPaoTypeEnum.PAO, startWarehouseId, destWarehouseId);

        if (zhongMatch || paoMatch) {
            CustomerDO customer = customerService.getCustomer(customerId);
            if (customer != null) {
                if (zhongMatch && customer.getWeightUnit() != null) {
                    ZhongPaoBestVO zhongPaoBestVO = new ZhongPaoBestVO();
                    zhongPaoBestVO.setZhongEdge(customer.getWeightUnit());
                    zhongPaoBestVO.setZhongFormulas(BigDecimal.ONE.divide(customer.getWeightUnit(), 6, RoundingMode.HALF_UP));
                    zhongPaoBestVO.setPaoFormulas(null);
                    zhongPaoBestVO.setPaoEdge(null);
                    zhongPaoList.add(zhongPaoBestVO);
                }
                if (paoMatch && customer.getLightUnit() != null) {
                    ZhongPaoBestVO zhongPaoBestVO = new ZhongPaoBestVO();
                    zhongPaoBestVO.setZhongEdge(null);
                    zhongPaoBestVO.setZhongFormulas(null);
                    zhongPaoBestVO.setPaoEdge(customer.getLightUnit());
                    zhongPaoBestVO.setPaoFormulas(customer.getLightUnit());
                    zhongPaoList.add(zhongPaoBestVO);
                }
            }
        }
    }

    private boolean isCustomerLineZhongPaoMatch(List<CustomerLineDO> customerLineList, ZhongPaoTypeEnum zhongPaoTypeEnum, Long startWarehouseId, Long destWarehouseId) {
        boolean match = false;
        //没有指定重货线路的情况
        if (customerLineList.stream().noneMatch(t -> t.getZhongPaoType() == zhongPaoTypeEnum.getValue())) {
            match = true;
        }
        // 有指定重货标准的线路
        else if (customerLineList.stream().anyMatch(t -> t.getZhongPaoType() == zhongPaoTypeEnum.getValue()
                && t.getDepartureId().equals(startWarehouseId)
                && StringUtils.isNotEmpty(t.getObjectiveIds())
                && Arrays.stream(t.getObjectiveIds().split(",")).anyMatch(o -> Long.valueOf(o).equals(destWarehouseId)))) {
            match = true;
        }
        return match;
    }

    @Override
    public void matchUser(List<ZhongPaoBackVO> list) {
        if(CollectionUtil.isNotEmpty(list)) {
            for (ZhongPaoBackVO zhongPaoBackVO : list) {
                String creator = zhongPaoBackVO.getCreator();
                String updater = zhongPaoBackVO.getUpdater();
                zhongPaoBackVO.setCreatorName(getUserName(creator));
                zhongPaoBackVO.setUpdaterName(getUserName(updater));
            }
        }
    }

    private String getUserName(String userId) {
        if (StringUtils.isEmpty(userId)) {
            return null;
        }
        return adminUserApi.getUser(Long.valueOf(userId)).getNickname();
    }
}
