package cn.iocoder.yudao.module.customer.vo.customer.vo;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.ToString;
import org.springframework.format.annotation.DateTimeFormat;

import java.math.BigDecimal;
import java.util.Date;

import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND;

@ApiModel("管理后台 - 客户 Response VO")
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
public class CustomerRespVO extends CustomerBaseVO {

    @ApiModelProperty(value = "id")
    private Long id;

    @ApiModelProperty(value = "客户编号")
    private String number;

    @ApiModelProperty(value = "创建人id")
    private Long founder;


    /**
     * 公司名称
     */
    @ApiModelProperty(value = "公司名称")
    private String company;
    /**
     * 公司名称英文
     */
    @ApiModelProperty(value = "公司名称英文")
    private String companyEn;

    @ApiModelProperty(value = "创建时间", required = true)
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private Date createTime;

    @ApiModelProperty(value = "客户货运类型, 参见customer_transport_type数据字典")
    private String transportType;

    @ApiModelProperty(value = "客户资源类型, 参见customer_resource_type数据字典")
    private Integer resourceType;

    @ApiModelProperty(value = "是否在公海")
    private Boolean isInOpenSea;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "进入公海时间")
    private Date enterOpenSeaTime;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "预计进入公海时间")
    private Date estimateEnterOpenSeaTime;
    /**
     * 跟进客服分配时间
     */
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "跟进客服分配时间")
    private Date customerServiceAssignedTime;

    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "公海池捞取时间")
    private Date catchTime;

    @ApiModelProperty(value = "主联系人Id")
    private Long defaultContactId ;

    @ApiModelProperty(value = "主联系人")
    private String defaultContactName;

    @ApiModelProperty(value = "主联系电话")
    private String defaultContactPhone;

    @ApiModelProperty(value = "主联系电话(不加区号)")
    private String contactPhone ;


    @ApiModelProperty(value = "主邮箱")
    private String defaultEmail;

    @ApiModelProperty(value = "主社交方式")
    private Integer defaultSocial;

    @ApiModelProperty(value = "主社交软件号码")
    private String defaultSocialNumber;

    @ApiModelProperty(value = "付款人")
    private String payerName;

    @ApiModelProperty(value = "是否被接收")
    private Boolean isCustomerServiceConfirmed;





    /**
     * 业务国家 逗号分隔
     */
    @ApiModelProperty(value = "业务国家 逗号分隔")
    private String busiCountryIds;

    /**
     * 常用提货网点
     */
    @ApiModelProperty(value = "常用提货网点")
    private String pickupPoints;


    /**
     * 竞争对手ID, 关联 customer_competitor 表主键
     */
    @ApiModelProperty(value = "竞争对手ID")
    private String competitorIds;

    /**
     * 竞争对手名称，多个以逗号分隔 可直接展示
     */
    @ApiModelProperty(value = "竞争对手名称，多个以逗号分隔 可直接展示")
    private String competitorNames;

    /**
     * 年度发货量
     */
    @ApiModelProperty(value = "年度发货量")
    private BigDecimal weightYearly;

    /**
     * 年度发货次数
     */
    @ApiModelProperty(value = "年度发货次数")
    private Integer numYearly;
    /**
     * 到仓确认
     */
    @ApiModelProperty(value = "到仓确认 1-是，0-否")
    private Integer arrivalConfirm;


    /**
     * 控货无收货人,1-是，0-否
     */
    @ApiModelProperty(value = "控货无收货人,1-是，0-否")
    private Boolean noConsignee ;
    /**
     * 默认付款，1-是，0-否
     */
    @ApiModelProperty(value = "默认付款，1-是，0-否")
    private Boolean defaultPay ;

    /**
     * 归属时间
     */
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "归属时间")
    private Date customerServiceConfirmedTime;

    /**
     * 首次成交时间
     */
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "首次成交时间")
    private Date firstDealTime;



    /**
     * 获取方式
     * <p>
     * 枚举 {@link TODO customer_get_method 字典}
     */
    @ApiModelProperty(value = "获取方式 customer_get_method 字典")
    private Integer getMethod;

    @ApiModelProperty(value = "创建人")
    private String creator;
    @ApiModelProperty(value = "创建人姓名")
    private String createUsername;

    /**
     * 最后更新时间
     */
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    @ApiModelProperty(value = "最后更新时间")
    private Date updateTime;


}
