package cn.iocoder.yudao.module.customer.vo.customer.vo;

import cn.iocoder.yudao.module.customer.vo.customer.customerContacts.CustomerContactsUpdateReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.line.CustomerLineUpdateReqVO;
import cn.iocoder.yudao.module.customer.vo.customerBank.CustomerBankUpdateReqVO;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.ToString;
import org.hibernate.validator.constraints.Length;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

@ApiModel("管理后台 - 客户更新 Request VO")
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
public class CustomerUpdateReqVO extends CustomerBaseVO {

    @ApiModelProperty(value = "主键", required = true)
    @NotNull(message = "主键不能为空")
    private Long id;

    @ApiModelProperty(value = "客户编号")
    private String number;

    @ApiModelProperty(value = "客户类型")
    private String type;

    @ApiModelProperty(value = "客户货运类型", notes = "参见customer_transport_type数据字典")
    private String transportType;

    @ApiModelProperty(value = "客户资源类型", notes = "参见customer_resource_type数据字典")
    private Integer resourceType;

    @ApiModelProperty(value = "所属代理")
    private Long agentId;

    @ApiModelProperty(value = "公司名称")
    private String company;

    @ApiModelProperty(value = "公司英文名称")
    private String companyEn;

    @ApiModelProperty(value = "联系地址")
    private String address;

    @ApiModelProperty(value = "产品类型")
    private Long productType;

    @ApiModelProperty(value = "产品ids")
    private String productIds;

    @ApiModelProperty(value = "常用提货网点ids")
    private String pickupPoints;

    @ApiModelProperty(value = "关联会员账号")
    private Long memberId;

    @ApiModelProperty(value = "客户生日", example = "1652017501000")
    private Date birthday;

    @ApiModelProperty(value = "结算方式")
    private Integer balance;

    @ApiModelProperty(value = "客户来源")
    private Integer source;

    @ApiModelProperty(value = "图片")
    private String picture;

    @ApiModelProperty(value = "推介人id")
    private Long promoter;

    @ApiModelProperty(value = "创建人id")
    private Long founder;


    //--------发票信息
    @ApiModelProperty(value = "发票抬头")
    private String invoiceTitle;

    @ApiModelProperty(value = "纳税人识别号")
    private String licenseNumber;

    @ApiModelProperty(value = "开户行")
    private String bank;

    @ApiModelProperty(value = "账户")
    private String bankNumber;

    @ApiModelProperty(value = "项目")
    private String project;

    @ApiModelProperty(value = "开票地址")
    private String billingAddress;

    @ApiModelProperty(value = "开票电话")
    @Length(max = 20, message = "开票电话最大长度不能超过20")
    private String billingTell;

    @ApiModelProperty(value = "税率")
    private BigDecimal taxRate;

    @ApiModelProperty(value = "备注")
    @Length(max = 255, message = "备注最大长度不能超过255")
    private String remarks;


    /*
     处理开启到仓确认，就报发货人异常的BUG
     */
    @ApiModelProperty(value = "到仓确认")
    private Integer arrivalConfirm;

    @ApiModelProperty(value = "重货标准(CBM)")
    private BigDecimal weightUnit;

    @ApiModelProperty(value = "泡货标准(CBM)")
    private BigDecimal lightUnit;


    @ApiModelProperty(value = "询价详情")
    private String enquiryInfo;


    /**
     * 是否显示提单价格
     */
    @ApiModelProperty(value = "是否显示提单价格")
    private Boolean isShowTidanPrice;


    /**
     * 联系人
     */
    @ApiModelProperty(value = "联系人列表", notes = "参见CustomerContactsCreateReqVO")
    @Valid
    private List<CustomerContactsUpdateReqVO> customerContacts;


    /**
     * 客户线路集合
     */
    @ApiModelProperty(value = "客户线路集合", notes = "参见CustomerLineCreateReqVO")
    @Valid
    private List<CustomerLineUpdateReqVO> customerLines;



    /**
     * 客户银行账号
     */
    @ApiModelProperty(value = "客户银行账户", notes = "参见CustomerBankUpdateReqVO")
    private List<CustomerBankUpdateReqVO> customerBanks;



    /**
     * 跟进客户是否确认接收
     */
    @ApiModelProperty(value = "客户是否确认接收")
    private Boolean isCustomerServiceConfirmed;


    /**
     * 询盘信息
     */
    @ApiModelProperty(value = "询盘信息")
    private String inquiry;


    /**
     * 是否潜在客户
     */
    @ApiModelProperty(value = "是否潜在客户")
    private Boolean isPotential = false;


    // ======== 20240821 add start
    /**
     * 客户角色
     * <p>
     * 枚举 {@link TODO customer_role 字典}
     */
    @ApiModelProperty(value = "客户角色")
    private String roles;


    /**
     * 业务国家 逗号分隔
     */
    @ApiModelProperty(value = "业务国家ids")
    private String busiCountryIds;


    /**
     * 竞争对手IDs, 关联 customer_competitor 表主键
     */
    @ApiModelProperty(value = "竞争对手IDs")
    private String competitorIds;

    /**
     * 年度发货量
     */
    @ApiModelProperty(value = "年度发货量")
    private BigDecimal weightYearly;

    /**
     * 年度发货次数
     */
    @ApiModelProperty(value = "年度发货次数")
    private Integer numYearly;


    /**
     * 默认开票，1-是，0-否
     */
    @ApiModelProperty(value = "默认开票，1-是，0-否")
    private Boolean defaultBilling;

    // ======== 20240821 add end

}
