package cn.iocoder.yudao.module.customer.vo.indirectCustomer;

import lombok.*;
import java.util.*;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import org.springframework.format.annotation.DateTimeFormat;

import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND;

/**
* 间接客户 Base VO，提供给添加、修改、详细的子 VO 使用
* 如果子 VO 存在差异的字段，请不要添加到这里，影响 Swagger 文档生成
*/
@Data
public class IndirectCustomerBaseVO {

    @ApiModelProperty(value = "客户编号")
    private String number;

    @ApiModelProperty(value = "客户名称", required = true)
    @NotNull(message = "客户名称不能为空")
    private String name;

    @ApiModelProperty(value = "客户等级")
    private Integer level;

    @ApiModelProperty(value = "信用等级")
    private Integer creditLevel;

    @ApiModelProperty(value = "国家")
    private Long country;

    @ApiModelProperty(value = "客户类型")
    private String type;

    @ApiModelProperty(value = "客户货运类型 见customer_transport_type")
    private String transportType;

    @ApiModelProperty(value = "客户资源类型 见customer_resource_type")
    private Integer resourceType;

    @ApiModelProperty(value = "所属代理")
    private Long agentId;

    @ApiModelProperty(value = "公司名称")
    private String company;

    @ApiModelProperty(value = "联系地址")
    private String address;

    @ApiModelProperty(value = "产品类型")
    private Long productType;

    @ApiModelProperty(value = "产品id")
    private Long productId;

    @ApiModelProperty(value = "常用提货网点")
    private Long pickupPoint;

    @ApiModelProperty(value = "关联会员账号")
    private Long memberId;

    @ApiModelProperty(value = "客户生日")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private Date birthday;

    @ApiModelProperty(value = "结算方式")
    private Integer balance;

    @ApiModelProperty(value = "客户来源")
    private Integer source;

    @ApiModelProperty(value = "图片")
    private String picture;

    @ApiModelProperty(value = "跟进客服")
    private Long customerService;

    @ApiModelProperty(value = "跟进客服分配时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private Date customerServiceAssignedTime;

    @ApiModelProperty(value = "推介人")
    private Long promoter;

    @ApiModelProperty(value = "客户状态：1.线索2.意向3.成交")
    private Integer status;

    @ApiModelProperty(value = "客户状态更新时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private Date statusUpdateTime;

    @ApiModelProperty(value = "是否被接收", required = true)
    @NotNull(message = "是否被接收不能为空")
    private Boolean isCustomerServiceConfirmed;

    @ApiModelProperty(value = "创建人")
    private Long founder;

    @ApiModelProperty(value = "部门")
    private String department;

    @ApiModelProperty(value = "发票抬头")
    private String invoiceTitle;

    @ApiModelProperty(value = "纳税人识别号")
    private String licenseNumber;

    @ApiModelProperty(value = "开户行")
    private String bank;

    @ApiModelProperty(value = "账户")
    private String bankNumber;

    @ApiModelProperty(value = "项目")
    private String project;

    @ApiModelProperty(value = "开票地址")
    private String billingAddress;

    @ApiModelProperty(value = "开票电话")
    private String billingTell;

    @ApiModelProperty(value = "税率")
    private java.math.BigDecimal taxRate;

    @ApiModelProperty(value = "完善时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private Date perfectTime;

    @ApiModelProperty(value = "备注")
    private String remarks;

    @ApiModelProperty(value = "管理员id")
    private Long adminId;

    @ApiModelProperty(value = "审批人id")
    private Long approver;

    @ApiModelProperty(value = "通知人id")
    private Long notifier;

    @ApiModelProperty(value = "其他审批id")
    private Long otherApprovalId;

    @ApiModelProperty(value = "到仓确认")
    private Integer arrivalConfirm;

    @ApiModelProperty(value = "重货标准(CBM)")
    private java.math.BigDecimal weightUnit;

    @ApiModelProperty(value = "泡货标准")
    private java.math.BigDecimal lightUnit;

    @ApiModelProperty(value = "重货标准修改人")
    private String weightEditWho;

    @ApiModelProperty(value = "重货标准修时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private Date weightEditTime;

    @ApiModelProperty(value = "询价详情")
    private String enquiryInfo;

    @ApiModelProperty(value = "是否系统创建 0为手动 1为注册时系统创建 2为下单时系统创建")
    private Integer isAuto;

    @ApiModelProperty(value = "是否公海客户")
    private Boolean isInOpenSea;

    @ApiModelProperty(value = "进入公海时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private Date enterOpenSeaTime;

    @ApiModelProperty(value = "预计进入公海池时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private Date estimateEnterOpenSeaTime;

    @ApiModelProperty(value = "延期x月")
    private Integer postponeMonth;

    @ApiModelProperty(value = "从公海池捞取时间")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private Date catchTime;

    @ApiModelProperty(value = "是否显示提单价格")
    private Boolean isShowTidanPrice;

    @ApiModelProperty(value = "vip等级分数")
    private Long vipLevelScore;

    @ApiModelProperty(value = "信用等级分数")
    private Long creditLevelScore;

}
