package cn.iocoder.yudao.module.customer.controller.admin.customer;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.exception.ErrorCode;
import cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.dict.core.util.DictFrameworkUtils;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.framework.web.core.util.WebFrameworkUtils;
import cn.iocoder.yudao.module.customer.convert.customer.CustomerConvert;
import cn.iocoder.yudao.module.customer.convert.customerBank.CustomerBankConvert;
import cn.iocoder.yudao.module.customer.convert.customerContacts.CustomerContactsConvert;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.CustomerDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.commission.CustomerCommissionDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.complaint.CustomerComplaintDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.follow.CustomerFollowDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerBank.CustomerBankDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerContacts.CustomerContactsDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerLevelLog.CustomerLevelLogDO;
import cn.iocoder.yudao.module.customer.dal.mysql.customerDetail.CustomerDetailMapper;
import cn.iocoder.yudao.module.customer.dto.CustomerDelayApprovalInfoDto;
import cn.iocoder.yudao.module.customer.dto.CustomerExportReqDTO;
import cn.iocoder.yudao.module.customer.dto.CustomerHandoverApprovalInfoDto;
import cn.iocoder.yudao.module.customer.dto.CustomerPageReqDTO;
import cn.iocoder.yudao.module.customer.service.customer.CustomerService;
import cn.iocoder.yudao.module.customer.service.customer.commission.CustomerCommissionService;
import cn.iocoder.yudao.module.customer.service.customer.complaint.CustomerComplaintService;
import cn.iocoder.yudao.module.customer.service.customer.complaint.follow.CustomerFollowService;
import cn.iocoder.yudao.module.customer.service.customerBank.CustomerBankService;
import cn.iocoder.yudao.module.customer.service.customerContacts.CustomerContactsService;
import cn.iocoder.yudao.module.customer.service.customerCredit.CustomerCreditService;
import cn.iocoder.yudao.module.customer.service.customerLevel.CustomerLevelService;
import cn.iocoder.yudao.module.customer.service.customerLevelLog.CustomerLevelLogServiceImpl;
import cn.iocoder.yudao.module.customer.service.customerPublicCatchLog.CustomerPublicCatchLogService;
import cn.iocoder.yudao.module.customer.service.zhongPao.ZhongPaoService;
import cn.iocoder.yudao.module.customer.vo.customer.customerContacts.CustomerContactsCreateReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.customerContacts.CustomerContactsRespVO;
import cn.iocoder.yudao.module.customer.vo.customer.customerContacts.CustomerContactsUpdateReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.vo.*;
import cn.iocoder.yudao.module.customer.vo.customerBank.CustomerBankBackVO;
import cn.iocoder.yudao.module.customer.vo.customerBank.CustomerBankQueryVO;
import cn.iocoder.yudao.module.ecw.api.paramValid.ParamValidatorApi;
import cn.iocoder.yudao.module.ecw.enums.CustomerContactsDefaultEnum;
import cn.iocoder.yudao.module.ecw.enums.CustomerCreateFromEnum;
import cn.iocoder.yudao.module.ecw.enums.CustomerOperateTypeEnum;
import cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants;
import cn.iocoder.yudao.module.ecw.service.busiPwd.BusiPwdService;
import cn.iocoder.yudao.module.ecw.service.country.CountryService;
import cn.iocoder.yudao.module.ecw.service.internalMessage.InternalMessageService;
import cn.iocoder.yudao.module.ecw.service.paramValid.ParamValidatorService;
import cn.iocoder.yudao.module.member.api.user.MemberUserApi;
import cn.iocoder.yudao.module.member.api.user.dto.UserRespDTO;
import cn.iocoder.yudao.module.product.dal.dataobject.productbrandempower.ProductBrandEmpowerDO;
import cn.iocoder.yudao.module.product.service.productbrandempower.ProductBrandEmpowerServiceImpl;
import cn.iocoder.yudao.module.sale.dal.dataobject.offer.OfferDO;
import cn.iocoder.yudao.module.sale.service.offer.OfferService;
import cn.iocoder.yudao.module.system.api.file.FileMakeApi;
import cn.iocoder.yudao.module.system.api.file.dto.FileMakeReqDTO;
import cn.iocoder.yudao.module.system.api.user.AdminUserApi;
import cn.iocoder.yudao.module.system.api.user.dto.AdminUserRespDTO;
import cn.iocoder.yudao.module.system.enums.download.DownloadTypeEnum;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receipt.ReceiptDO;
import cn.iocoder.yudao.module.wealth.service.receipt.ReceiptServiceImpl;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.jetbrains.annotations.NotNull;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.CUSTOMER_NOT_EXISTS;

@Slf4j
@Api(tags = "管理后台 - 客户")
@RestController
@RequestMapping("/ecw/customer")
@Validated
public class CustomerController {

    @Resource
    private CustomerService customerService;

    @Resource
    private CustomerBankService customerBankService;

    @Resource
    private CustomerPublicCatchLogService publicCatchLogService;

    @Resource
    private AdminUserApi adminUserApi;

    @Resource
    private CountryService countryService;

    @Resource
    private CustomerContactsService customerContactsService;

    @Resource
    private CustomerCommissionService customerCommissionService;

    @Resource
    private ZhongPaoService zhongPaoService;

    @Resource
    private BusiPwdService busiPwdService;

    @Resource
    private MemberUserApi memberUserApi;

    @Resource
    private ParamValidatorService paramValidatorService;
    @Resource
    private InternalMessageService internalMessageService;

    @Resource
    private CustomerLevelService customerLevelService;

    @Resource
    private CustomerCreditService customerCreditService;
    @Resource
    private ParamValidatorApi paramValidatorApi;

    @Resource
    private FileMakeApi fileMakeApi;

    //客户详情信息
    @Resource
    CustomerDetailMapper customerDetailMapper;

    @Resource
    OfferService offerService;

    //客户跟进
    @Resource
    CustomerFollowService customerFollowService;

    @Resource
    private CustomerComplaintService customerComplaintService;


    @Resource
    private ReceiptServiceImpl receiptService;

    @Resource
    private ProductBrandEmpowerServiceImpl productBrandEmpowerService;

    //客户登记日志
    @Resource
    private CustomerLevelLogServiceImpl customerLevelLogService;

    public CustomerController() {
    }

    @PostMapping("/create-potential")
    @ApiOperation("创建潜在客户")
    @Idempotent(timeout = 5)
    // @PreAuthorize("@ss.hasAnyPermissions('ecw:customer:create','ecw:customer:my-create','ecw:customer: treat-create','ecw:customer:distribution-create','ecw:customer:dep-create')")
    public CommonResult<Long> createPotential(@Valid @RequestBody CustomerCreateReqVO createReqVO, HttpServletRequest request) {

        createReqVO.setIsPotential(true);
        createReqVO.setCustomerOperateLogRemark("新增潜在客户");

        return createCustomerPrivate(createReqVO);
    }

    @PostMapping("/create")
    @ApiOperation("创建客户")
    @Idempotent(timeout = 5)
    // @PreAuthorize("@ss.hasAnyPermissions('ecw:customer:create','ecw:customer:my-create','ecw:customer: treat-create','ecw:customer:distribution-create','ecw:customer:dep-create')")
    public CommonResult<Long> createCustomer(
            @Valid @RequestBody CustomerCreateReqVO createReqVO,
            HttpServletRequest request) {
        createReqVO.setCustomerOperateLogRemark("管理端页面新建客户");
        return createCustomerPrivate(createReqVO);
    }

    @NotNull
    private CommonResult<Long> createCustomerPrivate(CustomerCreateReqVO createReqVO) {
        List<CustomerContactsCreateReqVO> customerContacts =
                createReqVO.getCustomerContacts();
        if (!CollectionUtils.isEmpty(customerContacts)) {
            long count = customerContacts.stream().filter(t -> CustomerContactsDefaultEnum.IS_DEFAULT.getValue().equals(t.getIsDefault())).count();
            if (count <= 0) {
                return error(ErrorCodeConstants.CUSTOMER_CONTACTS_NO_DEFAULT);
            }
            if (count > 1) {
                return error(ErrorCodeConstants.CUSTOMER_CONTACTS_MULTI_DEFAULT);
            }

            //判断联系人手机已绑定其他客户，则提示
            for (CustomerContactsCreateReqVO customerContact : customerContacts) {
                String phone = customerContact.getPhoneNew();
                String mobileCode = customerContact.getAreaCode() + StrUtil.COLON + phone;
                paramValidatorApi.validatorMobile(mobileCode);

                CustomerContactsDO customerContactsDO = customerContactsService.getUniqueOneAndValidate(customerContact.getAreaCode(), phone, null);
                if (customerContactsDO != null) {
                    throw exception(ErrorCodeConstants.CUSTOMER_CONTACTS_ALREADY_BIND, phone);
                }
            }

            //验证输入性重复
            Set<String> set = new HashSet<>();
            for (CustomerContactsCreateReqVO customerContact : customerContacts) {
                String phoneNew = customerContact.getPhoneNew();
                String phoneStandard = customerContact.getAreaCode() + StrUtil.COLON + (phoneNew.startsWith("0") ? phoneNew.substring(1) : phoneNew);
                if (set.stream().anyMatch(t -> t.equals(phoneStandard))) {
                    throw exception(ErrorCodeConstants.CUSTOMER_CONTACTS_PHONE_REPEATED, phoneNew);
                } else {
                    set.add(phoneStandard);
                }
            }

        }
        // 后台创建
        createReqVO.setCreateFrom(CustomerCreateFromEnum.BACKEND.getValue());

        return success(customerService.createCustomer(createReqVO));
    }


    @PutMapping("/update")
    @ApiOperation("更新客户")
    @PreAuthorize("@ss.hasAnyPermissions('ecw:customer:update', 'ecw:customer:dep-update', 'ecw:customer:my-update', 'ecw:customer:distribution-update')")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> updateCustomer(
            @Valid @RequestBody CustomerUpdateReqVO updateReqVO) {

        //SELECT arrival_confirm from ecw_customer  到仓确认
        List<CustomerContactsUpdateReqVO> customerContacts =
                updateReqVO.getCustomerContacts();
        if (!CollectionUtils.isEmpty(customerContacts)) {
            long count = customerContacts.stream().filter(t -> CustomerContactsDefaultEnum.IS_DEFAULT.getValue().equals(t.getIsDefault())).count();
            if (count <= 0) {
                return error(ErrorCodeConstants.CUSTOMER_CONTACTS_NO_DEFAULT);
            }
            if (count > 1) {
                return error(ErrorCodeConstants.CUSTOMER_CONTACTS_MULTI_DEFAULT);
            }

            //判断联系人的userid是否已绑定其他客户
            for (CustomerContactsUpdateReqVO customerContact : customerContacts) {
                Long userid = customerContact.getUserid();
                if (userid != null && userid != 0) { //老数据导入过来后部分userid=0
                    count = customerContactsService.count(
                            new LambdaQueryWrapperX<CustomerContactsDO>().eq(CustomerContactsDO::getUserid, userid)
                            .ne(CustomerContactsDO::getCustomerId, updateReqVO.getId()).eq(BaseDO::getDeleted, 0));
                    if (count > 0) {
                        UserRespDTO user = memberUserApi.getUser(userid);
                        throw exception(ErrorCodeConstants.CUSTOMER_CONTACTS_USERID_ALREADY_BIND, user != null ? user.getNickname() : "", user != null ? user.getMobile() : "");
                    }
                }
            }

            //判断联系人手机已绑定其他客户，则提示
            for (CustomerContactsUpdateReqVO customerContact : customerContacts) {

                String phone = customerContact.getPhoneNew();
                String mobileCode = customerContact.getAreaCode() + StrUtil.COLON + phone;
                paramValidatorApi.validatorMobile(mobileCode);

                CustomerContactsDO customerContactsDO = customerContactsService.getUniqueOneAndValidate(customerContact.getAreaCode(), phone, updateReqVO.getId());

                if (customerContactsDO != null) {
                    throw exception(ErrorCodeConstants.CUSTOMER_CONTACTS_ALREADY_BIND, phone);
                }
            }

            //验证输入性重复
            Set<String> set = new HashSet<>();
            for (CustomerContactsUpdateReqVO customerContact : customerContacts) {
                String phoneNew = customerContact.getPhoneNew();
                String phoneStandard = customerContact.getAreaCode() + StrUtil.COLON + (phoneNew.startsWith("0") ? phoneNew.substring(1) : phoneNew);
                if (set.stream().anyMatch(t -> t.equals(phoneStandard))) {
                    throw exception(ErrorCodeConstants.CUSTOMER_CONTACTS_PHONE_REPEATED, phoneNew);
                } else {
                    set.add(phoneStandard);
                }
            }
        }

        Long promoterNew = updateReqVO.getPromoter();
        if (promoterNew != null) {
            //若客户A已设置佣金规则，修改推介人时，推介人有佣金规则，不能客户修改推介人，提示“该推介人已设置佣金，客户与推介人只能有一个设置佣金规则”。
            //1、查看此客户自己是否已设置佣金规则
            Long customerId = updateReqVO.getId();
            CustomerCommissionDO customerCommission = customerCommissionService.getCustomerCommissionByCustomerId(customerId);

            if (customerCommission != null) {
                CustomerDO customerOriginal = customerService.getById(customerId);

                //若自己设置了佣金，然后看是否原customerService跟请求的customerService是否一致。一致说明没有更新，不一致说明有更新。
                //有更新customerService的话，则需要判断新的推介人是否有设备规则
                Long promoterOld = customerOriginal.getPromoter();
                if (!promoterNew.equals(promoterOld)) {

                    //查询新推介人的佣金规则
                    CustomerCommissionDO customerCommissionByCustomerId = customerCommissionService.getCustomerCommissionByCustomerId(promoterNew);
                    if (customerCommissionByCustomerId != null) {
                        return error(ErrorCodeConstants.CUSTOMER_SERVICE_COMMISSION_EXISTS);
                    }

                }

            }
        }

        this.customerService.updateCustomer(updateReqVO);
        return success(true);
    }

    @PutMapping("/fillup-customer-info")
    @ApiOperation("完善信息")
    @PreAuthorize("@ss.hasAnyPermissions('ecw:customer:update', 'ecw:customer:indirect:perfect', 'ecw:customer:treat-perfect')")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> fillupCustomerInfo(@Valid @RequestBody CustomerUpdateReqVO updateReqVO) {

        //完善信息要确认接收
        updateReqVO.setIsCustomerServiceConfirmed(true);

        //完善客户信息确认接收时要计算客户的业绩类型
        return this.updateCustomer(updateReqVO);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除客户")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasAnyPermissions('ecw:customer:delete', 'ecw:customer:dep-delete', 'ecw:customer:my-delete', 'ecw:customer:distribution-delete')")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> deleteCustomer(@RequestParam("id") Long id) {
        customerService.deleteCustomer(id);
        return success(true);
    }


    @GetMapping("/mergeCus")
    @ApiOperation("合并客户信息")
    public CommonResult<Boolean> mergeCus(
            @Valid CustomerMerge par) {
        //1. 保留客户和非主客户，不能在两个不同的客户经理名下，
        // 请先确认两个客户在同一个客户经理名下
        //2.  被合并客户，提交后会直接删除
        //3. 合并后，非主客户的联系人，跟进记录，报价单，订单，
        // 客户投诉，品牌授权都迁移到保留客户中，其他信息不会迁移，
        // 如需要维护非主客户的客户档案信息到保留客户中，请先维护好再操作

        if (par.getId1() == null) {
            return success(false);
        }
        if (par.getId2() == null) {
            return success(false);
        }
        CustomerDO customerDO1 =
                customerService.getById(par.getId1());
        if (customerDO1 == null) {
            throw exception(new ErrorCode(0, "保留客户未查询到"));

        }
        if (customerDO1.getCustomerService() == null) {
            throw exception(new ErrorCode(0, "保留客户的客户经理为空"));
        }
        CustomerDO customerDO2 =
                customerService.getById(par.getId2());
        if (customerDO2 == null) {
            throw exception(new ErrorCode(0, "被合并客户未查询到"));
        }

        if (customerDO2.getCustomerService() == null) {
            throw exception(new ErrorCode(0, "被合并客户客户经理为空"));
        }
        if (customerDO1.getCustomerService() != customerDO2.getCustomerService()) {
            throw exception(new ErrorCode(0,
                    "保留客户和被合并客户客户经理不相同"));
        }

        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        if (doMergeCus(customerDO1, customerDO2, loginUserId))
            return success(true);
        else
            return success(false);
    }

    private boolean doMergeCus(CustomerDO customerDO1,
                               CustomerDO customerDO2,
                               long loginUserId) {

        /*
        提交验证：
        当保留客户和非主客户的客户经理均不为空且不是同一个用户(允许客户在公海)，
        弹出提示：保留客户和非主客户，不能在两个不同的客户经理名下，
        请先确认两个客户在同一个客户经理名下，不允许提交
        当非主客户/保留客户在移交过程中，不允许合并，弹出提示
        当允许提交，先弹出上面的注意事项提示，确认后提交
        1. 非主客户中的客户联系人所有信息追加到保留客户中，注意：关联账号也需要一并追加
        2. 非主客户中的所有关联订单中的客户编号需要更新为保留客户，业绩归属为非主客户，
        也需要同时更新为保留客户（此处需要注意有一些订单的收发货人是同一个客户，
        还需要注意控货订单的情况，控货权）
        3. 非主客户的跟进记录，报价单，客户投诉，品牌授权，
        财务相关单据（收款单，佣金请款单，佣金付款单），都需要更新为保留客户
        4. 非主客户关联的注册会员信息
        4. 提交成功后，非主客户删除，插入记录到保留客户，
        备注：客户合并，删除XXX非主客户编号
         注意：此处要求是在保留客户中能查询到非主客户的所有信息，
         不清楚数据库逻辑结构，此处需要开发查找出所有的关联关系
         非主客户关联的订单/收款单/佣金情况单，佣金付款单有在途流程，
         需要测试这种情况是否影响合并
         */
        //把客户2的联系方式合并到客户1  1
        doMogeContact(customerDO1, customerDO2, loginUserId);
        //合并报价单  2
        doMogeOffer(customerDO1, customerDO2, loginUserId);
        //合并跟进记录  3
        doMergeCustomerFollow(customerDO1, customerDO2, loginUserId);
        //合并客户投诉  4
        doMergeCustomerComplaint(customerDO1, customerDO2, loginUserId);
        //合并收款单信息  5
        doMergeeReceipt(customerDO1, customerDO2, loginUserId);
        //合并产品品牌授权  6
        doMergeProductBrandEmpower(customerDO1,customerDO2,loginUserId);
        //合并客户登记日志  7
        doMergeCustomerLevelLog(customerDO1,customerDO2,loginUserId);
        //合并信用日志  8
        customerService.doMergeCustomerCreditLog(customerDO1.getId(),
                customerDO2.getId());
        //合并操作日志  9
        customerService.doMergeCustomerOperateLog(customerDO1.getId(),
                customerDO2.getId());

        return true;
    }

    //合并客户信用日志
    private void doMergeCustomerLevelLog(CustomerDO customerDO1,
                                          CustomerDO customerDO2,
                                          long loginUserId)
    {
        customerService.doMergeCustomerLevelLog(customerDO1.getId(),
                customerDO2.getId());
    }


    //合并产品品牌授权
    private void doMergeProductBrandEmpower(CustomerDO customerDO1,
                                             CustomerDO customerDO2,
                                             long loginUserId)
    {
        LambdaQueryWrapperX q2 =
                new LambdaQueryWrapperX<ProductBrandEmpowerDO>().
                        eq(ProductBrandEmpowerDO::getCustomerId,
                                customerDO2.getId());
        List<ProductBrandEmpowerDO> list =
                productBrandEmpowerService.selectList(q2);
        if (CollectionUtil.isEmpty(list)) return;

        for (ProductBrandEmpowerDO d : list
        ) {
            d.setCustomerId(customerDO1.getId());
            productBrandEmpowerService.updateById(d);
        }
    }

    private void doMergeCustomerComplaint(CustomerDO customerDO1,
                                          CustomerDO customerDO2,
                                          long loginUserId) {
        LambdaQueryWrapperX q2 =
                new LambdaQueryWrapperX<CustomerComplaintDO>().
                        eq(CustomerComplaintDO::getCustomerId,
                                customerDO2.getId());
        List<CustomerComplaintDO> list =
                customerComplaintService.selectList(q2);

        if (CollectionUtil.isEmpty(list)) return;

        for (CustomerComplaintDO d : list
        ) {
            d.setCustomerId(customerDO1.getId());
            customerComplaintService.updateById(d);
        }
    }

    //合并收款单
    private void doMergeeReceipt(CustomerDO customerDO1,
                                 CustomerDO customerDO2,
                                 long loginUserId) {
        LambdaQueryWrapperX q2 =
                new LambdaQueryWrapperX<ReceiptDO>().
                        eq(ReceiptDO::getCustomerId, customerDO2.getId());
        List<ReceiptDO> list = receiptService.selectList(q2);
        if (CollectionUtil.isEmpty(list)) return;
        for (ReceiptDO d : list
        ) {
            d.setCustomerId(customerDO1.getId());
            receiptService.updateById(d);
        }
    }

    //合并客户跟进
    private void doMergeCustomerFollow(CustomerDO customerDO1,
                                       CustomerDO customerDO2,
                                       long loginUserId) {
        LambdaQueryWrapperX q2 =
                new LambdaQueryWrapperX<CustomerFollowDO>().
                        eq(CustomerFollowDO::getCustomerId, customerDO2.getId());
        List<CustomerFollowDO> list = customerFollowService.selectList(q2);
        if (CollectionUtil.isEmpty(list)) return;

        for (CustomerFollowDO d : list
        ) {
            d.setCustomerId(customerDO1.getId());
            customerFollowService.updateById(d);
        }
    }

    //合并报价单
    private void doMogeOffer(CustomerDO customerDO1,
                             CustomerDO customerDO2,
                             long loginUserId) {
        LambdaQueryWrapperX q2 =
                new LambdaQueryWrapperX<OfferDO>().
                        eq(OfferDO::getRelationId, customerDO2.getId());
        List<OfferDO> list = offerService.selectList(q2);
        if (CollectionUtil.isEmpty(list)) return;

        for (OfferDO o : list
        ) {
            o.setRelationId(customerDO1.getId());
            o.setUpdateTime(new Date());
            offerService.updateById(o);
        }
    }


    //合并联系人
    private void doMogeContact(CustomerDO customerDO1,
                               CustomerDO customerDO2,
                               long loginUserId) {
        LambdaQueryWrapperX q2 =
                new LambdaQueryWrapperX<CustomerContactsDO>().
                        eq(CustomerContactsDO::getCustomerId, customerDO2.getId());
        List<CustomerContactsDO> cuscontactList2 =
                customerContactsService.selectList(q2);
        if (CollectionUtil.isNotEmpty(cuscontactList2)) {
            for (CustomerContactsDO d : cuscontactList2
            ) {
                d.setCustomerId(customerDO1.getId());
                d.setUpdateTime(new Date());
                d.setUpdater(String.valueOf(loginUserId));
                customerContactsService.updateById(d);
            }
        }
    }


    @GetMapping("/get")
    @ApiOperation("获得客户详情")
    @ApiImplicitParam(name = "id", value = "id", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:customer:query')")
    public CommonResult<CustomerDetailRespVO> getCustomer(@RequestParam("id") Long id) {
        CustomerDetailRespVO respVO = customerService.getCustomerDetailRespVO(id);
        if (Objects.isNull(respVO)) {
            throw exception(CUSTOMER_NOT_EXISTS);
        }

        Long founder = respVO.getFounder();
        if (founder != null) {
            if (Boolean.TRUE.equals(respVO.getIsWebOrderConsigneeSync())) {
                UserRespDTO user = memberUserApi.getUser(founder);
                if (user != null) {
                    respVO.setFounderName(user.getNickname() + "(会员)");
                }
            } else {
                AdminUserRespDTO user = adminUserApi.getUser(founder);
                if (user != null) {
                    respVO.setFounderName(user.getNickname());
                }
            }
        }
        if (null != respVO.getPromoter()) {
            CustomerDO customer1 = customerService.getCustomer(respVO.getPromoter());
            if (null != customer1) {
                respVO.setPromoterName(customer1.getName());
            }
        }

        //设置银行账号
        CustomerBankQueryVO customerBankQueryVO = new CustomerBankQueryVO();
        customerBankQueryVO.setCustomerId(id);
        List<CustomerBankDO> bankList = customerBankService.getBankList(customerBankQueryVO);
        List<CustomerBankBackVO> customerBankBackVOS = CustomerBankConvert.INSTANCE.convertList(bankList)
                .stream().sorted(Comparator.comparing(CustomerBankBackVO::getId)).collect(Collectors.toList());

        respVO.setCustomerBankBackVOList(customerBankBackVOS);
        return success(respVO);
    }

    private String getContactStr(Long customerId) {
        List<CustomerContactsDO> list =
                customerContactsService.getCustomerContactsListByCustomerId(customerId);
        List<CustomerContactsRespVO> respVOList =
                CustomerContactsConvert.INSTANCE.convertList(list);
        String sResult = "";
        if (CollectionUtil.isEmpty(respVOList)) {
            return sResult;
        }
        for (CustomerContactsRespVO m : respVOList
        ) {
            sResult += m.getName() + "," + m.getPhoneNew() + "\r\n";
        }
        //sResult = sResult.substring(sResult.length() - 4);
        return sResult;
    }


    @GetMapping("/get2")
    @ApiOperation("获得客户详情")
    @ApiImplicitParam(name = "id", value = "id", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:customer:query')")
    public CommonResult<CustomerDetailRespVO> getCustomer2(
            @RequestParam("id") Long id) {

        CustomerDetailRespVO respVO = customerService.getCustomerDetailRespVO(id);
        if (Objects.isNull(respVO)) {
            throw exception(CUSTOMER_NOT_EXISTS);
        }

        Long founder = respVO.getFounder();
        if (founder != null) {
            if (Boolean.TRUE.equals(respVO.getIsWebOrderConsigneeSync())) {
                UserRespDTO user = memberUserApi.getUser(founder);
                if (user != null) {
                    respVO.setFounderName(user.getNickname() + "(会员)");
                }
            } else {
                AdminUserRespDTO user = adminUserApi.getUser(founder);
                if (user != null) {
                    respVO.setFounderName(user.getNickname());
                }
            }
        }
        if (null != respVO.getPromoter()) {
            CustomerDO customer1 = customerService.getCustomer(respVO.getPromoter());
            if (null != customer1) {
                respVO.setPromoterName(customer1.getName());
            }
        }

        //设置银行账号
        CustomerBankQueryVO customerBankQueryVO = new CustomerBankQueryVO();
        customerBankQueryVO.setCustomerId(id);
        List<CustomerBankDO> bankList = customerBankService.getBankList(customerBankQueryVO);
        List<CustomerBankBackVO> customerBankBackVOS = CustomerBankConvert.INSTANCE.convertList(bankList)
                .stream().sorted(Comparator.comparing(CustomerBankBackVO::getId)).collect(Collectors.toList());

        respVO.setCustomerBankBackVOList(customerBankBackVOS);
        String sR=getContactStr(id);
        respVO.setContactstr(sR);
        return success(respVO);
    }



    @GetMapping("/list")
    @ApiOperation("根据客户id集合获得客户详情列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
//    @PreAuthorize("@ss.hasPermission('ecw:customer:query')")
    public CommonResult<List<CustomerRespVO>> getCustomerList(@RequestParam("ids") Collection<Long> ids) {
        List<CustomerDO> list = customerService.getCustomerList(ids);

        return success(CustomerConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/get-wait-for-confirm")
    @ApiOperation("获得待接收列表")
    //@PreAuthorize("@ss.hasPermission('ecw:customer:query')")
    public CommonResult<PageResult<CustomerRespVO>> getWaitConfirmList(
            @Valid CustomerPageReqDTO customerPageReqVO, PageVO page) {
        //customerPageReqVO.setCustomerService(WebFrameworkUtils.getLoginUserId());
        List<Long> setCustomerService = new ArrayList<>();
        setCustomerService.add(WebFrameworkUtils.getLoginUserId());
        customerPageReqVO.setCustomerService(setCustomerService);

        PageResult<CustomerDO> pageResult =
                customerService.getWaitToConfirmCustomerPageReq(customerPageReqVO,
                        page);
        return success(CustomerConvert.INSTANCE.convertPage(pageResult));
    }

    private Integer getCustomerTime(String customers_to_be_assigned, int i) {
        DictDataRespDTO dictDataRespDTO = DictFrameworkUtils.parseDictDataFromCache("customer_time", customers_to_be_assigned);
        return dictDataRespDTO != null ? Integer.valueOf(dictDataRespDTO.getValue()) : i;
    }

    @GetMapping("/get-potential")
    @ApiOperation("获得潜在客户列表")
//    @PreAuthorize("@ss.hasPermission('ecw:customer:query')")
    public CommonResult<PageResult<CustomerRespVO>> getPotentialList(
            @Valid CustomerPageReqDTO pageVO, PageVO page) {

        // 20230331 全部客户列表，当客户经理不是空时，排除公海客户
        if (pageVO.getCustomerService() != null) {
            pageVO.setIsInOpenSea(false);
        }

        //设置潜在客户标识
        pageVO.setIsPotential(true);

        //PageResult<CustomerDO> pageResult = customerService.getCustomerPage(pageVO);
        PageResult<CustomerDO> pageResult = customerService.getMyCustomerPage(pageVO, page);
        return success(CustomerConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/get-wait-for-distribution")
    @ApiOperation("获得待分配客户列表")
    //@PreAuthorize("@ss.hasPermission('ecw:customer:query')")
    public CommonResult<PageResult<CustomerRespVO>> getWaitForDistributionList(@Valid CustomerPageReqDTO customerPageReqVO, PageVO page) {
        //
        PageResult<CustomerDO> pageResult =
                customerService.getWaitToAssignedCustomerPageReq(customerPageReqVO, page);
        return success(CustomerConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/get-public")
    @ApiOperation("获得公海池客户列表")
//    @PreAuthorize("@ss.hasPermission('ecw:customer:query')")
    public CommonResult<PageResult<CustomerRespVO>> getPublicCustomerList(@Valid CustomerPageReqDTO customerPageReqVO, PageVO page) {

        PageResult<CustomerDO> pageResult =
                customerService.getOpenSeaCustomerPageReq(customerPageReqVO, page);
        return success(CustomerConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/delay/approval/info")
    @ApiOperation("获取客户延期调入公海池的审批详情")
    //@PreAuthorize("@ss.hasPermission('ecw:customer:query')")
    @ApiImplicitParam(value = "客户ID", name = "customerId", required = true, dataType = "Long")
    public CommonResult<CustomerDelayApprovalInfoDto> getDelayApprovalInfo(@RequestParam("customerId") Long customerId) {
        CustomerDelayApprovalInfoDto delayApprovalInfoDto =
                customerService.getDelayApprovalInfo(customerId);
        return success(delayApprovalInfoDto);
    }

    @GetMapping("/delay/approval")
    @ApiOperation("获取客户延期调入公海池的审批详情")
    //@PreAuthorize("@ss.hasPermission('ecw:customer:query')")
    @ApiImplicitParam(value = "审批ID", name = "approvalId", required = true, dataType = "Long")
    public CommonResult<CustomerDelayApprovalInfoDto> getDelayApproval(@RequestParam("approvalId") Long approvalId) {
        CustomerDelayApprovalInfoDto delayApprovalInfoDto = customerService.getDelayApproval(approvalId);
        return success(delayApprovalInfoDto);
    }

    @PostMapping("/delay/approval")
    @ApiOperation("客户延期调入公海池的审批申请")
    //@PreAuthorize("@ss.hasPermission('ecw:customer:postpone')")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> delayApproval(@RequestBody CustomerDelayApprovalInfoDto delayApprovalInfoDto) {
        customerService.delayApproval(delayApprovalInfoDto);
        return success(true);
    }

    @PutMapping("/cancel/delay/approval")
    @ApiOperation("取消客户延期调入公海池的审批申请")
    //@PreAuthorize("@ss.hasPermission('ecw:customer:update')")
    @ApiImplicitParams({
            @ApiImplicitParam(value = "审批id", name = "approveId", required = true, dataType = "Long"),
            @ApiImplicitParam(value = "原因", name = "reason", dataType = "String")
    })
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> cancelDelayApproval(@RequestParam("approveId") Long approveId, @RequestParam(value = "reason", required = false) String reason) {
        customerService.cancelDelayApproval(approveId, reason);
        return success(true);
    }


//     @GetMapping("/handover/approval/info")
//     @ApiOperation("获取客户移交的审批详情")
// //    @PreAuthorize("@ss.hasPermission('ecw:customer:query')")
//     @ApiImplicitParam(value = "客户ID", name = "customerId", required = true, dataType = "Long")
//     public CommonResult<CustomerHandoverApprovalInfoDto> getHandoverApprovalInfo(@RequestParam("customerId") Long customerId) {
//         CustomerDelayApprovalInfoDto delayApprovalInfoDto = customerService.getDelayApprovalInfo(customerId);
//         return success(delayApprovalInfoDto);
//     }

    @GetMapping("/handover/approval")
    @ApiOperation("获取客户移交的审批详情")
//    @PreAuthorize("@ss.hasPermission('ecw:customer:query')")
    @ApiImplicitParam(value = "审批ID", name = "approvalId", required = true, dataType = "Long")
    public CommonResult<CustomerHandoverApprovalInfoDto> getHandoverApproval(@RequestParam("approvalId") Long approvalId) {
        CustomerHandoverApprovalInfoDto handoverApprovalInfoDto = customerService.getHandoverApproval(approvalId);
        return success(handoverApprovalInfoDto);
    }

    @PostMapping("/handover/approval")
    @ApiOperation("客户移交/申请")
    // @PreAuthorize("@ss.hasPermission('ecw:customer:postpone')")
    @Idempotent(timeout = 5)
    public CommonResult<String> handoverApproval(@Valid @RequestBody CustomerHandOverReqVO customerHandOverReqVO) {
        return success(customerService.handoverApproval(customerHandOverReqVO));
    }

    @PutMapping("/cancel/handover/approval")
    @ApiOperation("取消客户移交的审批申请")
    //@PreAuthorize("@ss.hasPermission('ecw:customer:update')")
    @ApiImplicitParams({
            @ApiImplicitParam(value = "审批id", name = "approveId", required = true, dataType = "Long"),
            @ApiImplicitParam(value = "原因", name = "reason", dataType = "String")
    })
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> cancelHandoverApproval(@RequestParam("approveId") Long approveId, @RequestParam(value = "reason", required = false) String reason) {
        customerService.cancelHandoverApproval(approveId, reason);
        return success(true);
    }


    @GetMapping("/get-mine")
    @ApiOperation("获得我的客户列表")
    //@PreAuthorize("@ss.hasPermission('ecw:customer:query')")
    public CommonResult<PageResult<CustomerRespVO>> getMine(@Valid CustomerPageReqDTO customerPageReqVO, PageVO page) {

        //customerPageReqVO.setCustomerService(WebFrameworkUtils.getLoginUserId());
        List<Long> setCustomerService = new ArrayList<>();
        setCustomerService.add(WebFrameworkUtils.getLoginUserId());
        customerPageReqVO.setCustomerService(setCustomerService);

        //未被接收
        customerPageReqVO.setIsCustomerServiceConfirmed(true);

        // customerPageReqVO.setFlag4QueryCondition("Y");

        PageResult<CustomerDO> pageResult = customerService.getMyCustomerPage(customerPageReqVO, page);
        return success(CustomerConvert.INSTANCE.convertPage(pageResult));
    }

    @PutMapping("/confirm")
    @ApiOperation("跟进客户确认接收")
    @PreAuthorize("@ss.hasPermission('ecw:customer:treat-accept')")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> confirm(
            @Valid @RequestBody CustomerConfirmReqVO customerConfirmReqVO) {
        //添加接收确认后判断客户是新客户还是老客户逻辑
        Long id = customerConfirmReqVO.getCustomerId();
        customerService.confirmCustomer(id);
        return success(true);
    }

    @PutMapping("/handOver")
    @ApiOperation("分配客服")
    @PreAuthorize("@ss.hasAnyPermissions('ecw:customer:batch-transfer', 'ecw:customer:treat-transfer', 'ecw:customer:distribution-transfer', 'ecw:customer:distribution-customer-service', 'ecw:customer:dep-batch-transfer')")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> handOver(@Valid @RequestBody CustomerHandOverReqVO customerHandOverReqVO) {

        List<Long> idList = customerHandOverReqVO.getCustomerIdList();
        Long customerServiceId = customerHandOverReqVO.getCustomerServiceId();

        List<CustomerDO> customerList = customerService.getCustomerList(idList);

        if (CollectionUtil.isEmpty(customerList)) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.CUSTOMER_NOT_EXISTS);
        }

        String openSeaCustomerNum = customerList.stream().filter(t -> t.getIsInOpenSea()).map(CustomerDO::getNumber).collect(Collectors.joining(", "));
        if (StringUtils.isNotEmpty(openSeaCustomerNum)) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.CUSTOMER_IN_OPEN_SEA_CANNOT_BE_HANDOVER, openSeaCustomerNum);
        }

        Date now = new Date();
        Integer confirmTime = getCustomerTime("confirm_time", 24);

        // customerList.stream().forEach(customer -> {
        //     customer.setCustomerService(customerServiceId);
        //     customer.setIsCustomerServiceConfirmed(false);
        //     customer.setCustomerServiceAssignedTime(now);
        //     customer.setIsInOpenSea(false);
        //
        //     //分配后1天未确认，进入公海池
        //     customer.setEstimateEnterOpenSeaTime(DateUtil.offsetHour(now, confirmTime));
        // });
        //
        // this.customerService.updateBatchById(customerList);

        customerService.assignedCustomer(customerList, customerServiceId, CustomerOperateTypeEnum.ASSIGN, customerHandOverReqVO.getReason(), JSON.toJSONString(customerHandOverReqVO));

        return success(true);
    }

    @PutMapping("/catch-open-sea-customer")
    @ApiOperation("捞取")
    @ApiImplicitParam(name = "id", value = "客户id", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:customer:sea-gain')")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> catchCustomer(@RequestParam("id") Long id) {
        //公海池捞取客户，修改客户业绩类型需要调整此处的逻辑
        customerService.catchCustomer(id);
        return success(true);
    }

    @GetMapping("/page")
    @ApiOperation("获得客户分页")
    @PreAuthorize("@ss.hasPermission('ecw:customer:query')")
    public CommonResult<PageResult<CustomerRespVO>> getCustomerPage(
            @Valid CustomerPageReqDTO pageVO, PageVO page) {
        // 20230331 全部客户列表，当客户经理不是空时，排除公海客户
        if (pageVO.getCustomerService() != null) {
            pageVO.setIsInOpenSea(false);
        }

        //PageResult<CustomerDO> pageResult = customerService.getCustomerPage(pageVO);
        PageResult<CustomerDO> pageResult =
                customerService.getMyCustomerPage(pageVO, page);
        return success(CustomerConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/deptPage")
    @ApiOperation("获得部门客户")
    @PreAuthorize("@ss.hasAnyPermissions('ecw:customer:query', 'ecw:customer:dep-query')")
    public CommonResult<PageResult<CustomerRespVO>> getDeptCustomerPage(
            @Valid CustomerPageReqDTO pageVO, PageVO page) {
        PageResult<CustomerDO> pageResult = customerService.getDeptCustomerPageReq(pageVO, page);
        return success(CustomerConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/select")
    @ApiOperation("获得客户下拉列表")
//    @PreAuthorize("@ss.hasPermission('ecw:customer:query')")
    public CommonResult<PageResult<CustomerRespVO>> getCustomerSelect(@Valid CustomerSelectReqVO selectVo) {

        PageResult<CustomerDO> pageResult = customerService.searchCustomer(selectVo);
        return success(CustomerConvert.INSTANCE.convertPage(pageResult));
    }

    /**
     * 支持多选
     *
     * @param exportReqVO
     * @param downloadTypeEnum
     * @param fileName
     */
    private void sendFileMake(CustomerExportReqDTO exportReqVO, DownloadTypeEnum downloadTypeEnum, String fileName) {
        FileMakeReqDTO reqDTO = new FileMakeReqDTO();
        reqDTO.setType(downloadTypeEnum.getType());
        reqDTO.setName(fileName);
        reqDTO.setFileSuffix("xlsx");
        reqDTO.setTemporaryFile(true);
        reqDTO.setUserType(2);
        reqDTO.setLang(I18nMessage.getLang());
        reqDTO.setRequestParams(JSONObject.toJSONString(exportReqVO));
        fileMakeApi.sendFileMake(reqDTO);
    }

    /**
     * 单选
     *
     * @param exportReqVO
     * @param downloadTypeEnum
     * @param fileName
     */
    private void sendFileMake(CustomerExportReqVO exportReqVO, DownloadTypeEnum downloadTypeEnum, String fileName) {
        FileMakeReqDTO reqDTO = new FileMakeReqDTO();
        reqDTO.setType(downloadTypeEnum.getType());
        reqDTO.setName(fileName);
        reqDTO.setFileSuffix("xlsx");
        reqDTO.setTemporaryFile(true);
        reqDTO.setUserType(2);
        reqDTO.setLang(I18nMessage.getLang());
        reqDTO.setRequestParams(JSONObject.toJSONString(exportReqVO));
        fileMakeApi.sendFileMake(reqDTO);
    }

    @GetMapping("/export-excel")
    @ApiOperation("全部客户导出")
    @OperateLog(type = EXPORT)
    public CommonResult<Boolean> exportCustomerExcel(@Valid CustomerExportReqDTO exportReqVO,
                                                     HttpServletResponse response) throws IOException {
        sendFileMake(exportReqVO, DownloadTypeEnum.CUSTOMER_EXCEL_EXPORT, "客户导出Excel");
        return success(true);
    }

    @GetMapping("/mine-export-excel")
    @ApiOperation("我的客户列表导出")
    @OperateLog(type = EXPORT)
    public CommonResult<Boolean> exportMyCustomerExcel(@Valid CustomerExportReqDTO exportReqVO,
                                                       HttpServletResponse response) throws IOException {

        //exportReqVO.setCustomerService(WebFrameworkUtils.getLoginUserId());
        //未被接收
        exportReqVO.setIsCustomerServiceConfirmed(true);
        // customerPageReqVO.setFlag4QueryCondition("Y");
        List<Long> setCustomerService = new ArrayList<>();
        setCustomerService.add(WebFrameworkUtils.getLoginUserId());
        exportReqVO.setCustomerService(setCustomerService);

        sendFileMake(exportReqVO, DownloadTypeEnum.CUSTOMER_MINE_EXCEL_EXPORT, "我的客户导出Excel");
        return success(true);
    }


    @GetMapping("/waite-confirm-export-excel")
    @ApiOperation("我的待接收客户列表导出")
    @OperateLog(type = EXPORT)
    public CommonResult<Boolean> exportWaiteConfirmCustomerExcel(@Valid CustomerExportReqDTO exportReqVO,
                                                                 HttpServletResponse response) throws IOException {
        sendFileMake(exportReqVO, DownloadTypeEnum.CUSTOMER_WAIT_CONFIRM_PUBLIC_EXCEL_EXPORT, "我的待接收客户导出Excel");
        return success(true);
    }

    @GetMapping("/public-export-excel")
    @ApiOperation("公海池客户列表导出")
    @OperateLog(type = EXPORT)
    public CommonResult<Boolean> exportPublicCustomerExcel(@Valid CustomerExportReqDTO exportReqVO,
                                                           HttpServletResponse response) throws IOException {
        sendFileMake(exportReqVO, DownloadTypeEnum.CUSTOMER_PUBLIC_EXCEL_EXPORT, "公海池客户导出Excel");
        return success(true);

    }

    @GetMapping("/wait-distribution-export-excel")
    @ApiOperation("待分配客户列表导出")
    @OperateLog(type = EXPORT)
    public CommonResult<Boolean> exportWaitDistributionCustomerExcel(@Valid CustomerExportReqDTO exportReqVO,
                                                                     HttpServletResponse response) throws IOException {
        sendFileMake(exportReqVO, DownloadTypeEnum.CUSTOMER_WAIT_DISTRIBUTION_PUBLIC_EXCEL_EXPORT, "待分配客户导出Excel");
        return success(true);
    }

    @GetMapping("/potential-export-excel")
    @ApiOperation("潜在客户列表导出")
    @OperateLog(type = EXPORT)
    public CommonResult<Boolean> exportPotentialCustomerExcel(
            @Valid CustomerExportReqDTO exportReqVO,
            HttpServletResponse response) throws IOException {
        if (CollectionUtil.isNotEmpty(exportReqVO.getCustomerService())) {
            exportReqVO.setIsInOpenSea(false);
        }
        //设置潜在客户标识
        exportReqVO.setIsPotential(true);

        sendFileMake(exportReqVO, DownloadTypeEnum.CUSTOMER_POTENTIAL_EXCEL_EXPORT, "潜在客户导出Excel");
        return success(true);
    }

    @GetMapping("/dept-export-excel")
    @ApiOperation("部门客户列表导出")
    @OperateLog(type = EXPORT)
    public CommonResult<Boolean> exportDeptCustomerExcel(@Valid CustomerExportReqDTO exportReqVO,
                                                         HttpServletResponse response) throws IOException {
        sendFileMake(exportReqVO, DownloadTypeEnum.CUSTOMER_DEPT_EXCEL_EXPORT, "部门客户导出Excel");
        return success(true);
    }


    @ApiOperation("测试使用 - 根据客户id将客户丢入公海池")
    @GetMapping("/test-enter-to-open-sea")
    public CommonResult<Boolean> test(@RequestParam("customerId") Long customerId) {
        CustomerDO customer = customerService.getById(customerId);
        if (customer == null) {
            return error(ErrorCodeConstants.CUSTOMER_NOT_EXISTS);
        }
        customerService.enterToOpenSea(Arrays.asList(customer));

        return success(true);
    }


    /**
     * 将老的10位的客户number调整为新规则
     *
     * @return
     */
    @GetMapping("/change-old-customer-number")
    public CommonResult<Boolean> test() {

        customerService.changeOldCustomerNumber();

        return success(true);
    }


    @PutMapping("/change-customer-air")
    @ApiOperation("设置客户为空运客户/非空运客户")
    @PreAuthorize("@ss.hasPermission('ecw:customer:transport')")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> changeCustomerAir(@Valid @RequestBody CustomerChangeAirReqVO customerChangeAirReqVO) {

        customerService.changeCustomerAir(customerChangeAirReqVO);

        return success(true);
    }


    @PutMapping("/change-customer-fcl")
    @ApiOperation("设置客户为海运整柜客户/非海运整柜客户")
    @PreAuthorize("@ss.hasPermission('ecw:customer:fcl')")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> changeCustomerFcl(@Valid @RequestBody CustomerChangeFclReqVO customerChangeFclReqVO) {

        customerService.changeCustomerFcl(customerChangeFclReqVO);
        return success(true);
    }


    @PutMapping("/change-customer-no-consignee")
    @ApiOperation("设置客户是否是默认控货无收货人")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> changeCustomerNoConsignee(@Valid @RequestBody CustomerChangeNoConsigneeReqVO customerChangeNoConsigneeReqVO) {

        customerService.changeCustomerNoConsignee(customerChangeNoConsigneeReqVO);
        return success(true);
    }


    @PutMapping("/change-customer-default-pay")
    @ApiOperation("设置客户是否是默认付款")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> changeCustomerDefaultPay(@Valid @RequestBody CustomerChangeDefaultPayReqVO customerChangeDefaultPayReqVO) {

        customerService.changeCustomerDefaultPay(customerChangeDefaultPayReqVO);
        return success(true);
    }


    @PutMapping("/change-customer-default-billing")
    @ApiOperation("设置客户是否是默认开票")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> changeCustomerDefaultBilling(@Valid @RequestBody CustomerChangeDefaultBillingReqVO customerChangeDefaultBillingReqVO) {

        customerService.changeCustomerDefaultBilling(customerChangeDefaultBillingReqVO);
        return success(true);
    }


    @PutMapping("/recycle-unconfirmed-customer")
    @ApiOperation("回收已分配未接收的客户")
    // @PreAuthorize("@ss.hasPermission('ecw:customer:recycle:unconfirmed')")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> recycleUnConfirmedCustomer(@Valid @RequestBody CustomerUnConfirmReqVO customerUnConfirmReqVO) {

        customerService.recycleUnConfirmedCustomer(customerUnConfirmReqVO);

        return success(true);
    }


    @PutMapping("/update-customer-to-old")
    @ApiOperation("根据客户经理id更新成交状态的新客户为老客户")
    @ApiImplicitParam(name = "customerServiceId", value = "客户经理id", required = true, example = "1024", dataTypeClass = Long.class)
    // @PreAuthorize("@ss.hasPermission('ecw:customer:update-customer-to-old')")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> updateCustomerToOldByCustomerService(@RequestParam("customerServiceId") Long customerServiceId) {

        List<Long> newDealCustomerIdList = customerService.updateCustomerToOldByCustomerService(customerServiceId);

        // 异步纪录操作日志
        customerService.asyncLogUpdateCustomerToOldByCustomerService(customerServiceId, newDealCustomerIdList);

        return success(true);
    }

    @PutMapping("/update-customer-new-or-old")
    @ApiOperation("根据客户id更新老客户类型")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "customerId", value = "客户id", required = true, example = "1024", dataTypeClass = Long.class),
            @ApiImplicitParam(name = "isNew", value = "true新客户false老客户", dataType = "Boolean")
    })
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> updateCustomerNewOrOld(@RequestParam("customerId") Long customerId, @RequestParam("isNew") Boolean isNew) {

        customerService.updateCustomerNewOrOld(customerId, isNew);

        return success(true);
    }

}
