package cn.iocoder.yudao.module.customer.controller.admin.customer.commission;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.module.customer.convert.customer.commission.CustomerCommissionConvert;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.CustomerDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.commission.CustomerCommissionDO;
import cn.iocoder.yudao.module.customer.service.customer.CustomerService;
import cn.iocoder.yudao.module.customer.service.customer.commission.CustomerCommissionService;
import cn.iocoder.yudao.module.customer.vo.customer.commission.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.CUSTOMER_COMMISSION_EXISTS;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.CUSTOMER_SERVICE_COMMISSION_EXISTS;

@Api(tags = "管理后台 - 客户佣金类型")
@RestController
@RequestMapping("/ecw/customer-commission")
@Validated
public class CustomerCommissionController {

    @Resource
    private CustomerCommissionService customerCommissionService;

    @Resource
    private CustomerService customerService;

    @PostMapping("/create")
    @ApiOperation("创建客户佣金类型")
    // @PreAuthorize("@ss.hasPermission('ecw:customer-commission:create')")
    public CommonResult<Long> createCustomerCommission(@Valid @RequestBody CustomerCommissionCreateReqVO createReqVO) {

        Long customerId = createReqVO.getCustomerId();
        CustomerCommissionDO customerCommissionDO = customerCommissionService.getCustomerCommissionByCustomerId(customerId);
        // if (customerCommissionDO != null) {
        //     return error(CUSTOMER_COMMISSION_EXISTS);
        // }
        CustomerDO customer = customerService.getCustomer(customerId);
        //推介人 2.若客户A已设置佣金规则，推介人有佣金规则，不能客户修改推介人，提示“该推介人已设置佣金，客户与推介人只能有一个设置佣金规则”。
        Long promoter = customer.getPromoter();
        if (promoter != null) {
            customerCommissionDO = customerCommissionService.getCustomerCommissionByCustomerId(customerId);
            if (customerCommissionDO != null) {
                return error(CUSTOMER_SERVICE_COMMISSION_EXISTS);
            }
        }

        return success(customerCommissionService.createCustomerCommission(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新客户佣金")
    // @PreAuthorize("@ss.hasPermission('ecw:customer-commission:update')")
    public CommonResult<Boolean> updateCustomerCommission(@Valid @RequestBody CustomerCommissionUpdateReqVO updateReqVO) {
        customerCommissionService.updateCustomerCommission(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除客户佣金")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    // @PreAuthorize("@ss.hasPermission('ecw:customer-commission:delete')")
    public CommonResult<Boolean> deleteCustomerCommission(@RequestParam("id") Long id) {
        customerCommissionService.deleteCustomerCommission(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("根据主键获得佣金详情")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    // @PreAuthorize("@ss.hasPermission('ecw:customer-commission:query')")
    public CommonResult<CustomerCommissionRespVO> getCustomerCommission(@RequestParam("id") Long id) {
        CustomerCommissionExportReqVO exportReqVO = new CustomerCommissionExportReqVO();
        exportReqVO.setId(id);
        List<CustomerCommissionDO> customerCommissionList = customerCommissionService.getCustomerCommissionList(exportReqVO);

        List<CustomerCommissionRespVO> customerCommissionRespVOS = CustomerCommissionConvert.INSTANCE.convertList(customerCommissionList);
        if (!CollectionUtils.isEmpty(customerCommissionList)) {
            return success(customerCommissionRespVOS.stream().findFirst().get());
        }
        return success(null);
    }

    @PostMapping("/get-dark-return-commission")
    @ApiOperation("根据具体条件获得暗佣")
    // @PreAuthorize("@ss.hasPermission('ecw:customer-commission:query')")
    public CommonResult<CustomerDarkReturnCommissionQueryRespVO> getDarkReturnCommission(@Valid @RequestBody CustomerDarkReturnCommissionQueryReqVO reqVO) {

        CustomerDarkReturnCommissionQueryRespVO customerDarkReturnCommissionQueryRespVO = customerCommissionService.getDarkReturnCommission(reqVO);

        return success(customerDarkReturnCommissionQueryRespVO);
    }

    @GetMapping("/getByCustomerId")
    @ApiOperation("通过客户ID获得客户佣金详情")
    @ApiImplicitParam(name = "customerId", value = "客户id", required = true, example = "1024", dataTypeClass = Long.class)
    // @PreAuthorize("@ss.hasPermission('ecw:customer-commission:query')")
    public CommonResult<CustomerCommissionRespVO> getByCustomerId(@RequestParam("customerId") Long customerId) {
        CustomerCommissionExportReqVO exportReqVO = new CustomerCommissionExportReqVO();
        exportReqVO.setCustomerId(customerId);
        CustomerCommissionDO customerCommissionDO = customerCommissionService.getCustomerCommissionByCustomerId(customerId);
        return success(CustomerCommissionConvert.INSTANCE.convert(customerCommissionDO));
    }

    //@GetMapping("/list")
    //@ApiOperation("获得客户佣金列表")
    //@ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    //@PreAuthorize("@ss.hasPermission('ecw:customer-commission:query')")
    //public CommonResult<List<CustomerCommissionRespVO>> getCustomerCommissionList(@RequestParam("ids") Collection<Long> ids) {
    //    List<CustomerCommissionDO> list = customerCommissionService.getCustomerCommissionList(ids);
    //    return success(CustomerCommissionConvert.INSTANCE.convertList(list));
    //}

    @GetMapping("/page")
    @ApiOperation("获得客户佣金分页")
    // @PreAuthorize("@ss.hasPermission('ecw:customer-commission:query')")
    public CommonResult<PageResult<CustomerCommissionRespVO>> getCustomerCommissionPage(@Valid CustomerCommissionPageReqVO pageVO) {
        PageResult<CustomerCommissionDO> pageResult = customerCommissionService.getCustomerCommissionPage(pageVO);
        return success(CustomerCommissionConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出客户佣金类型 Excel")
    // @PreAuthorize("@ss.hasPermission('ecw:customer-commission:export')")
    @OperateLog(type = EXPORT)
    public void exportCustomerCommissionExcel(@Valid CustomerCommissionExportReqVO exportReqVO,
              HttpServletResponse response) throws IOException {
        List<CustomerCommissionDO> list = customerCommissionService.getCustomerCommissionList(exportReqVO);
        // 导出 Excel
        List<CustomerCommissionExcelVO> datas = CustomerCommissionConvert.INSTANCE.convertList02(list);
        ExcelUtils.write(response, "客户佣金类型.xls", "数据", CustomerCommissionExcelVO.class, datas);
    }

}
