package cn.iocoder.yudao.module.customer.controller.admin.customer.complaint;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.framework.security.core.annotations.PreAuthenticated;
import cn.iocoder.yudao.module.customer.convert.customer.complaint.CustomerComplaintConvert;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.complaint.CustomerComplaintDO;
import cn.iocoder.yudao.module.customer.service.customer.complaint.CustomerComplaintService;
import cn.iocoder.yudao.module.customer.vo.customer.complaint.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.Collection;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;

@Api(tags = "管理后台 - 客户投诉")
@RestController
@RequestMapping("/ecw/customer-complaint")
@Validated
public class CustomerComplaintController {

    @Resource
    private CustomerComplaintService customerComplaintService;

    @PostMapping("/create")
    @ApiOperation("创建客户投诉")
    //@PreAuthorize("@ss.hasPermission('ecw:customer-complaint:create')")
    @PreAuthenticated
    public CommonResult<Long> createCustomerComplaint(@Valid @RequestBody CustomerComplaintCreateReqVO createReqVO, HttpServletRequest request) {

        return success(customerComplaintService.createCustomerComplaint(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新客户投诉")
    @PreAuthorize("@ss.hasPermission('ecw:customer-complaint:update')")
    public CommonResult<Boolean> updateCustomerComplaint(@Valid @RequestBody CustomerComplaintUpdateReqVO updateReqVO) {
        customerComplaintService.updateCustomerComplaint(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除客户投诉")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:customer-complaint:delete')")
    public CommonResult<Boolean> deleteCustomerComplaint(@RequestParam("id") Long id) {
        customerComplaintService.deleteCustomerComplaint(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得客户投诉")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:customer-complaint:query')")
    public CommonResult<CustomerComplaintRespVO> getCustomerComplaint(@RequestParam("id") Long id) {
        CustomerComplaintDO customerComplaint = customerComplaintService.getCustomerComplaint(id);
        return success(CustomerComplaintConvert.INSTANCE.convert(customerComplaint));
    }

    @GetMapping("/list")
    @ApiOperation("获得客户投诉列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('ecw:customer-complaint:query')")
    public CommonResult<List<CustomerComplaintRespVO>> getCustomerComplaintList(@RequestParam("ids") Collection<Long> ids) {
        List<CustomerComplaintDO> list = customerComplaintService.getCustomerComplaintList(ids);
        return success(CustomerComplaintConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得客户投诉分页")
    @PreAuthorize("@ss.hasPermission('ecw:customer-complaint:query')")
    public CommonResult<PageResult<CustomerComplaintRespVO>> getCustomerComplaintPage(@Valid CustomerComplaintPageReqVO pageVO) {
        return success(customerComplaintService.getCustomerComplaintPage(pageVO));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出客户投诉 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:customer-complaint:export')")
    @OperateLog(type = EXPORT)
    public void exportCustomerComplaintExcel(@Valid CustomerComplaintExportReqVO exportReqVO,
              HttpServletResponse response) throws IOException {
        List<CustomerComplaintExcelVO> list = customerComplaintService.getCustomerComplaintList(exportReqVO);
        // 导出 Excel
        ExcelUtils.write(response, "客户投诉.xls", "数据", CustomerComplaintExcelVO.class, list);
    }

}
