package cn.iocoder.yudao.module.customer.controller.admin.customer.follow;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.customer.convert.customer.complaint.follow.CustomerFollowConvert;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.follow.CustomerFollowDO;
//import cn.iocoder.yudao.module.customer.dto.follow.CustomerFollowPageDTO;
import cn.iocoder.yudao.module.customer.service.customer.complaint.follow.CustomerFollowService;
import cn.iocoder.yudao.module.customer.vo.customer.follow.CustomerFollowCreateReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.follow.CustomerFollowPageReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.follow.CustomerFollowRespVO;
import cn.iocoder.yudao.module.customer.vo.customer.follow.CustomerFollowUpdateReqVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Api(tags = "管理后台 - 客户跟进")
@RestController
@RequestMapping("/ecw/customer-follow")
@Validated
public class CustomerFollowController {

    @Resource
    private CustomerFollowService customerFollowService;

    @PostMapping("/create")
    @ApiOperation("创建客户跟进")
    // @PreAuthorize("@ss.hasPermission('ecw:customer-follow:create')")
    public CommonResult<Long> createCustomerFollow(@Valid @RequestBody CustomerFollowCreateReqVO createReqVO) {
        return success(customerFollowService.createCustomerFollow(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新客户跟进")
    // @PreAuthorize("@ss.hasPermission('ecw:customer-follow:update')")
    public CommonResult<Boolean> updateCustomerFollow(@Valid @RequestBody CustomerFollowUpdateReqVO updateReqVO) {
        customerFollowService.updateCustomerFollow(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除客户跟进")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:customer-follow:delete')")
    public CommonResult<Boolean> deleteCustomerFollow(@RequestParam("id") Long id) {
        customerFollowService.deleteCustomerFollow(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("根据跟进ID获得跟进数据")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    // @PreAuthorize("@ss.hasPermission('ecw:customer-follow:query')")
    public CommonResult<CustomerFollowRespVO> getCustomerFollow(@RequestParam("id") Long id) {
        CustomerFollowDO customerFollow = customerFollowService.getCustomerFollow(id);
        return success(CustomerFollowConvert.INSTANCE.convert(customerFollow));
    }

    //@GetMapping("/list")
    //@ApiOperation("获得客户跟进列表")
    //@ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    //// @PreAuthorize("@ss.hasPermission('ecw:customer-follow:query')")
    //public CommonResult<List<CustomerFollowRespVO>> getCustomerFollowList(@RequestParam("ids") Collection<Long> ids) {
    //    List<CustomerFollowDO> list = customerFollowService.getCustomerFollowList(ids);
    //    return success(CustomerFollowConvert.INSTANCE.convertList(list));
    //}

    @GetMapping("/page")
    @ApiOperation("根据业务获得客户跟进数据-分页")
    // @PreAuthorize("@ss.hasPermission('ecw:customer-follow:query')")
    public CommonResult<PageResult<CustomerFollowRespVO>> getCustomerFollowPage(@Valid CustomerFollowPageReqVO pageVO) {
        PageResult<CustomerFollowDO> pageResult = customerFollowService.customerFollowPage(pageVO);
        return success(CustomerFollowConvert.INSTANCE.convertPage(pageResult));
    }


    //@GetMapping("/export-excel")
    //@ApiOperation("导出客户跟进 Excel")
    //// @PreAuthorize("@ss.hasPermission('ecw:customer-follow:export')")
    //@OperateLog(type = EXPORT)
    //public void exportCustomerFollowExcel(@Valid CustomerFollowExportReqVO exportReqVO,
    //          HttpServletResponse response) throws IOException {
    //    List<CustomerFollowDO> list = customerFollowService.getCustomerFollowList(exportReqVO);
    //    // 导出 Excel
    //    List<CustomerFollowExcelVO> datas = CustomerFollowConvert.INSTANCE.convertList02(list);
    //    ExcelUtils.write(response, "客户跟进.xls", "数据", CustomerFollowExcelVO.class, datas);
    //}





}
