package cn.iocoder.yudao.module.customer.controller.admin.customerAirLog;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;
import cn.iocoder.yudao.module.customer.vo.customerAirLog.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerAirLog.CustomerAirLogDO;
import cn.iocoder.yudao.module.customer.convert.customerAirLog.CustomerAirLogConvert;
import cn.iocoder.yudao.module.customer.service.customerAirLog.CustomerAirLogService;

@Validated
@RestController
@Api(tags = "管理后台 - 客户空运日志")
@RequestMapping("/customer/air-log")
public class CustomerAirLogController {

    @Resource
    private CustomerAirLogService airLogService;

    @PostMapping("/create")
    @ApiOperation("创建客户空运日志")
    @PreAuthorize("@ss.hasPermission('customer:air-log:create')")
    public CommonResult<Long> createAirLog(@Valid @RequestBody CustomerAirLogCreateReqVO createReqVO) {
        return success(airLogService.createAirLog(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新客户空运日志")
    @PreAuthorize("@ss.hasPermission('customer:air-log:update')")
    public CommonResult<Boolean> updateAirLog(@Valid @RequestBody CustomerAirLogUpdateReqVO updateReqVO) {
        airLogService.updateAirLog(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除客户空运日志")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('customer:air-log:delete')")
    public CommonResult<Boolean> deleteAirLog(@RequestParam("id") Long id) {
        airLogService.deleteAirLog(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得客户空运日志")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('customer:air-log:query')")
    public CommonResult<CustomerAirLogBackVO> getAirLog(@RequestParam("id") Long id) {
        CustomerAirLogDO airLog = airLogService.getAirLog(id);
        return success(CustomerAirLogConvert.INSTANCE.convert(airLog));
    }

    @GetMapping("/list")
    @ApiOperation("获得客户空运日志列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('customer:air-log:query')")
    public CommonResult<List<CustomerAirLogBackVO>> getAirLogList(@RequestParam("ids") Collection<Long> ids) {
        List<CustomerAirLogDO> list = airLogService.getAirLogList(ids);
        return success(CustomerAirLogConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得客户空运日志分页")
    @PreAuthorize("@ss.hasPermission('customer:air-log:query')")
    public CommonResult<PageResult<CustomerAirLogBackVO>> getAirLogPage(@Valid CustomerAirLogQueryVO query, PageVO page) {
        PageResult<CustomerAirLogDO> pageResult = airLogService.getAirLogPage(query, page);
        return success(CustomerAirLogConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出客户空运日志 Excel")
    @PreAuthorize("@ss.hasPermission('customer:air-log:export')")
    @OperateLog(type = EXPORT)
    public void exportAirLogExcel(@Valid CustomerAirLogQueryVO query,
              HttpServletResponse response) throws IOException {
        List<CustomerAirLogDO> list = airLogService.getAirLogList(query);
        // 导出 Excel
        List<CustomerAirLogBackVO> datas = CustomerAirLogConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "客户空运日志.xls", "数据", CustomerAirLogBackVO.class, datas);
    }
}
