package cn.iocoder.yudao.module.customer.controller.admin.customerBank;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;

import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;
import cn.iocoder.yudao.module.customer.vo.customerBank.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerBank.CustomerBankDO;
import cn.iocoder.yudao.module.customer.convert.customerBank.CustomerBankConvert;
import cn.iocoder.yudao.module.customer.service.customerBank.CustomerBankService;

@Validated
@RestController
@Api(tags = "管理后台 - 客户银行账户")
@RequestMapping("/customer/bank")
public class CustomerBankController {

    @Resource
    private CustomerBankService customerBankService;

    @PostMapping("/create")
    @ApiOperation("创建客户银行账户")
    @PreAuthorize("@ss.hasPermission('customer:bank:create')")
    public CommonResult<Long> createBank(@Valid @RequestBody CustomerBankCreateReqVO createReqVO) {
        return success(customerBankService.createBank(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新客户银行账户")
    @PreAuthorize("@ss.hasPermission('customer:bank:update')")
    public CommonResult<Boolean> updateBank(@Valid @RequestBody CustomerBankUpdateReqVO updateReqVO) {
        customerBankService.updateBank(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除客户银行账户")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('customer:bank:delete')")
    public CommonResult<Boolean> deleteBank(@RequestParam("id") Long id) {
        customerBankService.deleteBank(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得客户银行账户")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('customer:bank:query')")
    public CommonResult<CustomerBankBackVO> getBank(@RequestParam("id") Long id) {
        CustomerBankDO bank = customerBankService.getBank(id);
        return success(CustomerBankConvert.INSTANCE.convert(bank));
    }

    @GetMapping("/list")
    @ApiOperation("获得客户银行账户列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('customer:bank:query')")
    public CommonResult<List<CustomerBankBackVO>> getBankList(@RequestParam("ids") Collection<Long> ids) {
        List<CustomerBankDO> list = customerBankService.getBankList(ids);
        return success(CustomerBankConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得客户银行账户分页")
    @PreAuthorize("@ss.hasPermission('customer:bank:query')")
    public CommonResult<PageResult<CustomerBankBackVO>> getBankPage(@Valid CustomerBankQueryVO query, PageVO page) {
        PageResult<CustomerBankDO> pageResult = customerBankService.getBankPage(query, page);
        return success(CustomerBankConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出客户银行账户 Excel")
    @PreAuthorize("@ss.hasPermission('customer:bank:export')")
    @OperateLog(type = EXPORT)
    public void exportBankExcel(@Valid CustomerBankQueryVO query,
              HttpServletResponse response) throws IOException {
        List<CustomerBankDO> list = customerBankService.getBankList(query);
        // 导出 Excel
        List<CustomerBankBackVO> datas = CustomerBankConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "客户银行账户.xls", "数据", CustomerBankBackVO.class, datas);
    }
}
