package cn.iocoder.yudao.module.customer.controller.admin.customerContacts;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.customer.convert.customerContacts.CustomerContactsConvert;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.CustomerDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerContacts.CustomerContactsDO;
import cn.iocoder.yudao.module.customer.dto.CustomerContactsDto;
import cn.iocoder.yudao.module.customer.service.customer.CustomerService;
import cn.iocoder.yudao.module.customer.service.customerContacts.CustomerContactsService;
import cn.iocoder.yudao.module.customer.vo.customer.customerContacts.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.Collection;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;

@Api(tags = "管理后台 - 客户联系人")
@RestController
@RequestMapping("/ecw/customer-contacts")
@Validated
public class CustomerContactsController {

    @Autowired
    private CustomerContactsService customerContactsService;

    @Autowired
    private CustomerService customerService;

    @PostMapping("/create")
    @ApiOperation("创建客户联系人")
    // @PreAuthorize("@ss.hasPermission('ecw:customer-contacts:create')")
    public CommonResult<Long> createCustomerContacts(@Valid @RequestBody CustomerContactsCreateReqVO createReqVO) {
        return success(customerContactsService.createCustomerContacts(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新客户联系人")
    // @PreAuthorize("@ss.hasPermission('ecw:customer-contacts:update')")
    public CommonResult<Boolean> updateCustomerContacts(@Valid @RequestBody CustomerContactsUpdateReqVO updateReqVO) {
        customerContactsService.updateCustomerContacts(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除客户联系人")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:customer-contacts:delete')")
    public CommonResult<Boolean> deleteCustomerContacts(@RequestParam("id") Long id) {
        customerContactsService.deleteCustomerContacts(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得客户联系人")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    // @PreAuthorize("@ss.hasPermission('ecw:customer-contacts:query')")
    public CommonResult<CustomerContactsRespVO> getCustomerContacts(@RequestParam("id") Long id) {
        CustomerContactsDO customerContacts = customerContactsService.getCustomerContacts(id);
        CustomerContactsRespVO respVo = CustomerContactsConvert.INSTANCE.convert(customerContacts);
        if (respVo != null) {
            CustomerDO customer = customerService.getCustomer(respVo.getCustomerId());
            if (customer != null) {
                respVo.setCustomerName(customer.getName());
                respVo.setCompany(customer.getCompany());
            }
        }
        return success(respVo);
    }

    @GetMapping("/list")
    @ApiOperation("获得客户联系人列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    // @PreAuthorize("@ss.hasPermission('ecw:customer-contacts:query')")
    public CommonResult<List<CustomerContactsRespVO>> getCustomerContactsList(@RequestParam("ids") Collection<Long> ids) {
        List<CustomerContactsDO> list = customerContactsService.getCustomerContactsListByCustomerId(ids);
        return success(CustomerContactsConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/list-by-customer")
    @ApiOperation("根据客户ID获得联系人列表")
    @ApiImplicitParam(name = "customerId", value = "客户主键ID", required = true, example = "1024", dataTypeClass = Long.class)
    // @PreAuthorize("@ss.hasPermission('ecw:customer-contacts:query')")
    public CommonResult<List<CustomerContactsRespVO>>
    getCustomerContactsListByCustomerId(@RequestParam("customerId") Long customerId) {
        List<CustomerContactsDO> list =
                customerContactsService.getCustomerContactsListByCustomerId(customerId);
        List<CustomerContactsRespVO> respVOList =
                CustomerContactsConvert.INSTANCE.convertList(list);
        if (CollectionUtil.isNotEmpty(respVOList)) {
            CustomerDO customer = customerService.getById(customerId);
            if (customer != null) {
                respVOList.stream().forEach(customerContactsRespVO -> {
                    customerContactsRespVO.setCompany(customer.getCompany());
                });
            }
        }
        return success(respVOList);
    }

    //获取客户联系人信息
    @GetMapping("/getContactsString")
    @ApiOperation("根据客户ID获得联系人列表")
    public CommonResult getCustomerContactsListByCustomerIdString(
            @RequestParam("customerId") Long customerId) {
        List<CustomerContactsDO> list =
                customerContactsService.getCustomerContactsListByCustomerId(customerId);
        List<CustomerContactsRespVO> respVOList =
                CustomerContactsConvert.INSTANCE.convertList(list);
        String sResult = "";
        if (CollectionUtil.isEmpty(respVOList)) {
            return success(sResult);
        }
        for (CustomerContactsRespVO m : respVOList
        ) {
            sResult += m.getName() + "," + m.getPhoneNew()+"\r\n";
        }
        //sResult = sResult.substring(sResult.length() - 4);
        return success(sResult);
    }


    @GetMapping("/page")
    @ApiOperation("获得客户联系人分页")
    // @PreAuthorize("@ss.hasPermission('ecw:customer-contacts:query')")
    public CommonResult<PageResult<CustomerContactsRespVO>> getCustomerContactsPage(@Valid CustomerContactsPageReqVO pageVO) {
        PageResult<CustomerContactsDO> pageResult = customerContactsService.getCustomerContactsPage(pageVO);
        return success(CustomerContactsConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出客户联系人 Excel")
    // @PreAuthorize("@ss.hasPermission('ecw:customer-contacts:export')")
    @OperateLog(type = EXPORT)
    public void exportCustomerContactsExcel(@Valid CustomerContactsExportReqVO exportReqVO,
                                            HttpServletResponse response) throws IOException {
        List<CustomerContactsDO> list = customerContactsService.getCustomerContactsListByCustomerId(exportReqVO);
        // 导出 Excel
        List<CustomerContactsExcelVO> datas = CustomerContactsConvert.INSTANCE.convertList02(list);
        ExcelUtils.write(response, "客户联系人.xls", "数据", CustomerContactsExcelVO.class, datas);
    }


    @GetMapping("/select")
    @ApiOperation("获得客户联系人下拉框列表(单次最多显示20条，请输入联系人电话、联系人姓名、客户名称搜索)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "searchKey", value = "请输入联系人电话、联系人姓名、客户名称搜索", example = "姓名或电话", dataTypeClass = String.class),
            @ApiImplicitParam(name = "type", value = "客户类型", example = "姓名或电话", dataTypeClass = String.class),
            @ApiImplicitParam(name = "ids", value = "编号列表", example = "1024,2048", dataTypeClass = List.class),
            @ApiImplicitParam(name = "customerId", value = "客户ID", example = "1024", dataTypeClass = Long.class)
    })
    public CommonResult<PageResult<CustomerContactsDto>> selectBySearchKey(
            @RequestParam(value = "searchKey", required = false) String searchKey,
            @RequestParam(value = "type", required = false) String type,
            @RequestParam(value = "ids", required = false) Collection<Long> ids,
            @RequestParam(value = "customerId", required = false) Long customerId
            , PageParam page) {

        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        return success(customerContactsService.selectBySearchKey(searchKey,
                type, ids, customerId, page));
    }

    /*
     添加的新的功能接口
     用于报价单逻辑控制
     */
    @GetMapping("/select2")
    @ApiOperation("获得客户联系人下拉框列表(单次最多显示20条，请输入联系人电话、联系人姓名、客户名称搜索)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "searchKey", value = "请输入联系人电话、联系人姓名、客户名称搜索", example = "姓名或电话", dataTypeClass = String.class),
            @ApiImplicitParam(name = "type", value = "客户类型", example = "姓名或电话", dataTypeClass = String.class),
            @ApiImplicitParam(name = "ids", value = "编号列表", example = "1024,2048", dataTypeClass = List.class),
            @ApiImplicitParam(name = "customerId", value = "客户ID", example = "1024", dataTypeClass = Long.class)
    })
    public CommonResult<PageResult<CustomerContactsDto>> selectBySearchKey2(
            @RequestParam(value = "searchKey", required = false) String searchKey,
            @RequestParam(value = "type", required = false) String type,
            @RequestParam(value = "ids", required = false) Collection<Long> ids,
            @RequestParam(value = "customerId", required = false) Long customerId
            , PageParam page) {

        //修改报价单联系人选择
        /*
        return success(customerContactsService.selectBySearchKey(searchKey,
                type,  ids, customerId, page));*/
        //当前用户ID
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        return success(customerContactsService.selectBySearchKey2(searchKey,
                type, ids, customerId, page, loginUserId));
    }

}
