package cn.iocoder.yudao.module.customer.controller.admin.customerCredit;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;
import cn.iocoder.yudao.module.customer.vo.customerCredit.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerCredit.CustomerCreditDO;
import cn.iocoder.yudao.module.customer.convert.customerCredit.CustomerCreditConvert;
import cn.iocoder.yudao.module.customer.service.customerCredit.CustomerCreditService;

@Validated
@RestController
@Api(tags = "管理后台 - 信用等级")
@RequestMapping("/customer/credit")
public class CustomerCreditController {

    @Resource
    private CustomerCreditService creditService;

    @PostMapping("/create")
    @ApiOperation("创建信用等级")
    //@PreAuthorize("@ss.hasPermission('customer:credit:create')")
    public CommonResult<Long> createCredit(@Valid @RequestBody CustomerCreditCreateReqVO createReqVO) {
        return success(creditService.createCredit(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新信用等级")
    //@PreAuthorize("@ss.hasPermission('customer:credit:update')")
    public CommonResult<Boolean> updateCredit(@Valid @RequestBody CustomerCreditUpdateReqVO updateReqVO) {
        creditService.updateCredit(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除信用等级")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('customer:credit:delete')")
    public CommonResult<Boolean> deleteCredit(@RequestParam("id") Long id) {
        creditService.deleteCredit(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得信用等级")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('customer:credit:query')")
    public CommonResult<CustomerCreditBackVO> getCredit(@RequestParam("id") Long id) {
        CustomerCreditDO credit = creditService.getCredit(id);
        return success(CustomerCreditConvert.INSTANCE.convert(credit));
    }

    @GetMapping("/list")
    @ApiOperation("获得信用等级列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    //@PreAuthorize("@ss.hasPermission('customer:credit:query')")
    public CommonResult<List<CustomerCreditBackVO>> getCreditList(@RequestParam("ids") Collection<Long> ids) {
        List<CustomerCreditDO> list = creditService.getCreditList(ids);
        return success(CustomerCreditConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得信用等级分页")
    //@PreAuthorize("@ss.hasPermission('customer:credit:query')")
    public CommonResult<PageResult<CustomerCreditBackVO>> getCreditPage(@Valid CustomerCreditQueryVO query, PageVO page) {
        PageResult<CustomerCreditDO> pageResult = creditService.getCreditPage(query, page);
        return success(CustomerCreditConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出信用等级 Excel")
    //@PreAuthorize("@ss.hasPermission('customer:credit:export')")
    @OperateLog(type = EXPORT)
    public void exportCreditExcel(@Valid CustomerCreditQueryVO query,
              HttpServletResponse response) throws IOException {
        List<CustomerCreditDO> list = creditService.getCreditList(query);
        // 导出 Excel
        List<CustomerCreditBackVO> datas = CustomerCreditConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "信用等级.xls", "数据", CustomerCreditBackVO.class, datas);
    }
}
