package cn.iocoder.yudao.module.customer.controller.admin.customerCreditLog;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;
import cn.iocoder.yudao.module.customer.vo.customerCreditLog.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerCreditLog.CustomerCreditLogDO;
import cn.iocoder.yudao.module.customer.convert.customerCreditLog.CustomerCreditLogConvert;
import cn.iocoder.yudao.module.customer.service.customerCreditLog.CustomerCreditLogService;

@Validated
@RestController
@Api(tags = "管理后台 - 客户信用日志")
@RequestMapping("/customer/credit-log")
public class CustomerCreditLogController {

    @Resource
    private CustomerCreditLogService creditLogService;

    @PostMapping("/create")
    @ApiOperation("创建客户信用日志")
    //@PreAuthorize("@ss.hasPermission('customer:credit-log:create')")
    public CommonResult<Long> createCreditLog(@Valid @RequestBody CustomerCreditLogCreateReqVO createReqVO) {
        return success(creditLogService.createCreditLog(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新客户信用日志")
    //@PreAuthorize("@ss.hasPermission('customer:credit-log:update')")
    public CommonResult<Boolean> updateCreditLog(@Valid @RequestBody CustomerCreditLogUpdateReqVO updateReqVO) {
        creditLogService.updateCreditLog(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除客户信用日志")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('customer:credit-log:delete')")
    public CommonResult<Boolean> deleteCreditLog(@RequestParam("id") Long id) {
        creditLogService.deleteCreditLog(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得客户信用日志")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('customer:credit-log:query')")
    public CommonResult<CustomerCreditLogBackVO> getCreditLog(@RequestParam("id") Long id) {
        CustomerCreditLogDO creditLog = creditLogService.getCreditLog(id);
        return success(CustomerCreditLogConvert.INSTANCE.convert(creditLog));
    }

    @GetMapping("/list")
    @ApiOperation("获得客户信用日志列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    //@PreAuthorize("@ss.hasPermission('customer:credit-log:query')")
    public CommonResult<List<CustomerCreditLogBackVO>> getCreditLogList(@RequestParam("ids") Collection<Long> ids) {
        List<CustomerCreditLogDO> list = creditLogService.getCreditLogList(ids);
        return success(CustomerCreditLogConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得客户信用日志分页")
    //@PreAuthorize("@ss.hasPermission('customer:credit-log:query')")
    public CommonResult<PageResult<CustomerCreditLogBackVO>> getCreditLogPage(@Valid CustomerCreditLogQueryVO query, PageVO page) {
        return success(creditLogService.getCreditLogPage(query, page));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出客户信用日志 Excel")
    //@PreAuthorize("@ss.hasPermission('customer:credit-log:export')")
    @OperateLog(type = EXPORT)
    public void exportCreditLogExcel(@Valid CustomerCreditLogQueryVO query,
              HttpServletResponse response) throws IOException {
        List<CustomerCreditLogDO> list = creditLogService.getCreditLogList(query);
        // 导出 Excel
        List<CustomerCreditLogBackVO> datas = CustomerCreditLogConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "客户信用日志.xls", "数据", CustomerCreditLogBackVO.class, datas);
    }
}
