package cn.iocoder.yudao.module.customer.controller.admin.customerCreditRule;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;
import cn.iocoder.yudao.module.customer.vo.customerCreditRule.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerCreditRule.CustomerCreditRuleDO;
import cn.iocoder.yudao.module.customer.convert.customerCreditRule.CustomerCreditRuleConvert;
import cn.iocoder.yudao.module.customer.service.customerCreditRule.CustomerCreditRuleService;

@Validated
@RestController
@Api(tags = "管理后台 - 信用规则")
@RequestMapping("/customer/credit-rule")
public class CustomerCreditRuleController {

    @Resource
    private CustomerCreditRuleService creditRuleService;

    @PostMapping("/create")
    @ApiOperation("创建信用规则")
    //@PreAuthorize("@ss.hasPermission('customer:credit-rule:create')")
    public CommonResult<Long> createCreditRule(@Valid @RequestBody CustomerCreditRuleCreateReqVO createReqVO) {
        return success(creditRuleService.createCreditRule(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新信用规则")
    //@PreAuthorize("@ss.hasPermission('customer:credit-rule:update')")
    public CommonResult<Boolean> updateCreditRule(@Valid @RequestBody CustomerCreditRuleUpdateReqVO updateReqVO) {
        creditRuleService.updateCreditRule(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除信用规则")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('customer:credit-rule:delete')")
    public CommonResult<Boolean> deleteCreditRule(@RequestParam("id") Long id) {
        creditRuleService.deleteCreditRule(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得信用规则")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('customer:credit-rule:query')")
    public CommonResult<CustomerCreditRuleBackVO> getCreditRule(@RequestParam("id") Long id) {
        CustomerCreditRuleDO creditRule = creditRuleService.getCreditRule(id);
        return success(CustomerCreditRuleConvert.INSTANCE.convert(creditRule));
    }

    @GetMapping("/list")
    @ApiOperation("获得信用规则列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    //@PreAuthorize("@ss.hasPermission('customer:credit-rule:query')")
    public CommonResult<List<CustomerCreditRuleBackVO>> getCreditRuleList(@RequestParam("ids") Collection<Long> ids) {
        List<CustomerCreditRuleDO> list = creditRuleService.getCreditRuleList(ids);
        return success(CustomerCreditRuleConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得信用规则分页")
    //@PreAuthorize("@ss.hasPermission('customer:credit-rule:query')")
    public CommonResult<PageResult<CustomerCreditRuleBackVO>> getCreditRulePage(@Valid CustomerCreditRuleQueryVO query, PageVO page) {
        PageResult<CustomerCreditRuleDO> pageResult = creditRuleService.getCreditRulePage(query, page);
        return success(CustomerCreditRuleConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出信用规则 Excel")
    //@PreAuthorize("@ss.hasPermission('customer:credit-rule:export')")
    @OperateLog(type = EXPORT)
    public void exportCreditRuleExcel(@Valid CustomerCreditRuleQueryVO query,
              HttpServletResponse response) throws IOException {
        List<CustomerCreditRuleDO> list = creditRuleService.getCreditRuleList(query);
        // 导出 Excel
        List<CustomerCreditRuleBackVO> datas = CustomerCreditRuleConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "信用规则.xls", "数据", CustomerCreditRuleBackVO.class, datas);
    }
}
