package cn.iocoder.yudao.module.customer.controller.admin.customerDetail;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.customer.service.customerCreditLog.CustomerCreditLogService;
import cn.iocoder.yudao.module.customer.service.customerDetail.CustomerDetailService;
import cn.iocoder.yudao.module.customer.service.customerLevelLog.CustomerLevelLogService;
import cn.iocoder.yudao.module.customer.vo.customerCreditLog.CustomerCreditLogBackVO;
import cn.iocoder.yudao.module.customer.vo.customerDetail.*;
import cn.iocoder.yudao.module.customer.vo.customerLevelLog.CustomerLevelLogBackVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Validated
@RestController
@Api(tags = "管理后台 - 客户基本信息- 信息列表 ")
@RequestMapping("/customer/detail/infoList")
public class CustomerDetailController {

    @Resource
    private CustomerDetailService customerDetailService;

    @Resource
    private CustomerLevelLogService customerLevelLogService;

    @Resource
    private CustomerCreditLogService customerCreditLogService;

    //@GetMapping("/orderPage")
    //@ApiOperation("获取订单分页")
    //public CommonResult<PageResult<CustomerOrderBackVo>> getOrderPage(@Valid CustomerOrderQueryVO query) {
    //    PageResult<CustomerOrderBackVo> pageResult = customerDetailService.getCustomerOrderPage(query);
    //    return success(pageResult);
    //}

    @GetMapping("/offerPage")
    @ApiOperation("获取报价分页")
    public CommonResult<PageResult<CustomerOfferBackVo>> getOfferPage(CustomerOfferQueryVo offerQueryVo, PageParam page) {
        PageResult<CustomerOfferBackVo> pageResult =
                customerDetailService.getCustomerOfferPage(offerQueryVo, page);
        return success(pageResult);
    }

    @GetMapping("/orderStatistics")
    @ApiOperation("获取数据")
    public CommonResult<CustomerOrderStatisticsBackVo> orderStatistics(@RequestParam("customerId") Long customerId) {
        CustomerOrderStatisticsBackVo backVo = new CustomerOrderStatisticsBackVo();
        backVo.setAllOrderTotal(customerDetailService.getOrderCountByControl(customerId, null));
        backVo.setControlOrderTotal(customerDetailService.getOrderCountByControl(customerId, true));
        backVo.setHypgTotal(customerDetailService.getOrderCountByTransportId(customerId, 1l));
        backVo.setHyzgTotal(customerDetailService.getOrderCountByTransportId(customerId, 2l));
        backVo.setZxkyTotal(customerDetailService.getOrderCountByTransportId(customerId, 3l));
        backVo.setHklyTotal(customerDetailService.getOrderCountByTransportId(customerId, 4l));
        OrderDto orderDto = customerDetailService.getLastOrder(customerId);
        if (null != orderDto) {
            backVo.setLastBusinessOrderNo(orderDto.getOrderNo());
            backVo.setLastBusinessDate(orderDto.getCreateTime());
        }
        backVo.setOfferOrderTotal(customerDetailService.getBaojiaCount(customerId));
        backVo.setPickOrderTotal(customerDetailService.getXiadanCount(customerId));
        backVo.setWareHousingTotal(customerDetailService.getRucangCount(customerId));
        backVo.setShipmentTotal(customerDetailService.getChuhuoCount(customerId));
        Long startStock = customerDetailService.getStartWarehouseCount(customerId);
        Long endStock = customerDetailService.getEndWarehouseCount(customerId);
        backVo.setStock((null == startStock ? 0 : startStock) + "/" + (null == endStock ? 0 : endStock));
        return success(backVo);
    }

    @GetMapping("/receiptPage")
    @ApiOperation("获取账单分页")
    public CommonResult<PageResult<CustomerReceivableBackVO>> getReceiptPage(@RequestParam("customerId") Long customerId, PageParam page) {
        PageResult<CustomerReceivableBackVO> pageResult =
                customerDetailService.getReceiptPage(customerId, page);
        return success(pageResult);
    }


    @GetMapping("/followPage")
    @ApiOperation("获取跟进分页")
    @ApiImplicitParam(name = "customerId", value = "客户ID", required = true, dataType = "Long")
    public CommonResult<PageResult<CustomerFollowBackVo>> getFollwPage(@RequestParam("customerId") Long customerId, PageParam page) {
        PageResult<CustomerFollowBackVo> pageResult =
                customerDetailService.getFollowPage(customerId, page);
        return success(pageResult);
    }

    @GetMapping("/complaintPage")
    @ApiOperation("获取客诉分页")
    public CommonResult<PageResult<CustomerComplaintBackVO>> getComplaintPage(@RequestParam("customerId") Long customerId, PageParam page) {
        PageResult<CustomerComplaintBackVO> pageResult = customerDetailService.getComplaintPage(customerId, page);
        return success(pageResult);
    }

    @GetMapping("/levelLogPage")
    @ApiOperation("获得客户等级日志分页")
    public CommonResult<PageResult<CustomerLevelLogBackVO>> getLevelLogPage(
            @RequestParam("customerId") Long customerId, PageParam page) {
        return success(customerLevelLogService.getLevelLogPage(customerId, page));
    }

    @GetMapping("/creditScoreStatistic")
    @ApiOperation("获得客户信用统计")
    public CommonResult<List<CustomerCreditRuleStatisticBackVo>> getCreditScore(@RequestParam("customerId") Long customerId) {
        List<CustomerCreditRuleStatisticBackVo> list = customerCreditLogService.getCreditRuleStatistic(customerId);
        CustomerCreditRuleStatisticBackVo v1 = new CustomerCreditRuleStatisticBackVo();
        v1.setType("all");
        //加上初始信用分
        BigDecimal sum = customerCreditLogService.getCreditRuleStatisticAll(customerId);
        v1.setScore(BigDecimal.valueOf(400).add(sum == null ? BigDecimal.ZERO : sum));
        list.add(v1);
        return success(list);
    }

    @GetMapping("/creditLogPage")
    @ApiOperation("获得客户信用日志分页")
    public CommonResult<PageResult<CustomerCreditLogBackVO>> getCreditLogPage(
            @RequestParam("customerId") Long customerId, PageParam page) {
        return success(customerCreditLogService.getCreditLogPage(customerId, page));
    }
}
