package cn.iocoder.yudao.module.customer.controller.admin.customerFollowup;

import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.customer.dto.CustomerExportReqDTO;
import cn.iocoder.yudao.module.system.api.file.FileMakeApi;
import cn.iocoder.yudao.module.system.api.file.dto.FileMakeReqDTO;
import cn.iocoder.yudao.module.system.enums.download.DownloadTypeEnum;
import com.alibaba.fastjson.JSONObject;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;
import cn.iocoder.yudao.module.customer.vo.customerFollowup.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerFollowup.CustomerFollowupDO;
import cn.iocoder.yudao.module.customer.convert.customerFollowup.CustomerFollowupConvert;
import cn.iocoder.yudao.module.customer.service.customerFollowup.CustomerFollowupService;

@Validated
@RestController
@Api(tags = "管理后台 - 客户跟进-新")
@RequestMapping("/customer/followup")
public class CustomerFollowupController {

    @Resource
    private CustomerFollowupService followupService;



    @Resource
    private FileMakeApi fileMakeApi;

    @PostMapping("/create")
    @ApiOperation("创建客户跟进")
    // @PreAuthorize("@ss.hasPermission('customer:followup:create')")
    public CommonResult<Long> createFollowup(@Valid @RequestBody CustomerFollowupCreateReqVO createReqVO) {
        return success(followupService.createFollowup(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新客户跟进")
    // @PreAuthorize("@ss.hasPermission('customer:followup:update')")
    public CommonResult<Boolean> updateFollowup(@Valid @RequestBody CustomerFollowupUpdateReqVO updateReqVO) {
        followupService.updateFollowup(updateReqVO);
        return success(true);
    }

    @PutMapping("/update-status")
    @ApiOperation("更新客户跟进状态")
    // @PreAuthorize("@ss.hasPermission('customer:followup:update')")
    public CommonResult<Boolean> updateFollowupStatus(@Valid @RequestBody CustomerFollowupUpdateStatusReqVO updateStatusReqVO) {
        followupService.updateFollowupStatus(updateStatusReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除客户跟进")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    // @PreAuthorize("@ss.hasPermission('customer:followup:delete')")
    public CommonResult<Boolean> deleteFollowup(@RequestParam("id") Long id) {
        followupService.deleteFollowup(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得客户跟进")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    // @PreAuthorize("@ss.hasPermission('customer:followup:query')")
    public CommonResult<CustomerFollowupBackVO> getFollowup(@RequestParam("id") Long id) {
        CustomerFollowupDO followup = followupService.getFollowup(id);
        return success(CustomerFollowupConvert.INSTANCE.convert(followup));
    }

    @GetMapping("/list")
    @ApiOperation("获得客户跟进列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    // @PreAuthorize("@ss.hasPermission('customer:followup:query')")
    public CommonResult<List<CustomerFollowupBackVO>> getFollowupList(@RequestParam("ids") Collection<Long> ids) {
        List<CustomerFollowupDO> list = followupService.getFollowupList(ids);
        return success(CustomerFollowupConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得客户跟进分页")
    // @PreAuthorize("@ss.hasPermission('customer:followup:query')")
    public CommonResult<PageResult<CustomerFollowupBackVO>> getFollowupPage(@Valid CustomerFollowupQueryVO query, PageVO page) {
        PageResult<CustomerFollowupDO> pageResult = followupService.getFollowupPage(query, page);
        return success(CustomerFollowupConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出客户跟进 Excel")
    // @PreAuthorize("@ss.hasPermission('customer:followup:export')")
    @OperateLog(type = EXPORT)
    public CommonResult<Boolean> exportFollowupExcel(@Valid CustomerFollowupQueryVO query,
              HttpServletResponse response) throws IOException {
        sendFileMake(query, DownloadTypeEnum.CUSTOMER_FOLLOWUP_EXPORT, "跟进纪录导出Excel");
        return success(true);
    }

    /**
     * 支持多选
     *
     * @param query
     * @param downloadTypeEnum
     * @param fileName
     */
    private void sendFileMake(CustomerFollowupQueryVO query, DownloadTypeEnum downloadTypeEnum, String fileName) {
        FileMakeReqDTO reqDTO = new FileMakeReqDTO();
        reqDTO.setType(downloadTypeEnum.getType());
        reqDTO.setName(fileName);
        reqDTO.setFileSuffix("xlsx");
        reqDTO.setTemporaryFile(true);
        reqDTO.setUserType(2);
        reqDTO.setLang(I18nMessage.getLang());
        reqDTO.setRequestParams(JSONObject.toJSONString(query));
        fileMakeApi.sendFileMake(reqDTO);
    }


}
