package cn.iocoder.yudao.module.customer.controller.admin.customerLevel;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;
import cn.iocoder.yudao.module.customer.vo.customerLevel.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerLevel.CustomerLevelDO;
import cn.iocoder.yudao.module.customer.convert.customerLevel.CustomerLevelConvert;
import cn.iocoder.yudao.module.customer.service.customerLevel.CustomerLevelService;

@Validated
@RestController
@Api(tags = "管理后台 - 客户级别")
@RequestMapping("/customer/level")
public class CustomerLevelController {

    @Resource
    private CustomerLevelService levelService;

    @PostMapping("/create")
    @ApiOperation("创建客户级别")
    //@PreAuthorize("@ss.hasPermission('customer:level:create')")
    public CommonResult<Long> createLevel(@Valid @RequestBody CustomerLevelCreateReqVO createReqVO) {
        return success(levelService.createLevel(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新客户级别")
    //@PreAuthorize("@ss.hasPermission('customer:level:update')")
    public CommonResult<Boolean> updateLevel(@Valid @RequestBody CustomerLevelUpdateReqVO updateReqVO) {
        levelService.updateLevel(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除客户级别")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('customer:level:delete')")
    public CommonResult<Boolean> deleteLevel(@RequestParam("id") Long id) {
        levelService.deleteLevel(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得客户级别")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('customer:level:query')")
    public CommonResult<CustomerLevelBackVO> getLevel(@RequestParam("id") Long id) {
        CustomerLevelDO level = levelService.getLevel(id);
        return success(CustomerLevelConvert.INSTANCE.convert(level));
    }

    @GetMapping("/list")
    @ApiOperation("获得客户级别列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    //@PreAuthorize("@ss.hasPermission('customer:level:query')")
    public CommonResult<List<CustomerLevelBackVO>> getLevelList(@RequestParam("ids") Collection<Long> ids) {
        List<CustomerLevelDO> list = levelService.getLevelList(ids);
        return success(CustomerLevelConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得客户级别分页")
    //@PreAuthorize("@ss.hasPermission('customer:level:query')")
    public CommonResult<PageResult<CustomerLevelBackVO>> getLevelPage(@Valid CustomerLevelQueryVO query, PageVO page) {
        PageResult<CustomerLevelDO> pageResult = levelService.getLevelPage(query, page);
        return success(CustomerLevelConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出客户级别 Excel")
    //@PreAuthorize("@ss.hasPermission('customer:level:export')")
    @OperateLog(type = EXPORT)
    public void exportLevelExcel(@Valid CustomerLevelQueryVO query,
              HttpServletResponse response) throws IOException {
        List<CustomerLevelDO> list = levelService.getLevelList(query);
        // 导出 Excel
        List<CustomerLevelBackVO> datas = CustomerLevelConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "客户级别.xls", "数据", CustomerLevelBackVO.class, datas);
    }
}
