package cn.iocoder.yudao.module.customer.controller.admin.customerLevelLog;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;
import cn.iocoder.yudao.module.customer.vo.customerLevelLog.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerLevelLog.CustomerLevelLogDO;
import cn.iocoder.yudao.module.customer.convert.customerLevelLog.CustomerLevelLogConvert;
import cn.iocoder.yudao.module.customer.service.customerLevelLog.CustomerLevelLogService;

@Validated
@RestController
@Api(tags = "管理后台 - 客户等级日志")
@RequestMapping("/customer/level-log")
public class CustomerLevelLogController {

    @Resource
    private CustomerLevelLogService levelLogService;

    @PostMapping("/create")
    @ApiOperation("创建客户等级日志")
    //@PreAuthorize("@ss.hasPermission('customer:level-log:create')")
    public CommonResult<Long> createLevelLog(@Valid @RequestBody CustomerLevelLogCreateReqVO createReqVO) {
        return success(levelLogService.createLevelLog(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新客户等级日志")
    //@PreAuthorize("@ss.hasPermission('customer:level-log:update')")
    public CommonResult<Boolean> updateLevelLog(@Valid @RequestBody CustomerLevelLogUpdateReqVO updateReqVO) {
        levelLogService.updateLevelLog(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除客户等级日志")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('customer:level-log:delete')")
    public CommonResult<Boolean> deleteLevelLog(@RequestParam("id") Long id) {
        levelLogService.deleteLevelLog(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得客户等级日志")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('customer:level-log:query')")
    public CommonResult<CustomerLevelLogBackVO> getLevelLog(@RequestParam("id") Long id) {
        CustomerLevelLogDO levelLog = levelLogService.getLevelLog(id);
        return success(CustomerLevelLogConvert.INSTANCE.convert(levelLog));
    }

    @GetMapping("/list")
    @ApiOperation("获得客户等级日志列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    //@PreAuthorize("@ss.hasPermission('customer:level-log:query')")
    public CommonResult<List<CustomerLevelLogBackVO>> getLevelLogList(@RequestParam("ids") Collection<Long> ids) {
        List<CustomerLevelLogDO> list = levelLogService.getLevelLogList(ids);
        return success(CustomerLevelLogConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得客户等级日志分页")
    //@PreAuthorize("@ss.hasPermission('customer:level-log:query')")
    public CommonResult<PageResult<CustomerLevelLogBackVO>> getLevelLogPage(@Valid CustomerLevelLogQueryVO query, PageVO page) {
        return success(levelLogService.getLevelLogPage(query, page));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出客户等级日志 Excel")
    //@PreAuthorize("@ss.hasPermission('customer:level-log:export')")
    @OperateLog(type = EXPORT)
    public void exportLevelLogExcel(@Valid CustomerLevelLogQueryVO query,
              HttpServletResponse response) throws IOException {
        List<CustomerLevelLogDO> list = levelLogService.getLevelLogList(query);
        // 导出 Excel
        List<CustomerLevelLogBackVO> datas = CustomerLevelLogConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "客户等级日志.xls", "数据", CustomerLevelLogBackVO.class, datas);
    }
}
