package cn.iocoder.yudao.module.customer.controller.admin.customerLevelRule;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;
import cn.iocoder.yudao.module.customer.vo.customerLevelRule.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerLevelRule.CustomerLevelRuleDO;
import cn.iocoder.yudao.module.customer.convert.customerLevelRule.CustomerLevelRuleConvert;
import cn.iocoder.yudao.module.customer.service.customerLevelRule.CustomerLevelRuleService;

@Validated
@RestController
@Api(tags = "管理后台 - 客户级别规则")
@RequestMapping("/customer/level-rule")
public class CustomerLevelRuleController {

    @Resource
    private CustomerLevelRuleService levelRuleService;

    @PostMapping("/create")
    @ApiOperation("创建客户级别规则")
    //@PreAuthorize("@ss.hasPermission('customer:level-rule:create')")
    public CommonResult<Long> createLevelRule(@Valid @RequestBody CustomerLevelRuleCreateReqVO createReqVO) {
        return success(levelRuleService.createLevelRule(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新客户级别规则")
    //@PreAuthorize("@ss.hasPermission('customer:level-rule:update')")
    public CommonResult<Boolean> updateLevelRule(@Valid @RequestBody CustomerLevelRuleUpdateReqVO updateReqVO) {
        levelRuleService.updateLevelRule(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除客户级别规则")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('customer:level-rule:delete')")
    public CommonResult<Boolean> deleteLevelRule(@RequestParam("id") Long id) {
        levelRuleService.deleteLevelRule(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得客户级别规则")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('customer:level-rule:query')")
    public CommonResult<CustomerLevelRuleBackVO> getLevelRule(@RequestParam("id") Long id) {
        CustomerLevelRuleDO levelRule = levelRuleService.getLevelRule(id);
        return success(CustomerLevelRuleConvert.INSTANCE.convert(levelRule));
    }

    @GetMapping("/list")
    @ApiOperation("获得客户级别规则列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    //@PreAuthorize("@ss.hasPermission('customer:level-rule:query')")
    public CommonResult<List<CustomerLevelRuleBackVO>> getLevelRuleList(@RequestParam("ids") Collection<Long> ids) {
        List<CustomerLevelRuleDO> list = levelRuleService.getLevelRuleList(ids);
        return success(CustomerLevelRuleConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得客户级别规则分页")
    //@PreAuthorize("@ss.hasPermission('customer:level-rule:query')")
    public CommonResult<PageResult<CustomerLevelRuleBackVO>> getLevelRulePage(@Valid CustomerLevelRuleQueryVO query, PageVO page) {
        PageResult<CustomerLevelRuleDO> pageResult = levelRuleService.getLevelRulePage(query, page);
        return success(CustomerLevelRuleConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出客户级别规则 Excel")
    //@PreAuthorize("@ss.hasPermission('customer:level-rule:export')")
    @OperateLog(type = EXPORT)
    public void exportLevelRuleExcel(@Valid CustomerLevelRuleQueryVO query,
              HttpServletResponse response) throws IOException {
        List<CustomerLevelRuleDO> list = levelRuleService.getLevelRuleList(query);
        // 导出 Excel
        List<CustomerLevelRuleBackVO> datas = CustomerLevelRuleConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "客户级别规则.xls", "数据", CustomerLevelRuleBackVO.class, datas);
    }
}
