package cn.iocoder.yudao.module.customer.controller.admin.customerOperateLog;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;
import cn.iocoder.yudao.module.customer.vo.customerOperateLog.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerOperateLog.CustomerOperateLogDO;
import cn.iocoder.yudao.module.customer.convert.customerOperateLog.CustomerOperateLogConvert;
import cn.iocoder.yudao.module.customer.service.customerOperateLog.CustomerOperateLogService;

@Validated
@RestController
@Api(tags = "管理后台 - 客户操作日志")
@RequestMapping("/customer/operate-log")
public class CustomerOperateLogController {

    @Resource
    private CustomerOperateLogService operateLogService;

    @PostMapping("/create")
    @ApiOperation("创建客户操作日志")
    @PreAuthorize("@ss.hasPermission('customer:operate-log:create')")
    public CommonResult<Long> createOperateLog(@Valid @RequestBody CustomerOperateLogCreateReqVO createReqVO) {
        return success(operateLogService.createOperateLog(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新客户操作日志")
    @PreAuthorize("@ss.hasPermission('customer:operate-log:update')")
    public CommonResult<Boolean> updateOperateLog(@Valid @RequestBody CustomerOperateLogUpdateReqVO updateReqVO) {
        operateLogService.updateOperateLog(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除客户操作日志")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('customer:operate-log:delete')")
    public CommonResult<Boolean> deleteOperateLog(@RequestParam("id") Long id) {
        operateLogService.deleteOperateLog(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得客户操作日志")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('customer:operate-log:query')")
    public CommonResult<CustomerOperateLogBackVO> getOperateLog(@RequestParam("id") Long id) {
        CustomerOperateLogDO operateLog = operateLogService.getOperateLog(id);
        return success(CustomerOperateLogConvert.INSTANCE.convert(operateLog));
    }

    @GetMapping("/list")
    @ApiOperation("获得客户操作日志列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('customer:operate-log:query')")
    public CommonResult<List<CustomerOperateLogBackVO>> getOperateLogList(@RequestParam("ids") Collection<Long> ids) {
        List<CustomerOperateLogDO> list = operateLogService.getOperateLogList(ids);
        return success(CustomerOperateLogConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得客户操作日志分页")
    @PreAuthorize("@ss.hasPermission('customer:operate-log:query')")
    public CommonResult<PageResult<CustomerOperateLogBackVO>> getOperateLogPage(@Valid CustomerOperateLogQueryVO query, PageVO page) {
        PageResult<CustomerOperateLogDO> pageResult =
                operateLogService.getOperateLogPage(query, page);
        return success(CustomerOperateLogConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出客户操作日志 Excel")
    @PreAuthorize("@ss.hasPermission('customer:operate-log:export')")
    @OperateLog(type = EXPORT)
    public void exportOperateLogExcel(@Valid CustomerOperateLogQueryVO query,
              HttpServletResponse response) throws IOException {
        List<CustomerOperateLogDO> list = operateLogService.getOperateLogList(query);
        // 导出 Excel
        List<CustomerOperateLogBackVO> datas = CustomerOperateLogConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "客户操作日志.xls", "数据", CustomerOperateLogBackVO.class, datas);
    }
}
