package cn.iocoder.yudao.module.customer.controller.admin.customerPublicCatchLog;

import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;
import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;
import cn.iocoder.yudao.module.customer.vo.customerPublicCatchLog.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerPublicCatchLog.CustomerPublicCatchLogDO;
import cn.iocoder.yudao.module.customer.convert.customerPublicCatchLog.CustomerPublicCatchLogConvert;
import cn.iocoder.yudao.module.customer.service.customerPublicCatchLog.CustomerPublicCatchLogService;

@Validated
@RestController
@Api(tags = "管理后台 - 公海客户捞取日志")
@RequestMapping("/customer/public-catch-log")
public class CustomerPublicCatchLogController {

    @Resource
    private CustomerPublicCatchLogService publicCatchLogService;

    @PostMapping("/create")
    @ApiOperation("创建公海客户捞取日志")
    // @PreAuthorize("@ss.hasPermission('customer:public-catch-log:create')")
    public CommonResult<Long> createPublicCatchLog(@Valid @RequestBody CustomerPublicCatchLogCreateReqVO createReqVO) {
        return success(publicCatchLogService.createPublicCatchLog(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新公海客户捞取日志")
    // @PreAuthorize("@ss.hasPermission('customer:public-catch-log:update')")
    public CommonResult<Boolean> updatePublicCatchLog(@Valid @RequestBody CustomerPublicCatchLogUpdateReqVO updateReqVO) {
        publicCatchLogService.updatePublicCatchLog(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除公海客户捞取日志")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    // @PreAuthorize("@ss.hasPermission('customer:public-catch-log:delete')")
    public CommonResult<Boolean> deletePublicCatchLog(@RequestParam("id") Long id) {
        publicCatchLogService.deletePublicCatchLog(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得公海客户捞取日志")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    // @PreAuthorize("@ss.hasPermission('customer:public-catch-log:query')")
    public CommonResult<CustomerPublicCatchLogBackVO> getPublicCatchLog(@RequestParam("id") Long id) {
        CustomerPublicCatchLogDO publicCatchLog = publicCatchLogService.getPublicCatchLog(id);
        return success(CustomerPublicCatchLogConvert.INSTANCE.convert(publicCatchLog));
    }

    @GetMapping("/list")
    @ApiOperation("获得公海客户捞取日志列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    // @PreAuthorize("@ss.hasPermission('customer:public-catch-log:query')")
    public CommonResult<List<CustomerPublicCatchLogBackVO>> getPublicCatchLogList(@RequestParam("ids") Collection<Long> ids) {
        List<CustomerPublicCatchLogDO> list = publicCatchLogService.getPublicCatchLogList(ids);
        return success(CustomerPublicCatchLogConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得公海客户捞取日志分页")
    // @PreAuthorize("@ss.hasPermission('customer:public-catch-log:query')")
    public CommonResult<PageResult<CustomerPublicCatchLogBackVO>> getPublicCatchLogPage(@Valid CustomerPublicCatchLogQueryVO query, PageVO page) {
        PageResult<CustomerPublicCatchLogDO> pageResult = publicCatchLogService.getPublicCatchLogPage(query, page);
        return success(CustomerPublicCatchLogConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出公海客户捞取日志 Excel")
    // @PreAuthorize("@ss.hasPermission('customer:public-catch-log:export')")
    @OperateLog(type = EXPORT)
    public void exportPublicCatchLogExcel(@Valid CustomerPublicCatchLogQueryVO query,
              HttpServletResponse response) throws IOException {
        List<CustomerPublicCatchLogDO> list = publicCatchLogService.getPublicCatchLogList(query);
        // 导出 Excel
        List<CustomerPublicCatchLogBackVO> datas = CustomerPublicCatchLogConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "公海客户捞取日志.xls", "数据", CustomerPublicCatchLogBackVO.class, datas);
    }
}
