package cn.iocoder.yudao.module.customer.controller.admin.indirectCustomer;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.module.customer.convert.indirectCustomer.IndirectCustomerConvert;
import cn.iocoder.yudao.module.customer.dal.dataobject.indirectCustomer.IndirectCustomerDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.indirectCustomerContacts.IndirectCustomerContactsDO;
import cn.iocoder.yudao.module.customer.dto.indirectCustomer.IndirectCustomerPageReqDTO;
import cn.iocoder.yudao.module.customer.service.indirectCustomer.IndirectCustomerService;
import cn.iocoder.yudao.module.customer.service.indirectCustomerContacts.IndirectCustomerContactsService;
import cn.iocoder.yudao.module.customer.vo.customer.vo.CustomerExportReqVO;
import cn.iocoder.yudao.module.customer.vo.indirectCustomer.*;
import cn.iocoder.yudao.module.customer.vo.indirectCustomerContacts.IndirectCustomerContactsUpdateReqVO;
import cn.iocoder.yudao.module.ecw.enums.CustomerContactsDefaultEnum;
import cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants;
import cn.iocoder.yudao.module.system.api.file.FileMakeApi;
import cn.iocoder.yudao.module.system.api.file.dto.FileMakeReqDTO;
import cn.iocoder.yudao.module.system.enums.download.DownloadTypeEnum;
import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;

@Validated
@RestController
@Api(tags = "管理后台 - 间接客户")
@RequestMapping("/ecw/indirect-customer")
public class IndirectCustomerController {

    @Resource
    private IndirectCustomerService indirectCustomerService;

    @Resource
    private IndirectCustomerContactsService indirectCustomerContactsService;

    @Resource
    private FileMakeApi fileMakeApi;


    @PutMapping("/update")
    @ApiOperation("更新间接客户")
    // @PreAuthorize("@ss.hasPermission('customer:indirect-customer:update')")
    public CommonResult<Boolean> updateIndirectCustomer(@Valid @RequestBody IndirectCustomerUpdateReqVO updateReqVO) {
        List<IndirectCustomerContactsUpdateReqVO> customerContacts = updateReqVO.getCustomerContacts();
        if (!CollectionUtils.isEmpty(customerContacts)) {
            long count = customerContacts.stream().filter(t -> CustomerContactsDefaultEnum.IS_DEFAULT.getValue().equals(t.getIsDefault())).count();
            if (count <= 0) {
                return error(ErrorCodeConstants.CUSTOMER_CONTACTS_NO_DEFAULT);
            }
            if (count > 1) {
                return error(ErrorCodeConstants.CUSTOMER_CONTACTS_MULTI_DEFAULT);
            }


            //判断联系人手机已绑定其他客户，则提示
            for (IndirectCustomerContactsUpdateReqVO customerContact : customerContacts) {
                String phone = customerContact.getPhoneNew();

                IndirectCustomerContactsDO indirectCustomerContactsDO = indirectCustomerContactsService.getOne(new LambdaQueryWrapperX<IndirectCustomerContactsDO>()
                        .eq(IndirectCustomerContactsDO::getAreaCode, customerContact.getAreaCode())
                        .eq(IndirectCustomerContactsDO::getPhoneNew, customerContact.getPhoneNew())
                        .apply("customer_id is not null and customer_id != " + updateReqVO.getId(), null)
                        .orderByDesc(IndirectCustomerContactsDO::getId).last("limit 1"));
                if (indirectCustomerContactsDO != null) {
                    throw exception(ErrorCodeConstants.CUSTOMER_CONTACTS_ALREADY_BIND, phone);
                }
            }
        }


        this.indirectCustomerService.updateCustomer(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除间接客户")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    // @PreAuthorize("@ss.hasPermission('customer:indirect-customer:delete')")
    public CommonResult<Boolean> deleteIndirectCustomer(@RequestParam("id") Long id) {
        indirectCustomerService.deleteIndirectCustomer(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得间接客户")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    // @PreAuthorize("@ss.hasPermission('customer:indirect-customer:query')")
    public CommonResult<IndirectCustomerDetailRespVO> getIndirectCustomer(@RequestParam("id") Long id) {
        IndirectCustomerDO indirectCustomer = indirectCustomerService.getIndirectCustomer(id);
        return success(IndirectCustomerConvert.INSTANCE.convertDetail(indirectCustomer));
    }

    @GetMapping("/page")
    @ApiOperation("获得间接客户分页")
    // @PreAuthorize("@ss.hasPermission('customer:indirect-customer:query')")
    public CommonResult<PageResult<IndirectCustomerBackVO>> getIndirectCustomerPage(@Valid IndirectCustomerPageReqDTO pageVO, PageVO page) {
        PageResult<IndirectCustomerDO> pageResult =
                indirectCustomerService.getIndirectCustomerPageReq(pageVO,page);

        return success(IndirectCustomerConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("间接客户导出")
    // @PreAuthorize("@ss.hasPermission('customer:indirect-customer:export')")
    @OperateLog(type = EXPORT)
    public CommonResult<Boolean> exportIndirectCustomerExcel(@Valid IndirectCustomerPageReqDTO query,
                                            HttpServletResponse response) throws IOException {
        // List<IndirectCustomerDO> list = indirectCustomerService.getIndirectCustomerList(query);
        // // 导出 Excel
        // List<IndirectCustomerExcelVO> datas = IndirectCustomerConvert.INSTANCE.convertExcelList(list);
        // ExcelUtils.write(response, "间接客户.xls", "数据", IndirectCustomerExcelVO.class, datas);

        sendFileMake(query, DownloadTypeEnum.CUSTOMER_INDIRECT_EXCEL_EXPORT, "间接客户导出Excel");
        return success(true);
    }

    /**
     * 多选
     * @param exportReqVO
     * @param downloadTypeEnum
     * @param fileName
     */
    private void sendFileMake(IndirectCustomerPageReqDTO exportReqVO, DownloadTypeEnum downloadTypeEnum, String fileName) {
        FileMakeReqDTO reqDTO = new FileMakeReqDTO();
        reqDTO.setType(downloadTypeEnum.getType());
        reqDTO.setName(fileName);
        reqDTO.setFileSuffix("xlsx");
        reqDTO.setTemporaryFile(true);
        reqDTO.setUserType(2);
        reqDTO.setLang(I18nMessage.getLang());
        reqDTO.setRequestParams(JSONObject.toJSONString(exportReqVO));
        fileMakeApi.sendFileMake(reqDTO);
    }

    private void sendFileMake(IndirectCustomerExportReqVO exportReqVO, DownloadTypeEnum downloadTypeEnum, String fileName) {
        FileMakeReqDTO reqDTO = new FileMakeReqDTO();
        reqDTO.setType(downloadTypeEnum.getType());
        reqDTO.setName(fileName);
        reqDTO.setFileSuffix("xlsx");
        reqDTO.setTemporaryFile(true);
        reqDTO.setUserType(2);
        reqDTO.setLang(I18nMessage.getLang());
        reqDTO.setRequestParams(JSONObject.toJSONString(exportReqVO));
        fileMakeApi.sendFileMake(reqDTO);
    }
}
