package cn.iocoder.yudao.module.customer.controller.admin.zhongPao;

import cn.iocoder.yudao.framework.common.constant.Constant;
import cn.iocoder.yudao.framework.common.util.spring.enums.CommonStatusEnum;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.module.customer.convert.zhongPao.ZhongPaoConvert;
import cn.iocoder.yudao.module.customer.dal.dataobject.zhongPao.ZhongPaoDO;
import cn.iocoder.yudao.module.customer.service.zhongPao.ZhongPaoService;
import cn.iocoder.yudao.module.customer.vo.zhongPao.*;
import cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;

@Validated
@RestController
@Api(tags = "管理后台 - 重泡货配置")
@RequestMapping("/ecw/zhong-pao")
public class ZhongPaoController {


    @Resource
    private ZhongPaoService zhongPaoService;

    @PostMapping("/create")
    @ApiOperation("创建重泡货配置")
    @PreAuthorize("@ss.hasPermission('ecw:zhong-pao:create')")
    public CommonResult<Long> createZhongPao(@Valid @RequestBody ZhongPaoCreateReqVO createReqVO) {
        List<ZhongPaoDO> zhongPaoDOList = zhongPaoService.selectList(ZhongPaoDO::getStatus, CommonStatusEnum.ENABLE.getStatus());

        if (!CollectionUtils.isEmpty(zhongPaoDOList)) {
            boolean zhongPaoLineConfigExists = isZhongPaoLineConfigExists(createReqVO, zhongPaoDOList);
            if (zhongPaoLineConfigExists) {
                return error(ErrorCodeConstants.ZHONG_PAO_LINE_CONFIG_EXISTS);
            }
        }

        return success(zhongPaoService.createZhongPao(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新重泡货配置")
    @PreAuthorize("@ss.hasPermission('ecw:zhong-pao:update')")
    public CommonResult<Boolean> updateZhongPao(@Valid @RequestBody ZhongPaoUpdateReqVO updateReqVO) {
        List<ZhongPaoDO> zhongPaoDOList = zhongPaoService.selectList(ZhongPaoDO::getStatus, CommonStatusEnum.ENABLE.getStatus());

        if (!CollectionUtils.isEmpty(zhongPaoDOList)) {
            zhongPaoDOList = zhongPaoDOList.stream().filter(t -> !t.getId().equals(updateReqVO.getId())).collect(Collectors.toList());

            boolean zhongPaoLineConfigExists = isZhongPaoLineConfigExists(updateReqVO, zhongPaoDOList);
            if (zhongPaoLineConfigExists) {
                return error(ErrorCodeConstants.ZHONG_PAO_LINE_CONFIG_EXISTS);
            }

        }

        zhongPaoService.updateZhongPao(updateReqVO);
        return success(true);
    }

    @PutMapping("/update-status")
    @ApiOperation("更新重泡货配置状态")
    @PreAuthorize("@ss.hasPermission('ecw:zhong-pao:update')")
    public CommonResult<Boolean> updateZhongPaoStatus(@Valid @RequestBody ZhongPaoUpdateStatusReqVO updateStatusReqVO) {

        Integer status = updateStatusReqVO.getStatus();

        List<ZhongPaoDO> zhongPaoDOList = zhongPaoService.selectList(ZhongPaoDO::getStatus, CommonStatusEnum.ENABLE.getStatus());
        if (CommonStatusEnum.ENABLE.getStatus().equals(status)) {

            if (!CollectionUtils.isEmpty(zhongPaoDOList)) {
                zhongPaoDOList = zhongPaoDOList.stream().filter(t -> !t.getId().equals(updateStatusReqVO.getId())).collect(Collectors.toList());

                ZhongPaoDO zhongPaoDO = zhongPaoDOList.stream().filter(t -> t.getId().equals(updateStatusReqVO.getId())).findFirst().orElse(null);
                if (zhongPaoDO != null) {
                    ZhongPaoCreateReqVO zhongPaoCreateReqVO = ZhongPaoConvert.INSTANCE.convertToCreateReqVo(zhongPaoDO);

                    boolean zhongPaoLineConfigExists = isZhongPaoLineConfigExists(zhongPaoCreateReqVO, zhongPaoDOList);
                    if (zhongPaoLineConfigExists) {
                        return error(ErrorCodeConstants.ZHONG_PAO_LINE_CONFIG_EXISTS);
                    }

                }
            }
        }
        ZhongPaoDO zhongPaoDO = zhongPaoDOList.stream().filter(t -> t.getId().equals(updateStatusReqVO.getId())).findFirst().orElse(null);
        zhongPaoDO.setStatus(status);
        zhongPaoService.updateById(zhongPaoDO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除重泡货配置")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:zhong-pao:delete')")
    public CommonResult<Boolean> deleteZhongPao(@RequestParam("id") Long id) {
        zhongPaoService.deleteZhongPao(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得重泡货配置")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:zhong-pao:query')")
    public CommonResult<ZhongPaoBackVO> getZhongPao(@RequestParam("id") Long id) {
        ZhongPaoDO zhongPao = zhongPaoService.getZhongPao(id);
        return success(ZhongPaoConvert.INSTANCE.convert(zhongPao));
    }

    @GetMapping("/list")
    @ApiOperation("获得重泡货配置列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    @PreAuthorize("@ss.hasPermission('ecw:zhong-pao:query')")
    public CommonResult<List<ZhongPaoBackVO>> getZhongPaoList(@RequestParam("ids") Collection<Long> ids) {
        List<ZhongPaoDO> list = zhongPaoService.getZhongPaoList(ids);
        return success(ZhongPaoConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得重泡货配置分页")
    @PreAuthorize("@ss.hasPermission('ecw:zhong-pao:query')")
    public CommonResult<PageResult<ZhongPaoBackVO>> getZhongPaoPage(@Valid ZhongPaoQueryVO query, PageVO page) {
        PageResult<ZhongPaoDO> pageResult = zhongPaoService.getZhongPaoPage(query, page);
        PageResult<ZhongPaoBackVO> result = ZhongPaoConvert.INSTANCE.convertPage(pageResult);
        zhongPaoService.matchUser(result.getList());
        return success(result);
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出重泡货配置 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:zhong-pao:export')")
    @OperateLog(type = EXPORT)
    public void exportZhongPaoExcel(@Valid ZhongPaoQueryVO query,
              HttpServletResponse response) throws IOException {
        List<ZhongPaoDO> list = zhongPaoService.getZhongPaoList(query);
        // 导出 Excel
        List<ZhongPaoBackVO> datas = ZhongPaoConvert.INSTANCE.convertList(list);
        ExcelUtils.write(response, "重泡货配置.xls", "数据", ZhongPaoBackVO.class, datas);
    }


    @GetMapping("/get-customer-best-zhong-pao")
    @ApiOperation("获得客户最优惠的重泡货配置")
    @PreAuthorize("@ss.hasPermission('ecw:customer:query')")
    public CommonResult<ZhongPaoBestVO> getCustomerBestZhongPao(@Valid CustomerZhongPaoQueryVO customerZhongPaoQueryVO) {
        ZhongPaoBestVO result = zhongPaoService.getCustomerBestZhongPao(customerZhongPaoQueryVO);
        return success(result);
    }

    private boolean isZhongPaoLineConfigExists(ZhongPaoBaseVO zhongPaoBaseVO, List<ZhongPaoDO> zhongPaoDOList) {
        Long departureId = zhongPaoBaseVO.getDepartureId();
        String objectiveIds = zhongPaoBaseVO.getObjectiveIds();
        Integer type = zhongPaoBaseVO.getType();

       if (!departureId.equals(Constant.DEPARTURE_FULL) && !Constant.OBJECTIVE_FULL.equals(objectiveIds)) {
            String[] split = objectiveIds.split(",");
            for (String objectiveId : split) {
                if (zhongPaoDOList.stream().anyMatch(t -> !departureId.equals(Constant.DEPARTURE_FULL) && !Constant.OBJECTIVE_FULL.equals(objectiveIds)
                        &&  t.getDepartureId().equals(departureId) && t.getObjectiveIds().contains(objectiveId) && t.getType().equals(type))) {
                    return true;
                }
            }
        } else {
            //始发-目的 都是全部的，验证全部 是否唯一
            return zhongPaoDOList.stream().anyMatch(t -> t.getType().equals(type)
                    && t.getDepartureId().equals(Constant.DEPARTURE_FULL) && Constant.OBJECTIVE_FULL.equals(objectiveIds));
        }

        return false;
    }

}
